﻿# Copyright (C) 2007 Intel Corporation

Function Get-AMT3PDS {
<#
  .Synopsis 
    Retrieves data from the Intel Active Management Technology Third Party Data Storage
  .Description
    This CmdLet enables the user to retreive data from Intel Active Management Technology (AMT) Third Party Data Storage (3PDS) from clients that have Intel AMT firmware version 3.0 or higher.
  .Notes
    Supported AMT Firmware Versions: 3.0 or higher.

    Understanding 3PDS structure:
    Data stored within the 3PDS is stored within blocks of nonvolatile memory in a hierarchical structure. Each block must be associated to a tiered structure of Enterprise -> Vendor -> Application -> Block Name.  

    3PDS Machine UUID:
    When a block is created the application that created the block will specify a GUID to identify itself as the entity that created the block. When modifying blocks that were created by a different entity it may be necessary to specify the Machine UUID as part of the CMDLet parameter.
    
    AMT Provisioning:
      The vPro client AMT firmware must be provisioned prior to accessing AMT functionality. This CMDLet will fail if it is run against a vPro client that has not been provisioned.
        
    AMT Client Authentication:
      To invoke commands against AMT enabled clients credentials to authenticate must be specified. 
      When no credential is provided as a parameter, the script will use the local logged on Kerberos credential.
      When only the username (Kerberos or Digest) parameter is included the user will be prompted to provide the associated password.
      Credentials should be stored as a PowerShell variable then passed into the Cmdlet with the credential parameter.
      $AMTCredential = get-credential
     
    AMT Client Encryption:
      If the Intel vPro client has been configured to use TLS (a web server certificate has been issued to the Intel Management Engine) the Cmdlet must be called with a -TLS switch.

      When managing an Intel vPro client over TLS (Port 16993) it is important that the computername parameter matchs the primary subject name of the issued TLS certificate. Typically this is the fully qualified domain name (FQDN).

	   If Mutual TLS is desired, the Cmdlet must be called with -TLS switch and with a valid certificate name from the certificate store in the -CertificateName parameter.
    Status:
      Status output designates if the Cmdlet was run successfully. For failed attempts additional status may be provided.
  .Link
    http:\\vproexpert.com
    http:\\www.intel.com\vpro
    http:\\www.intel.com

  .Example
    Get-AMT3PDS -computerName:vproclient.vprodemo.com -TLS -CertificateName "myCertificate" -Operation:ListBlocks

    Retrieves all the available blocks.

    ComputerName   : vproclient.vprodemo.com
    Port           : 16993
    Operation      : listblocks
    Status         : Success
    UUID           : BEAEE8BF2C09406DAE533F16080D2A6F
    Enterprise     : Virtualization
    Vendor         : Microsoft
    Application    : System Center ConfigMgr
    BlockName      : Out Of Band Management
    NumberOfBlocks : 1

  .Example
    Get-AMT3PDS 192.168.1.100 ListBlocks -credential $AMTCredential -TLS

    ComputerName   : 192.168.1.100
    Port           : 16993
    Operation      : listblocks
    Status         : Success
    UUID           : A1DF77DC16E2469188B2E1F389E5A472
    Enterprise     : Intel
    Vendor         : Intel
    Application    : PowerShell
    BlockName      : Test
    NumberOfBlocks : 1

  .Example
    Get-AMT3PDS vproclient.vprodemo.com -Operation:Read -Username:vprodemo\ITHelpDesk -Enterprise:Intel -Vendor:Intel -Application:PowerShell -Block:Test

    Will prompt for Kerberos username password and then retrieve Data.

    ComputerName : vproclient.vprodemo.com
    Port         : 16992
    Operation    : read
    Status       : Success
    Blocks       : 1
    Data         : Test Data

  .Example
    Get-Content computers.txt | Get-AMT3PDS -TLS -Operation:ListBlocks 

    Will pull the list of amt clients from a text file and pipe them in the Get-AMT3PDS CMDLet.
  .Example
    Get-AMT3PDS-computerName:vproclient.vprodemo.com -port:16993 -Operation:read -Enterprise:"Virtualization" -Vendor:"Microsoft" -Application:"System Center ConfigMgr" -Block:"Out Of Band Management" -MachineUUID:"BEAEE8BF2C09406DAE533F16080D2A6F"

        Example to pull data from the AMT 3PDS accessible by System Center Configuration Manager
#>
[CmdletBinding()]
Param (
  [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$true, position=0, HelpMessage="Hostname, FQDN, or IP Address")] [String[]] $ComputerName,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Valid Ports are 16992 (non-TLS) or 16993 (TLS)")][ValidateSet("16992", "16993")] [String] $Port,
  [Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=1, HelpMessage="Valid Operations are Read and ListBlocks")][ValidateSet("Read","ListBlocks")] [String] $Operation,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=2, HelpMessage="Name of the Enterprise")] [string] $Enterprise,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=3, HelpMessage="Name of the Vendor")] [string] $Vendor,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=4, HelpMessage="Name of the Application")] [string] $Application,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=5, HelpMessage="Name of the Block")] [string] $Block,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=6, HelpMessage="Machine UUID")] [string] $MachineUUID,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Use TLS (Port 16993)")] [switch] $TLS,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Accept self-signed certificate for TLS connection (skip any certificate checks.)")] [switch] $AcceptSelfSignedCert,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Name of certificate. (Use for mutual TLS)")] [string] $CertificateName,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest of Kerberos User")] [string] $Username,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest of Kerberos Password")] [string] $Password,
  [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=7, HelpMessage="PS Credential")] [System.Management.Automation.PSCredential] $Credential
) 


PROCESS {

  function CreateApplication($Connection, $VendorName, $AppName)
  {
    $AdminService_EPR = $Connection.NewReference("AMT_ThirdPartyDataStorageAdministrationService")
    $InputObj = $AdminService_EPR.CreateMethodInput("AddStorageFpaclEntry")
    $InputObj.SetProperty("AttrType","1")
    $InputObj.SetProperty("ApplicationName",$AppName)
    $InputObj.SetProperty("VendorName",$VendorName)
    $InputObj.SetProperty("IsPartner", "true")
    $InputObj.SetProperty("TotalAllocationSize","49152")
    
    $Output = $AdminService_EPR.InvokeMethod($InputObj)
    if($Output.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    else
    {
        return $true
    }
  }
  
  function CreateEnterprise($Connection, $enterpriseName, [ref]$Handle)
  {
    $AdminService_EPR = $Connection.NewReference("AMT_ThirdPartyDataStorageAdministrationService")
    $InputObj = $AdminService_EPR.CreateMethodInput("AddStorageEaclEntry")
    $InputObj.SetProperty("EnterpriseName",$enterpriseName)
    $Output = $AdminService_EPR.InvokeMethod($InputObj)    
    
    if($Output.GetProperty("ReturnValue").ToString() -ne "0")
    {
        # Error
        return $false  
    }
    $Handle.Value = $Output.GetProperty("Handle").ToString()
    return $true
  }
  
  function EnumerateEnterprise($Connection)
  {
    $RetVal = @()
    
    $AdminService_EPR = $Connection.NewReference("AMT_ThirdPartyDataStorageAdministrationService")
    $InputObj = $AdminService_EPR.CreateMethodInput("EnumerateStorageEaclEntries")
    $Output = $AdminService_EPR.InvokeMethod($InputObj) 
    $Handles = $Output.GetProperty("Handles")
    for($i=0;$i -lt $Handles.Count;++$i)
    {
        $InputObj = $AdminService_EPR.CreateMethodInput("GetStorageEaclEntry")
        $InputObj.SetProperty("Handle",$Handles.Item($i).ToString())
        $OutObj = $AdminService_EPR.InvokeMethod($InputObj)
        $RetVal += @($OutObj.GetProperty("EnterpriseName").ToString())
    }
    return ,$RetVal
  }
  function DeallocateBlock($Connection, $sessionHandle, $blockHandle)
  {
    $DataStorageService_EPR = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObject = $DataStorageService_EPR.CreateMethodInput("DeallocateBlock")
    $InputObject.SetProperty("SessionHandle",$sessionHandle)
    $InputObject.SetProperty("BlockHandle",$blockHandle)
    $OutputObject = $DataStorageService_EPR.InvokeMethod($InputObject)
    if($OutputObject.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    else
    {
        return $true
    }
  }
  function AllocateBlock($Connection, $sessionHandle, $blockName, $blockHidden, [ref]$BlockHandle)
  {
    $DataStorageService_EPR = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObject = $DataStorageService_EPR.CreateMethodInput("AllocateBlock")
    $InputObject.SetProperty("SessionHandle",$sessionHandle)
    $InputObject.SetProperty("BytesRequested","4096")
    if($blockHidden)
    {
        $InputObject.SetProperty("BlockHidden","1")
    }
    else
    {
        $InputObject.SetProperty("BlockHidden","0")
    }
    $InputObject.SetProperty("BlockName",$blockName)
    $OutputObject = $DataStorageService_EPR.InvokeMethod($InputObject)
    
    if($OutputObject.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return($false)
    }
    else
    {
        $BlockHandle.Value = $OutputObject.GetProperty("BlockHandle").ToString()
        return($true)
    }
  }

  function GetUUID()
  {
    $compname = hostname
    $uuid = (Get-WmiObject Win32_ComputerSystemProduct -ComputerName $compname).UUID
    
    $uuid = $uuid.Replace("-","")
     
    return $uuid
  }
  
  function RegisterApplication($Connection, $uuid, $vendorName, $appName, $entName, [ref]$SessionHandle)
  {
        
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("RegisterApplication")
    $InputObj.SetProperty("CallerUUID", $uuid)
    $InputObj.SetProperty("VendorName", $vendorName)
    $InputObj.SetProperty("ApplicationName", $appName)
    $InputObj.SetProperty("EnterpriseName", $entName)
    $OutObj = $objRef.InvokeMethod($InputObj)
    
    if($OutObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return($false)
    }
    else
    {
       $SessionHandle.Value = $OutObj.GetProperty('SessionHandle').ToString()
       return($true) 
    }
  }
  
  function GetApplicationHandle($Connection, $SessionHandle, [ref]$ApplicationHandle)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("GetCurrentApplicationHandle")
    $InputObj.SetProperty("SessionHandle", "$SessionHandle")
    $OutObj = $objRef.InvokeMethod($InputObj)
    
    if($OutObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    else
    {
        $ApplicationHandle.Value = $OutObj.GetProperty('ApplicationHandle').ToString()
        return $true 
    }    
  }
  
  function GetAllocatedBlocks($Connection, $SessionHandle, $ApplicationHandle)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("GetAllocatedBlocks")
    $InputObj.SetProperty("SessionHandle", "$SessionHandle")
    $InputObj.SetProperty("BlockOwnerApplication", "$ApplicationHandle")
    $OutObj = $objRef.InvokeMethod($InputObj)
    
    $BlockHandles = $OutObj.GetProperty('BlockHandles')
    $blocks = @()
    
    if($BlockHandles.IsArray)
    {
        for($i=0;$i -lt $BlockHandles.Count;++$i)
        {
           $blocks = $blocks + @($BlockHandles.Item($i).ToString()) 
        }
    }
    else 
    {
        if(!$BlockHandles.IsNull)
        {
            $blocks = @($BlockHandles.ToString())
        }
        $blocks = ,$blocks
    }
    
    Return $blocks
  }
  
  function GetBlockName($Connection, $SessionHandle, $BlockHandle)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("GetBlockAttributes")
    $InputObj.SetProperty("SessionHandle", "$SessionHandle")
    $InputObj.SetProperty("BlockHandle", "$BlockHandle")
    $OutObj = $objRef.InvokeMethod($InputObj)
    
    $BlockName = $OutObj.GetProperty('BlockName').ToString()
    $BlockName
  }
  
  function ReadBlockRaw($Connection, $SessionHandle, $BlockHandle, $ReadOffset, $ReadLength, [ref]$Bytes)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("ReadBlock")
    $InputObj.SetProperty("SessionHandle", "$SessionHandle")
    $InputObj.SetProperty("BlockHandle", "$BlockHandle")
    $InputObj.SetProperty("ByteOffset", $ReadOffset)
    $InputObj.SetProperty("ByteCount", $ReadLength)
    $OutObj = $objRef.InvokeMethod($InputObj)
    
    if($OutObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    else
    {    
        $Bytes.Value = $OutObj.GetProperty('Data').ToString()
        return $true   
    }
  }
  
  function CalculateBase64Length($InputBufferSize)
  {
    $input_size = $InputBufferSize
    $code_size    = (($input_size * 4) / 3)
    if($input_size % 3 -ne 0)
    {
        $padding_size = 3 - ($input_size % 3)
    }
    else
    {
        $padding_size = 0
    }    
    $total_size   = $code_size + $padding_size 
    return $total_size
  }
  function CalculateMaxProcessingSize($InputBufferSize, $OutputBufferSizeContraint)
  {
    do
    {
        $OutSize = CalculateBase64Length $InputBufferSize
        --$InputBufferSize
    }while($OutSize -gt $OutputBufferSizeContraint)
    return $InputBufferSize + 1
  }
  
  function WriteBlockRaw($Connection, $sessionHandle, $blockHandle, $RawBytes, $RawBytesOffset, $RawBytesLength, $start)
  {
    $encodedData = [System.Convert]::ToBase64String($RawBytes, $RawBytesOffset, $RawBytesLength)
  
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("WriteBlock")
    $InputObj.SetProperty("SessionHandle",$sessionHandle)
    $InputObj.SetProperty("BlockHandle",$blockHandle)
    $InputObj.SetProperty("ByteOffset",$start.ToString())
    $InputObj.SetProperty("Data",$encodedData)
    
    $OutputObj = $objRef.InvokeMethod($InputObj)
    if($OutputObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    else
    {
        return $true
    } 
  }
  function WriteBlock($Connection, $sessionHandle, $blockHandle, $data, $start)
  {
    $bytes = [System.Text.Encoding]::UTF8.GetBytes($data)
    $Success = WriteBlockRaw $Connection $sessionHandle $blockHandle $bytes 0 $bytes.Length $start
    return $Success
  }  
  
  function WriteBlock_Smart($Connection, $SessionHandle, $BlockName, $CurrentBlockHandle, $BlockHidden, $data)
  {
   
    # Read data that is already written in this block   
    $ReadData = ""
    $Success = ReadBlock $Connection $SessionHandle $CurrentBlockHandle ([ref]$ReadData)
    if(!$Success)
    {
        return $false
    }
      
    $WriteString = $ReadData + $data 
    $rawBytes = [System.Text.Encoding]::UTF8.GetBytes($WriteString)
    $Position = 0
    
    $Success = $true
    do
    {
        $WriteLength = CalculateMaxProcessingSize ($rawBytes.Length - $Position) (4095)
        $encodedData = [System.Convert]::ToBase64String($rawBytes, $Position, $WriteLength)
        
        $Success = WriteBlockRaw $Connection $SessionHandle $CurrentBlockHandle $rawBytes $Position $WriteLength 0
        if(!$Success)
        {
            break
        }
        $Position += $WriteLength
        if($Position -lt $rawBytes.Length)
        {
            # Still have more data to write, so we need to allocate more blocks
            $Success = AllocateBlock $Connection $SessionHandle $BlockName $BlockHidden ([ref]$CurrentBlockHandle)
            if(!$Success)
            {
                break
            }
        }
    } while($Position -lt $rawBytes.Length)
    
    return $Success
  }
  
  function ReadBlock($Connection, $SessionHandle, $BlockHandle, [ref]$OutputData)
  {
    $Bytes = ""
    $Success = ReadBlockRaw $Connection $SessionHandle $BlockHandle "0" "4095" ([ref]$Bytes)
    if(!$Success)
    {
        return $false
    }

        
    $DecodedData = [System.Convert]::FromBase64String($Bytes)
    
    foreach ($byte in $DecodedData)
    {
        if ($byte -ne 255)
        {
            $tempArray = $tempArray + @($byte)
        }
    }
    
    if ($tempArray.length -gt 0)
    {
        $data = [System.Text.Encoding]::UTF8.GetString($tempArray)
    }
    else
    {
        $data = ""   
    }
    $OutputData.Value = $data
    return $true
  }
  
  function UnregisterApplication($Connection, $sessionHandle)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageService")
    $InputObj = $objref.CreateMethodInput("UnregisterApplication")
    $InputObj.SetProperty("SessionHandle", "$SessionHandle")
    $OutObj = $objRef.InvokeMethod($InputObj)
  }
  
  function AdminGetRegisteredApplications([ref]$ApplicationHandles)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageAdministrationService")
    $InputObj = $objref.CreateMethodInput("AdminGetRegisteredApplications")
    $OutObj = $objRef.InvokeMethod($InputObj)
    if($OutObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    
    $en = $OutObj.GetProperty('ApplicationHandles')
    $ApplicationHandles.Value = @()
    if($en.IsArray)
    {
        For($i=0;$i -lt $en.Count;++$i)
        {
            $ApplicationHandles.Value += @($en.Item($i).ToString())
        }
    }
    else
    {
        $ApplicationHandles.Value += @($en.ToString())
    }
    return $true
  }
  
  function AdminGetApplicationAttributes($appHandle, [ref]$uuid, [ref]$VendorName, [ref]$ApplicationName, [ref]$EnterpriseName)
  {
    $objRef = $Connection.NewReference("AMT_ThirdPartyDataStorageAdministrationService")
    $InputObj = $objref.CreateMethodInput("AdminGetApplicationAttributes")
    $InputObj.SetProperty("Handle", "$appHandle")
    
    $OutObj = $objRef.InvokeMethod($InputObj)
    if($OutObj.GetProperty("ReturnValue").ToString() -ne "0")
    {
        return $false
    }
    
    $uuid.Value = $OutObj.GetProperty('UUID').ToString()
    $VendorName.Value = $OutObj.GetProperty('VendorName').ToString()
    $ApplicationName.Value = $OutObj.GetProperty('ApplicationName').ToString()
    $EnterpriseName.Value = $OutObj.GetProperty('EnterpriseName').ToString()
    return $true
  }
	
	$Results = @()
	$Obj = new-object psobject
	try
	{
		#create a connection object
		$Connection = New-Object Intel.Management.Wsman.WsmanConnection 

		if ($Credential.username.Length -gt 0) {
			$Connection.SetCredentials($Credential.Username, $Credential.Password)  
		} elseif ($Username.length -gt 0) {
			if ($Password.length -gt 0) {
				$Connection.SetCredentials($Username, $Password)  
			} else {
				$Cred = Get-Credential $Username
				$Connection.SetCredentials($Cred.Username, $Cred.Password)
			}
		}
		
		if ($Port.Length -lt 1) {
			if ($TLS.IsPresent) {
			  $Port = 16993;
			}
			else {
			  $Port = 16992;
			}
		}
		
		if($CertificateName.Length -gt 0)
		{
			$Connection.Options.SetClientCertificateByCertificateName($CertificateName)
		}
		
		if($AcceptSelfSignedCert.IsPresent)
		{
			$Connection.Options.AcceptSelfSignedCertificate=$true;			
		}


#----------------- Code below this line is specific to each script.


		ForEach ($Comp in $ComputerName) 
		{
			$skip = $false
			
			$Obj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
			$Obj | Add-Member -MemberType noteproperty -Name Port -value $Port
			$Obj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
			
			try {
				#Attempt Connection with Client
				$Connection.SetHost($Comp, $Port)
		  #---------------------------------
		  
		  switch ($operation)
		  {
			"Read" 
			{
				if($MachineUUID -eq "")
				{
					$uuid = GetUUID
				}
				else
				{
					$uuid = $MachineUUID
				}
				$SessionHandle = ""
				$ApplicationHandle = ""
				$Success = RegisterApplication $Connection $uuid $Vendor $Application $Enterprise ([ref]$SessionHandle)
				if(!$Success)
				{
					$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "RegisterApplicationFailed"
					continue  
				}
				$Success = GetApplicationHandle $Connection $SessionHandle ([ref]$ApplicationHandle)
				if(!$Success)
				{
					$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "GetApplicationHandleFailed"
					continue  
				}

				$BlockHandles = GetAllocatedBlocks $Connection $SessionHandle $ApplicationHandle 
				$ReadBuffer = New-Object System.Text.StringBuilder           
				$NumberOfBlocks = 0
				foreach ($tempBlockHandle in $BlockHandles)
				{      
					$tempBlockName = GetBlockName $Connection $SessionHandle $tempBlockHandle
					if ($tempBlockName -eq $Block)
					{ 
						# Read data that is already written in this block   
						$ReadData = ""
						$Success = ReadBlock $Connection $SessionHandle $tempBlockHandle ([ref]$ReadData)
						if(!$Success)
						{
							$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "ReadFailed"
							break;
						}
						[void] $ReadBuffer.Append($ReadData)
						++$NumberOfBlocks
					}
				}
				$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Successful"
				$Obj | Add-Member -MemberType noteproperty -Name "Blocks" -value $NumberOfBlocks.ToString()
				$Obj | Add-Member -MemberType noteproperty -Name "Data" -value $ReadBuffer.ToString()

				UnregisterApplication $Connection $SessionHandle
			}
			"ListBlocks" 
			{
			  $AppHandles = ""
			  $Success = AdminGetRegisteredApplications ([ref]$AppHandles)
			  if(!$Success)
			  {
				$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "AdminGetRegisteredApplicationsFailed"
				continue  
			  }
			  
			  if($AppHandles.Count -eq 0)
			  {
				# No Registered Applications, so let's just enumerate Enterprises
				$EnterpriseNames = EnumerateEnterprise $Connection
				foreach($EnterpriseName in $EnterpriseNames)
				{
					$TObj = new-object psobject
					$TObj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
					$TObj | Add-Member -MemberType noteproperty -Name Port -value $Port
					$TObj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
					$TObj | Add-Member -MemberType noteproperty -Name Status -value "Successful"
					$TObj | Add-Member -MemberType noteproperty -Name Enterprise -value $EnterpriseName
					$skip = $true
						
					Write-Output $TObj
				}
			  }
			  
			  foreach ($tempAppHandle in $AppHandles)
			  {
				$uuid = ""
				$VendorName = ""
				$ApplicationName = ""
				$EnterpriseName = ""
				$sessionHandle = ""
				
				$Success = AdminGetApplicationAttributes $tempAppHandle ([ref]$uuid) ([ref]$VendorName) ([ref]$ApplicationName) ([ref]$EnterpriseName)
				if(!$Success)
				{
					$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "AdminGetApplicationAttributesFailed"
					continue  
				}
				$uuid = [System.BitConverter]::ToString([System.Convert]::FromBase64String($uuid)).Replace("-","")
				
				$Success = RegisterApplication $Connection $uuid $vendorName $applicationName $enterpriseName ([ref]$sessionHandle)
				if(!$Success)
				{
					$Obj | Add-Member -MemberType noteproperty -Name Status -value "RegisterApplicationFailed"
					continue
				}
				if($Enterprise -ne "" -and $Enterprise -ne $EnterpriseName)
				{
					continue
				}
				if($Application -ne "" -and $ApplicationName -ne $Application)
				{
					continue
				}
				if($Vendor -ne "" -and $VendorName -ne $Vendor)
				{
					continue
				}
				
				$blockHandles = GetAllocatedBlocks $Connection $sessionHandle $tempAppHandle
				if($blockHandles.Count -eq 0)
				{
					$TObj = new-object psobject
					$TObj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
					$TObj | Add-Member -MemberType noteproperty -Name Port -value $Port
					$TObj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
					$TObj | Add-Member -MemberType noteproperty -Name Status -value "Successful"
					$TObj | Add-Member -MemberType noteproperty -Name UUID -value $uuid
					$TObj | Add-Member -MemberType noteproperty -Name Enterprise -value $EnterpriseName
					$TObj | Add-Member -MemberType noteproperty -Name Vendor -value $vendorName
					$TObj | Add-Member -MemberType noteproperty -Name Application -value $ApplicationName
					$skip = $true
						
					Write-Output $TObj
				}
				else
				{
					$BlockTable = New-Object System.Collections.Hashtable
					for($i=0;$i -lt $blockHandles.Count;++$i)
					{
						$tempBlockName = GetBlockName $Connection $sessionHandle $blockHandles[$i]
						$Counter = 0
						if($BlockTable.ContainsKey($tempBlockName))
						{
							$Counter = $tempBlockName[$tempBlockName]
						}

						++$Counter
						$BlockTable[$tempBlockName] = $Counter              
					}
		
					$en = $BlockTable.GetEnumerator()

					while($en.MoveNext())
					{
						$TObj = new-object psobject
						$TObj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
						$TObj | Add-Member -MemberType noteproperty -Name Port -value $Port
						$TObj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
						$TObj | Add-Member -MemberType noteproperty -Name Status -value "Successful"
						$TObj | Add-Member -MemberType noteproperty -Name UUID -value $uuid
						$TObj | Add-Member -MemberType noteproperty -Name Enterprise -value $EnterpriseName
						$TObj | Add-Member -MemberType noteproperty -Name Vendor -value $vendorName
						$TObj | Add-Member -MemberType noteproperty -Name Application -value $ApplicationName
						$TObj | Add-Member -MemberType noteproperty -Name BlockName -value $en.Key.ToString()
						$TObj | Add-Member -MemberType noteproperty -Name NumberOfBlocks -value $en.Value.ToString()
						$skip = $true
						
						Write-Output $TObj
					}
				}  
			  }  
			  if(!$skip)
			  {
				$Obj | Add-Member -MemberType noteproperty -Name Status -value "No Entries"
			  }       
			}
		  }
		  
		  #------------------------------------


			} 
			catch {
				#Add Member to Object noting failed attempt 
				$Obj | Add-Member -MemberType noteproperty -Name Status -value "ExceptionThrown"
				$Obj | Add-Member -MemberType noteproperty -Name Reason -value $_.Exception.Message
			}

			#Append to Result 
			if(!$skip)
			{
			  $Results += $Obj
			}
		}
		
	}
	catch
	{
		$Obj | Add-Member -MemberType noteproperty -Name Status -value "ExceptionThrown"
		$Obj | Add-Member -MemberType noteproperty -Name Reason -value $_.Exception.Message
		$Results += $Obj
	}
	
	# Set back to false for the next cmdlet, if PS is still open
	$Connection.Options.AcceptSelfSignedCertificate=$false;
	
	#Write out Results
	Write-Output $Results
}
}
# SIG # Begin signature block
# MIIogwYJKoZIhvcNAQcCoIIodDCCKHACAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCCiKbF5CfcaGEnU
# QQfwzpCRepAWUY204svNu/a9yRpnjqCCEfIwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYcMIIEBKADAgECAhAz1wio
# kUBTGeKlu9M5ua1uMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLjAs
# BgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBFViBSMzYwggGi
# MA0GCSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQC70f4et0JbePWQp64sg/GNIdMw
# hoV739PN2RZLrIXFuwHP4owoEXIEdiyBxasSekBKxRDogRQ5G19PB/YwMDB/NSXl
# wHM9QAmU6Kj46zkLVdW2DIseJ/jePiLBv+9l7nPuZd0o3bsffZsyf7eZVReqskmo
# PBBqOsMhspmoQ9c7gqgZYbU+alpduLyeE9AKnvVbj2k4aOqlH1vKI+4L7bzQHkND
# brBTjMJzKkQxbr6PuMYC9ruCBBV5DFIg6JgncWHvL+T4AvszWbX0w1Xn3/YIIq62
# 0QlZ7AGfc4m3Q0/V8tm9VlkJ3bcX9sR0gLqHRqwG29sEDdVOuu6MCTQZlRvmcBME
# Jd+PuNeEM4xspgzraLqVT3xE6NRpjSV5wyHxNXf4T7YSVZXQVugYAtXueciGoWnx
# G06UE2oHYvDQa5mll1CeHDOhHu5hiwVoHI717iaQg9b+cYWnmvINFD42tRKtd3V6
# zOdGNmqQU8vGlHHeBzoh+dYyZ+CcblSGoGSgg8sCAwEAAaOCAWMwggFfMB8GA1Ud
# IwQYMBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBSBMpJBKyjNRsjE
# osYqORLsSKk/FDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADAT
# BgNVHSUEDDAKBggrBgEFBQcDAzAaBgNVHSAEEzARMAYGBFUdIAAwBwYFZ4EMAQMw
# SwYDVR0fBEQwQjBAoD6gPIY6aHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UHVibGljQ29kZVNpZ25pbmdSb290UjQ2LmNybDB7BggrBgEFBQcBAQRvMG0wRgYI
# KwYBBQUHMAKGOmh0dHA6Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0Nv
# ZGVTaWduaW5nUm9vdFI0Ni5wN2MwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMA0GCSqGSIb3DQEBDAUAA4ICAQBfNqz7+fZyWhS38Asd3tj9lwHS
# /QHumS2G6Pa38Dn/1oFKWqdCSgotFZ3mlP3FaUqy10vxFhJM9r6QZmWLLXTUqwj3
# ahEDCHd8vmnhsNufJIkD1t5cpOCy1rTP4zjVuW3MJ9bOZBHoEHJ20/ng6SyJ6UnT
# s5eWBgrh9grIQZqRXYHYNneYyoBBl6j4kT9jn6rNVFRLgOr1F2bTlHH9nv1HMePp
# GoYd074g0j+xUl+yk72MlQmYco+VAfSYQ6VK+xQmqp02v3Kw/Ny9hA3s7TSoXpUr
# OBZjBXXZ9jEuFWvilLIq0nQ1tZiao/74Ky+2F0snbFrmuXZe2obdq2TWauqDGIgb
# MYL1iLOUJcAhLwhpAuNMu0wqETDrgXkG4UGVKtQg9guT5Hx2DJ0dJmtfhAH2KpnN
# r97H8OQYok6bLyoMZqaSdSa+2UA1E2+upjcaeuitHFFjBypWBmztfhj24+xkc6Zt
# CDaLrw+ZrnVrFyvCTWrDUUZBVumPwo3/E3Gb2u2e05+r5UWmEsUUWlJBl6MGAAjF
# 5hzqJ4I8O9vmRsTvLQA1E802fZ3lqicIBczOwDYOSxlP0GOabb/FKVMxItt1UHeG
# 0PL4au5rBhs+hSMrl8h+eplBDN1Yfw6owxI9OjWb4J0sjBeBVESoeh2YnZZ/WVim
# VGX/UUIL+Efrz/jlvzCCBlswggTDoAMCAQICEHAQMvP0AIEdrcUef3h12LwwDQYJ
# KoZIhvcNAQELBQAwVzELMAkGA1UEBhMCR0IxGDAWBgNVBAoTD1NlY3RpZ28gTGlt
# aXRlZDEuMCwGA1UEAxMlU2VjdGlnbyBQdWJsaWMgQ29kZSBTaWduaW5nIENBIEVW
# IFIzNjAeFw0yMzA0MDQwMDAwMDBaFw0yNDA0MDMyMzU5NTlaMIG7MRAwDgYDVQQF
# EwcyMTg5MDc0MRMwEQYLKwYBBAGCNzwCAQMTAlVTMRkwFwYLKwYBBAGCNzwCAQIT
# CERlbGF3YXJlMR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjELMAkGA1UE
# BhMCVVMxEzARBgNVBAgMCkNhbGlmb3JuaWExGjAYBgNVBAoMEUludGVsIENvcnBv
# cmF0aW9uMRowGAYDVQQDDBFJbnRlbCBDb3Jwb3JhdGlvbjCCAaIwDQYJKoZIhvcN
# AQEBBQADggGPADCCAYoCggGBANO3BjDeeK28E+zFxToyp77/6fB+Mp6913C+9t1U
# JguD/sBVnxljfvQScWzRrvF0chAOz5rjlnvUTXUFsgvJ4XFIZQtG/Rfyb4qvQbFE
# vdoS1kFlMAtXnMuuDSMbncPju4R/5gLfMzHFLRpHOoUmeRFlwEkyAYVwjDvH+OcR
# WW2Vm72ggaVeqgW5kIx3R4VQSXgJ3ToPP/49/9YjEwhM0yLccTPJKVB/6unAJaY6
# /j15gi4owbo6K88625JlZTy7drEhZvvI9vlStkPUOxbHQmU72QG7bF1ofao/4lTH
# pE6RAXTppoWlu24kRhLcXaIIAitMDqs9E0SFdV2qcAxc7d38v0l2sds/sy2EWMU9
# RZAjJD4fH/Bm0hJVm5937cq/sf20lucUQnu96+5zVHXSIuD65dkQ/6aoGxwZty49
# rdTFY494ri++yfw6NWrbQkIoc2c1HG0jHRAV6m3dgzwY0104g/BpZkLogT4EEEO5
# XRm6k1PSMxoM194ytHU+pJ2B2QIDAQABo4IBvDCCAbgwHwYDVR0jBBgwFoAUgTKS
# QSsozUbIxKLGKjkS7EipPxQwHQYDVR0OBBYEFJLGD4aF5QaeHQosntNMcCm3IDAZ
# MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMEkGA1UdIARCMEAwNQYMKwYBBAGyMQECAQYBMCUwIwYIKwYBBQUHAgEWF2h0
# dHBzOi8vc2VjdGlnby5jb20vQ1BTMAcGBWeBDAEDMEsGA1UdHwREMEIwQKA+oDyG
# Omh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVTaWduaW5n
# Q0FFVlIzNi5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsGAQUFBzAChjpodHRwOi8v
# Y3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmluZ0NBRVZSMzYu
# Y3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTAuBgNVHREE
# JzAloCMGCCsGAQUFBwgDoBcwFQwTVVMtREVMQVdBUkUtMjE4OTA3NDANBgkqhkiG
# 9w0BAQsFAAOCAYEAttRv5ICWB9YKIXk6xtQ385RQks/95gKJP+T62x8e2rHfQXY4
# GQChplzrceWnolz+TBJF1Dt8HYLUjpj0tgQHcU1mT54Lz4vt8eRHfldjRUC3I+Sy
# FxfWncIWYru27DPKAUEVmCHNF6++myfE7FLTFsL2ogZhZF1sUIBAvDd7d3j0wtz4
# N4d9MhjIfncOOqTOctwEbLtSTX9xIOry4ZgsDVs/QWiqyMyUaG649bcWWXDWxIq+
# 6jtY4WJn4KQUhPVzP53e45Y7Leb+69GCSkfQ5cEZlSOZq2PeFRqP3PwVQ946hbm8
# saXEl3iaZ8s5YY2JorFvstkqYXiQ9O55wAZ4fTeyhWjVNpkqDiIHpO+UYREJ5AHG
# nsZEJJkx8E/Y+zX7YuDMJmV7+WLI1CvIyCLoNhUMupfReykiQKD1P+CCe6UzSOey
# mSrh0X/XpOcDz/+sSgUIziD5HUuMYp0czD4Up/FFrl3qX3te9N4hQt74Ifh35DLR
# g/DdRlqPI70u7jl5MYIV5zCCFeMCAQEwazBXMQswCQYDVQQGEwJHQjEYMBYGA1UE
# ChMPU2VjdGlnbyBMaW1pdGVkMS4wLAYDVQQDEyVTZWN0aWdvIFB1YmxpYyBDb2Rl
# IFNpZ25pbmcgQ0EgRVYgUjM2AhBwEDLz9ACBHa3FHn94ddi8MA0GCWCGSAFlAwQC
# AQUAoHwwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIE
# IEqdpvPDpMo52bHy2RYVa29dNLv6qx2qh5+zINRZkMd4MA0GCSqGSIb3DQEBAQUA
# BIIBgEbDmOLZ5DDAvRNU9V6FtWnDkFsNBqJJlI/sYuOWrYC3teNzehbvuTaEKfdl
# F8Er9GnbAvTYNYj3/3+TDVkTFFgoNqrgud/EpPDQYgYlOI3GgKj2CuBgwdF1JWE8
# 2davukIoBvQmwlFC+zwe94mlAaD1hOpxlZVnHXFktqYZRp+c71AAN45UjY8hpICF
# wv0vh+tj6o/i6lFqE7hELzlaenacpRsfGmKlCC1BBoyYkloERhWSFoHdhQsNMVms
# MtAiwEKOU7pxqofsheeW7Zyyg15jNo7tmWfhzSXy6ezi4oNO4xin0Wm/Owh40DQU
# inycB+5lxkPxPJAZ1KozxfBHkPAOTqwnz18WrCgfEtWIEob5uSm1lE5Wh19NfQbU
# QdcjWfF8MLJ+KPDa3ky4gXFSd4M3NJWfCFKGiHsu/wICLRBzUwRJVnvbdXAswCP3
# d6h9fUlsAPJydXoFYXkvigxt/MIjqHSOGJPFSCxzHaO8yNz6gv14mSej5UpeVbrB
# TeV8ZqGCE08wghNLBgorBgEEAYI3AwMBMYITOzCCEzcGCSqGSIb3DQEHAqCCEygw
# ghMkAgEDMQ8wDQYJYIZIAWUDBAICBQAwgfAGCyqGSIb3DQEJEAEEoIHgBIHdMIHa
# AgEBBgorBgEEAbIxAgEBMDEwDQYJYIZIAWUDBAIBBQAEIKyVKMJQ4RdlgNN7JnlE
# fakRmRdQMQ9T9qtJcmHP3yZ2AhUA3O18/CTOOEDzjqZxH+27TfS7SbAYDzIwMjMw
# ODEzMDczMjIxWqBupGwwajELMAkGA1UEBhMCR0IxEzARBgNVBAgTCk1hbmNoZXN0
# ZXIxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDEsMCoGA1UEAwwjU2VjdGlnbyBS
# U0EgVGltZSBTdGFtcGluZyBTaWduZXIgIzSggg3pMIIG9TCCBN2gAwIBAgIQOUwl
# 4XygbSeoZeI72R0i1DANBgkqhkiG9w0BAQwFADB9MQswCQYDVQQGEwJHQjEbMBkG
# A1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUg
# U3RhbXBpbmcgQ0EwHhcNMjMwNTAzMDAwMDAwWhcNMzQwODAyMjM1OTU5WjBqMQsw
# CQYDVQQGEwJHQjETMBEGA1UECBMKTWFuY2hlc3RlcjEYMBYGA1UEChMPU2VjdGln
# byBMaW1pdGVkMSwwKgYDVQQDDCNTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIFNp
# Z25lciAjNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKSTKFJLzyeH
# dqQpHJk4wOcO1NEc7GjLAWTkis13sHFlgryf/Iu7u5WY+yURjlqICWYRFFiyuiJb
# 5vYy8V0twHqiDuDgVmTtoeWBIHIgZEFsx8MI+vN9Xe8hmsJ+1yzDuhGYHvzTIAhC
# s1+/f4hYMqsws9iMepZKGRNcrPznq+kcFi6wsDiVSs+FUKtnAyWhuzjpD2+pWpqR
# KBM1uR/zPeEkyGuxmegN77tN5T2MVAOR0Pwtz1UzOHoJHAfRIuBjhqe+/dKDcxIU
# m5pMCUa9NLzhS1B7cuBb/Rm7HzxqGXtuuy1EKr48TMysigSTxleGoHM2K4GX+hub
# foiH2FJ5if5udzfXu1Cf+hglTxPyXnypsSBaKaujQod34PRMAkjdWKVTpqOg7RmW
# ZRUpxe0zMCXmloOBmvZgZpBYB4DNQnWs+7SR0MXdAUBqtqgQ7vaNereeda/TpUsY
# oQyfV7BeJUeRdM11EtGcb+ReDZvsdSbu/tP1ki9ShejaRFEqoswAyodmQ6MbAO+i
# tZadYq0nC/IbSsnDlEI3iCCEqIeuw7ojcnv4VO/4ayewhfWnQ4XYKzl021p3AtGk
# +vXNnD3MH65R0Hts2B0tEUJTcXTC5TWqLVIS2SXP8NPQkUMS1zJ9mGzjd0HI/x8k
# VO9urcY+VXvxXIc6ZPFgSwVP77kv7AkTAgMBAAGjggGCMIIBfjAfBgNVHSMEGDAW
# gBQaofhhGSAPw0F3RSiO0TVfBhIEVTAdBgNVHQ4EFgQUAw8xyJEqk71j89FdTaQ0
# D9KVARgwDgYDVR0PAQH/BAQDAgbAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwSgYDVR0gBEMwQTA1BgwrBgEEAbIxAQIBAwgwJTAjBggrBgEF
# BQcCARYXaHR0cHM6Ly9zZWN0aWdvLmNvbS9DUFMwCAYGZ4EMAQQCMEQGA1UdHwQ9
# MDswOaA3oDWGM2h0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVT
# dGFtcGluZ0NBLmNybDB0BggrBgEFBQcBAQRoMGYwPwYIKwYBBQUHMAKGM2h0dHA6
# Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVTdGFtcGluZ0NBLmNydDAj
# BggrBgEFBQcwAYYXaHR0cDovL29jc3Auc2VjdGlnby5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAEybZVj64HnP7xXDMm3eM5Hrd1ji673LSjx13n6UbcMixwSV32VpYRMM
# 9gye9YkgXsGHxwMkysel8Cbf+PgxZQ3g621RV6aMhFIIRhwqwt7y2opF87739i7E
# fu347Wi/elZI6WHlmjl3vL66kWSIdf9dhRY0J9Ipy//tLdr/vpMM7G2iDczD8W69
# IZEaIwBSrZfUYngqhHmo1z2sIY9wwyR5OpfxDaOjW1PYqwC6WPs1gE9fKHFsGV7C
# g3KQruDG2PKZ++q0kmV8B3w1RB2tWBhrYvvebMQKqWzTIUZw3C+NdUwjwkHQepY7
# w0vdzZImdHZcN6CaJJ5OX07Tjw/lE09ZRGVLQ2TPSPhnZ7lNv8wNsTow0KE9SK16
# ZeTs3+AB8LMqSjmswaT5qX010DJAoLEZKhghssh9BXEaSyc2quCYHIN158d+S4RD
# zUP7kJd2KhKsQMFwW5kKQPqAbZRhe8huuchnZyRcUI0BIN4H9wHU+C4RzZ2D5fjK
# JRxEPSflsIZHKgsbhHZ9e2hPjbf3E7TtoC3ucw/ZELqdmSx813UfjxDElOZ+JOWV
# SoiMJ9aFZh35rmR2kehI/shVCu0pwx/eOKbAFPsyPfipg2I2yMO+AIccq/pKQhyJ
# A9z1XHxw2V14Tu6fXiDmCWp8KwijSPUV/ARP380hHHrl9Y4a1LlAMIIG7DCCBNSg
# AwIBAgIQMA9vrN1mmHR8qUY2p3gtuTANBgkqhkiG9w0BAQwFADCBiDELMAkGA1UE
# BhMCVVMxEzARBgNVBAgTCk5ldyBKZXJzZXkxFDASBgNVBAcTC0plcnNleSBDaXR5
# MR4wHAYDVQQKExVUaGUgVVNFUlRSVVNUIE5ldHdvcmsxLjAsBgNVBAMTJVVTRVJU
# cnVzdCBSU0EgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNMTkwNTAyMDAwMDAw
# WhcNMzgwMTE4MjM1OTU5WjB9MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3JlYXRl
# ciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0aWdv
# IExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgQ0Ew
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDIGwGv2Sx+iJl9AZg/IJC9
# nIAhVJO5z6A+U++zWsB21hoEpc5Hg7XrxMxJNMvzRWW5+adkFiYJ+9UyUnkuyWPC
# E5u2hj8BBZJmbyGr1XEQeYf0RirNxFrJ29ddSU1yVg/cyeNTmDoqHvzOWEnTv/M5
# u7mkI0Ks0BXDf56iXNc48RaycNOjxN+zxXKsLgp3/A2UUrf8H5VzJD0BKLwPDU+z
# kQGObp0ndVXRFzs0IXuXAZSvf4DP0REKV4TJf1bgvUacgr6Unb+0ILBgfrhN9Q0/
# 29DqhYyKVnHRLZRMyIw80xSinL0m/9NTIMdgaZtYClT0Bef9Maz5yIUXx7gpGaQp
# L0bj3duRX58/Nj4OMGcrRrc1r5a+2kxgzKi7nw0U1BjEMJh0giHPYla1IXMSHv2q
# yghYh3ekFesZVf/QOVQtJu5FGjpvzdeE8NfwKMVPZIMC1Pvi3vG8Aij0bdonigbS
# lofe6GsO8Ft96XZpkyAcSpcsdxkrk5WYnJee647BeFbGRCXfBhKaBi2fA179g6JT
# Z8qx+o2hZMmIklnLqEbAyfKm/31X2xJ2+opBJNQb/HKlFKLUrUMcpEmLQTkUAx4p
# +hulIq6lw02C0I3aa7fb9xhAV3PwcaP7Sn1FNsH3jYL6uckNU4B9+rY5WDLvbxhQ
# iddPnTO9GrWdod6VQXqngwIDAQABo4IBWjCCAVYwHwYDVR0jBBgwFoAUU3m/Wqor
# Ss9UgOHYm8Cd8rIDZsswHQYDVR0OBBYEFBqh+GEZIA/DQXdFKI7RNV8GEgRVMA4G
# A1UdDwEB/wQEAwIBhjASBgNVHRMBAf8ECDAGAQH/AgEAMBMGA1UdJQQMMAoGCCsG
# AQUFBwMIMBEGA1UdIAQKMAgwBgYEVR0gADBQBgNVHR8ESTBHMEWgQ6BBhj9odHRw
# Oi8vY3JsLnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQ2VydGlmaWNhdGlvbkF1
# dGhvcml0eS5jcmwwdgYIKwYBBQUHAQEEajBoMD8GCCsGAQUFBzAChjNodHRwOi8v
# Y3J0LnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQWRkVHJ1c3RDQS5jcnQwJQYI
# KwYBBQUHMAGGGWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAG1UgaUzXRbhtVOBkXXfA3oyCy0lhBGysNsqfSoF9bw7J/RaoLlJWZAp
# bGHLtVDb4n35nwDvQMOt0+LkVvlYQc/xQuUQff+wdB+PxlwJ+TNe6qAcJlhc87QR
# D9XVw+K81Vh4v0h24URnbY+wQxAPjeT5OGK/EwHFhaNMxcyyUzCVpNb0llYIuM1c
# fwGWvnJSajtCN3wWeDmTk5SbsdyybUFtZ83Jb5A9f0VywRsj1sJVhGbks8VmBvbz
# 1kteraMrQoohkv6ob1olcGKBc2NeoLvY3NdK0z2vgwY4Eh0khy3k/ALWPncEvAQ2
# ted3y5wujSMYuaPCRx3wXdahc1cFaJqnyTdlHb7qvNhCg0MFpYumCf/RoZSmTqo9
# CfUFbLfSZFrYKiLCS53xOV5M3kg9mzSWmglfjv33sVKRzj+J9hyhtal1H3G/W0Nd
# ZT1QgW6r8NDT/LKzH7aZlib0PHmLXGTMze4nmuWgwAxyh8FuTVrTHurwROYybxzr
# F06Uw3hlIDsPQaof6aFBnf6xuKBlKjTg3qj5PObBMLvAoGMs/FwWAKjQxH/qEZ0e
# BsambTJdtDgJK0kHqv3sMNrxpy/Pt/360KOE2See+wFmd7lWEOEgbsausfm2usg1
# XTN2jvF8IAwqd661ogKGuinutFoAsYyr4/kKyVRd1LlqdJ69SK6YMYIELDCCBCgC
# AQEwgZEwfTELMAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3Rl
# cjEQMA4GA1UEBxMHU2FsZm9yZDEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSUw
# IwYDVQQDExxTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIENBAhA5TCXhfKBtJ6hl
# 4jvZHSLUMA0GCWCGSAFlAwQCAgUAoIIBazAaBgkqhkiG9w0BCQMxDQYLKoZIhvcN
# AQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTIzMDgxMzA3MzIyMVowPwYJKoZIhvcNAQkE
# MTIEMAnKorMOQI6yXVcDJT6DBZmIOlK24z4W1Av/irJbpYi5IARxjkpRK1uW21Pt
# G/Q5RTCB7QYLKoZIhvcNAQkQAgwxgd0wgdowgdcwFgQUrmKvdQoMvUfWRh91aOK8
# jOfKT5QwgbwEFALWW5Xig3DBVwCV+oj5I92Tf62PMIGjMIGOpIGLMIGIMQswCQYD
# VQQGEwJVUzETMBEGA1UECBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENp
# dHkxHjAcBgNVBAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNF
# UlRydXN0IFJTQSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eQIQMA9vrN1mmHR8qUY2
# p3gtuTANBgkqhkiG9w0BAQEFAASCAgB/5e4t5fgo2Ux6Pg4G0BqcjOp2PaWJGAm7
# QN2MNTKtq4t0GI7os0phnRu/2zaW3rwDCZtgaHby6MbEL8JP6k+I8FPYZL6dpzB/
# bOLGqgOGiuAyxG+DKSHYQswy7+DNPHMwSpKsQMAPqDMh7N8youzvjFhqUS3D9yM+
# fWbt7J1Bf+2RHYmV2I9PteyUIzO/X/GX9lUcXPBl9LUN3Kl2Uzu5mkEYSHN1MD78
# V0WIWdelvuMCet6J49xh321WJlFtTHyTmb7LGbRLRHIVl2jpaLbU+xS21YahJNyZ
# 3YRDrLxjYYYWn63tm6eCFEuHER9JTg7KSjoHfg37U3TbF++Rrp93rsln7abvBVey
# E9zDXLLpSG9KzPkI6srA8gpx1okGvKZ2IPD7TdXls40UBDIlbLxQaP4WmJlBvEKM
# 5dE3/j234wfcSa4ZAxXP3FxVfzBZtPAH45/T4cW1UtoR8yd+7I1YyTz8ulqVwR1v
# KYZ7S0HMnlT6buM/6UTsVg6khUiNtKV0LG+cnx5tDoRKljf0QYQWBq9IPvI5i1Un
# ANCAHnfFWsjSIxcWBSEdDurgCtDjlt18gAWqZvktt2kjmWt+fAFzK0scbGXd3xle
# s+dU+l+kFuWaxQj5s0EZDzHT7wRsDRuPd6mL9No1SSrlyFXYhXE3JNpMQwRzp9ht
# m/3pEdLqaA==
# SIG # End signature block
