# Copyright (C) 2007 Intel Corporation

Function Invoke-AMTForceBoot {
<#
  .Synopsis 
    Invokes the Intel Active Management Technology force boot command 
  .Description
    This CmdLet invokes an Intel Active Management Technology (AMT) force boot to a PXE server, the local hard drive, CD/DVD ROM drive, or remote DVD/CD ISO image from clients that have Intel AMT firmware version 3.0 or higher.
  .Notes
    Supported AMT Firmware Versions: 3.0 or higher.
    
   AMT Provisioning:
      The vPro client AMT firmware must be provisioned prior to accessing AMT functionality. This Cmdlet will fail if it is run against a vPro client that has not been provisioned.
        
    AMT Client Authentication:
      To invoke commands against AMT enabled clients credentials to authenticate must be specified. 
      When no credential is provided as a parameter, the script will use the local logged on Kerberos credential.
      When only the username (Kerberos or Digest) parameter is included the user will be prompted to provide the associated password.
      Credentials should be stored as a PowerShell variable then passed into the Cmdlet with the credential parameter.
      $AMTCredential = get-credential
     
    AMT Client Encryption:
      If the Intel vPro client has been configured to use TLS (a web server certificate has been issued to the Intel Management Engine) the Cmdlet must be called with a -TLS switch.

      When managing an Intel vPro client over TLS (Port 16993) it is important that the computername parameter matchs the primary subject name of the issued TLS certificate. Typically this is the fully qualified domain name (FQDN).

	  If Mutual TLS is desired, the Cmdlet must be called with -TLS switch and with a valid certificate name from the certificate store in the -CertificateName parameter.
    Status:
      Status output designates if the Cmdlet was run successfully. For failed attempts additional status may be provided.
  .Link
    http:\\vproexpert.com
    http:\\www.intel.com\vpro
    http:\\www.intel.com

  .Example
    Invoke-AMTForceBoot vproclient.vprodemo.com -TLS -CertificateName "myCertificate" -Operation:PowerOn -Device:PXE -Credential $AMTCredential

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    vproclient.vprodemo.com 16993                   PowerOn                 PXE                     Successful
  .Example
    Invoke-AMTForceBoot vproclient.vprodemo.com -Operation PowerOn -Device:HardDrive -Username:vprodemo\ITHelpDesk

    Will prompt for Kerberos username password.

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    vproclient.vprodemo.com 16992                   PowerOn                 HardDrive               Successful
  .Example
    Invoke-AMTForceBoot vproclient -Operation:Reset -Device:IDER -IDERPath:"C:\bootable_image.iso" -Credential $AMTCredential
             
    Will boot remote client to ISO img with no SOL interaction.
 	
    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    vproclient              16992                   PowerOn                 IDER                    Successful
  .Example
    Invoke-AMTForceBoot Computer:vproclient -Operation:Reset -Device:BIOSSetup -Credential $AMTCredential -Console SOL -SOLTerminalPath $SOLTerminalPath -SOLTerminalArgList $SOLTerminalArgList
             
    Will boot remote client to BIOS with SOL interaction through a specifed terminal client.
	To use Microsoft telnet the following would be defined
	$SOLTerminalPath = "telnet"
	$SOLTerminalArgList = "-t ANSI 127.0.0.1 %Port"
 	The $SOLTerminalArgList must have a %Port defined so the CMDLet will know where to insert the port

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    vproclient              16992                   PowerOn                 BIOSSetup               Successful
  .Example
    Invoke-AMTForceBoot -ComputerName computer1.vprodemo.com,doesnotexist.vprodemo.com -TLS -Operation Reset -Device:Optical | Where {$_.Status -eq "Failed"}

    Will perform the power operation on every AMT client in the list, but only display the ones that failed.

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    doesnotexist.vprodem... 16993                   Reset                   Optical                 Failed
  .Example
    Get-Content computers.txt | Invoke-AMTForceBoot -TLS -Operation:PowerOn -Device:PXE

    Will pull the list of amt clients from a text file and pipe them in the Invoke-AMTForceBoot CMDLet.

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    computer1.vprodemo.com  16993                   PowerOn                 PXE                     Successful
    computer2.vprodemo.com  16993                   PowerOn                 PXE                     Successful
    computer3.vprodemo.com  16993                   PowerOn                 PXE                     Successful
  .Example
    Get-SomeDataFromOtherCMDLet | Select ComputerName | Invoke-AMTForceBoot -TLS -Operation PowerOn -Device:HardDrive 

    Get-SomeDataFromOtherCMDLet is a custom script that has an output of ComputerName, Port, and Operation; however, you only select ComputerName.  Remaining parameters are manually provided.

    ComputerName            Port                    Operation               Device                  Status
    ------------            ----                    ---------               ------                  ------
    computer1.vprodemo.com  16993                   PowerOn                 HardDrive               Successful
    computer2.vprodemo.com  16993                   PowerOn                 HardDrive               Successful
    computer3.vprodemo.com  16993                   PowerOn                 HardDrive               Successful
  .Example  
    Invoke-AMTForceBoot vproclient.vprodemo.com -Operation:Reset -Device:BIOSSetup -Credential $AMTCredential -Console SOL -SOLTerminalPath "telnet" -SOLTerminalArgList "-t ANSI 127.0.0.1 %Port" 
   
    This will reboot the client to the BIOS Setup screens while connecting SOL to a telnet window.

    Ok

    ComputerName : 192.168.1.106
    Port         : 16992
    Operation    : reset
    Device       : BIOSSetup
    Status       : Successful

#>
[CmdletBinding()]
Param (
	[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$true, position=0, HelpMessage="Hostname, FQDN, or IP Address")] [String[]] $ComputerName,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=1, HelpMessage="Valid Ports are 16992 (non-TLS) or 16993 (TLS)")][ValidateSet("16992", "16993")] [String] $Port,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Use TLS (Port 16993)")] [switch] $TLS,  
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Accept self-signed certificate for TLS connection (skip any certificate checks.)")] [switch] $AcceptSelfSignedCert,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Name of certificate. (Use for mutual TLS)")] [string] $CertificateName,
	[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=2, HelpMessage="Valid Operations are PowerOn or Reset")][ValidateSet("PowerOn", "Reset")] [String] $Operation,
	[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=3, HelpMessage="Valid Devices are Default, HardDrive, PXE, Optical, IDER or BIOSSetup")][ValidateSet("Default", "HardDrive", "PXE", "Optical", "IDER", "BIOSSetup")] [String] $Device,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=4, HelpMessage="Enter valid path to ISO or IMG file")] [String] $IDERPath,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=5, HelpMessage="Valid Consoles are SOL")][ValidateSet("SOL")] [String] $Console,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=6, HelpMessage="Path to SOL console")] [String] $SOLTerminalPath,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=7, HelpMessage="Path to SOL console")] [String] $SOLTerminalArgList,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest or Kerberos User")] [string] $Username,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest or Kerberos Password")] [string] $Password,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, position=8, HelpMessage="PS Credential")] [System.Management.Automation.PSCredential] $Credential
)


Process {
	#create a connection object
	try
	{
		$Results = @()
		$Obj = new-object psobject
		$Connection = New-Object Intel.Management.Wsman.WsmanConnection 

		if ($Credential.username.Length -gt 0) {
		  $Connection.SetCredentials($Credential.Username, $Credential.Password)  
		} elseif ($Username.length -gt 0) {
		  if ($Password.length -gt 0) {
			 $Connection.SetCredentials($Username, $Password)  
		  } else {
			 $Cred = Get-Credential $Username
			 $Connection.SetCredentials($Cred.Username, $Cred.Password)
		  }
		}  
		   
		if ($Port.Length -lt 1) {
		   if ($TLS.IsPresent) {
			 $Port = 16993;
		   }
		   else {
			 $Port = 16992;
		   }
		}

		if($CertificateName.Length -gt 0)
		{
			$Connection.Options.SetClientCertificateByCertificateName($CertificateName)
		}
		
		if($AcceptSelfSignedCert.IsPresent)
		{
			$Connection.Options.AcceptSelfSignedCertificate=$true;			
		}

		switch ($Operation) {
		  "PowerOn" {$PowerInt = 2}
		  "Reset" {$PowerInt = 10}
		}

		if($Console -eq "SOL")
			{
				if($SOLTerminalArgList -ne "")
				{
					if($SOLTerminalArgList.contains("%Port")) {}
					else
					{
					write-output ""
					write-output 'The $SOLTerminalArgList parameter must contain a "%Port" parameter.'
					write-output 'To use Microsoft telnet the following would be defined'
					write-output '$SOLTerminalPath = "telnet"'
					write-output '$SOLTerminalArgList = "-t ANSI 127.0.0.1 %Port"'
					write-output ""
					return
					}		
				}

				if(($SOLTerminalPath -eq "") -or ($SOLTerminalArgList -eq ""))
				{
					write-output ""
					write-output 'If "-console SOL" is specifed:'
					write-output '  -SOLTerminalPath parameter containing the path to the terminal application must be present.'
					write-output '  -SOLTerminalArgList parameter containing the arguments to pass to the terminal application must be present.'

					write-output 'To use Microsoft telnet the following would be defined'
					write-output '$SOLTerminalPath = "telnet"'
					write-output '$SOLTerminalArgList = "-t ANSI 127.0.0.1 %Port"'
					write-output ""
					return
				}
			}
	

		ForEach ($Comp in $ComputerName) {
		
			try {
				#Attempt Connection with Client
				$Connection.SetHost($Comp, $Port)
				#Identify if redirection library required
				if (($device -eq "IDER") -or ($console -eq "SOL")) {
		
					if ($device -eq "IDER") {
						$IDERPath = $IDERPath.ToLower()

						#params for doing redirection
						if ($IDERPath.EndsWith("iso")) {
							$floppyFile = "$env:TEMP\floppy.img"
							$cdFile = $IDERPath
							$iderBootDevice = "1"

							write-host "Performing IDER boot with CD image."
						} elseif ($IDERPath.EndsWith("img")) {
							$cdFile = "$env:TEMP\cd.iso"
							$floppyFile = $IDERPath
							$iderBootDevice = "0"
							write-host "Performing IDER boot with floppy image."
						} else {
							write-host "You must enter a valid ISO or IMG file."
							return
						}
					}

					$cimRef = $Connection.NewReference("SELECT * FROM http://intel.com/wbem/wscim/1/amt-schema/1/AMT_RedirectionService")
					$cimObj = $cimRef.Get();
					try{
					if (!$cimObj.GetProperty("EnabledState").ToString().Equals("32771")) {
						$cimObj.SetProperty("EnabledState", "32771")
						$cimRef.Put($cimObj)
					}
					if ($cimObj.GetProperty("ListenerEnabled").ToString().Equals("false")) {
						$cimObj.SetProperty("ListenerEnabled", "true")
						$cimRef.Put($cimObj)
					}
					}catch
					{
					Write-Host $_.Exception.Message
					}
		
					$LibSettingsFile="$env:TEMP\MC.ini"
					$CertTrustFile="$env:TEMP\certs.pem"

					$imrLib = New-Object Intel.Management.Redirection.MediaRedirection

					#Initialize the Intel Media Redirection Library (imrLib)
					[Intel.Management.Redirection.IMRVersion]$libVersion = New-Object Intel.Management.Redirection.IMRVersion
					$imrResult = $imrLib.Init([ref] $libVersion, $LibSettingsFile)
					
					$imrResult=$imrLib.RemoveAllClients()

					#define a redirection client based on the WsmanConnection Address
					$imrClient = $imrLib.GetClientInfo($Connection.Address)

					#add the redirection client to the Library (Library return a value clientID)
					[System.UInt32]$clientId=[System.UInt32]::MaxValue
					$imrResult = $imrLib.AddClient($imrClient, [ref] $clientId)
					
					#SOL session cannot be open while boot to HD \ Cd\ PXE is performed.
					if(($Device -eq "PXE") -or ($Device -eq "Optical") -or ($Device -eq "HardDrive")){
						write-output "SOL or IDER session cannot be open during boot to Hard Drive, CD or PXE."
					} else {
					
					if ($device -eq "IDER") {
						#create redirection session parameters (e.g user name and password) based on the Wsman Connection
						$iderParams = $imrLib.CreateSessionParams($Connection.Username, $Connection.Password)

						#define session timeouts
						$iderTimeouts = $imrLib.GetDefaultIDERTout()

						#Open an IDER session
						$imrResult = $imrLib.IDEROpenTCPSession($clientId, $iderParams, $iderTimeouts, $floppyFile, $cdFile)

						#after opening the session the next thing we will do is send a command to enable the IDER device for immediate use
						[Intel.Management.Redirection.IDERDeviceResult]$deviceResult = New-Object Intel.Management.Redirection.IDERDeviceResult
						[Intel.Management.Redirection.IDERDeviceCmd]$deviceCommand = New-Object Intel.Management.Redirection.IDERDeviceCmd

						$deviceCommand.pri_op = [Intel.Management.Redirection.SetOperation]::Enable
						$deviceCommand.pri_timing = [Intel.Management.Redirection.SetOption]::Immediately

						#enable the ider device for immediate use by setting the device state
						$imrResult = $imrLib.IDERSetDeviceState($clientId, $deviceCommand, [ref] $deviceResult);
					}

					
					if ($console -eq "SOL"){
						
						#create redirection session parameters (e.g user name and password) based on the Wsman Connection
						$solParams = $imrLib.CreateSessionParams($Connection.Username, $Connection.Password)

						#define session timeouts
						$solTimeouts = $imrLib.GetDefaultSOLTout()

						#openSol Session
						$imrResult = $imrLib.SOLOpenTCPSession($clientId, $solParams, $solTimeouts, $imrLib.GetDefaultSOLParams())

						Write-Host $imrResult

						$link= $imrLib.CreateTerminalLink($clientId);
						if($link.Open() -eq $false) {
							write-output "Error opening SOL Link"
							#close the session
							$imrResult = $imrLib.SOLCloseSession($clientId)
							#remove client
							$imrResult = $imrLib.RemoveClient($clientId)
							#close the library
							$imrResult = $imrLib.Close()
							return
						}
					   }
					}
				}
				#Set the boot source
				$SourceRef = $Connection.NewReference("SELECT * FROM CIM_BootSourceSetting")

				switch ($Device) {
					"HardDrive" {$InstanceID = 'Intel(r) AMT: Force Hard-drive Boot'}
					"PXE" {$InstanceID = 'Intel(r) AMT: Force PXE Boot'}
					"Optical" {$InstanceID = 'Intel(r) AMT: Force CD/DVD Boot'}
				}

				$SourceRef.AddSelector("InstanceID", $InstanceID)

				$objRef = $Connection.NewReference("SELECT * FROM CIM_BootConfigSetting")
				$objRef.AddSelector("InstanceID", 'Intel(r) AMT: Boot Configuration 0')

				$InputObj = $objRef.CreateMethodInput("ChangeBootOrder")
				if (($device -eq "HardDrive") -or ($device -eq "PXE") -or ($device -eq "Optical")) {
					$InputObj.SetProperty("Source", $SourceRef)
				}

				$OutObj = $objRef.InvokeMethod($InputObj)
							
				#Set the AMT boot settings
				$cimRef = $Connection.NewReference("SELECT * FROM AMT_BootSettingData")
				$cimObj = $cimRef.Get()


				if ($device -eq "IDER") {
					$cimObj.SetProperty("UseIDER", "true")
					$cimObj.SetProperty("IDERBootDevice", $iderBootDevice)
				} else {
					$cimObj.SetProperty("UseIDER", "false")
				}

				if ($device -eq "BIOSSetup") {
					$cimObj.SetProperty("BIOSSetup", "true")
				} else {
					$cimObj.SetProperty("BIOSSetup", "false")
				}

				if(($console -eq "SOL") -and ($Device -ne "PXE") -and ($Device -ne "Optical") -and ($Device -ne "Harddrive")) {
					$cimObj.SetProperty("UseSOL", "true")
				} else {
					$cimObj.SetProperty("UseSOL", "false")
				}

				$cimObj.SetProperty("BootMediaIndex", "0");
				$cimObj.SetProperty("UserPasswordBypass", "false")
				$putResult=$cimRef.Put($cimObj)

				$BootConfigSettingRef = $Connection.NewReference("SELECT * FROM CIM_BootConfigSetting")
				$BootConfigSettingRef.AddSelector("InstanceID", 'Intel(r) AMT: Boot Configuration 0')

				$objRef = $Connection.NewReference("CIM_BootService")
				$InputObj = $objref.CreateMethodInput("SetBootConfigRole")
				$InputObj.SetProperty("BootConfigSetting", $BootConfigSettingRef)
				$InputObj.SetProperty("Role", "1")
				$OutObj = $objRef.InvokeMethod($InputObj)

				#Get a reference to the ManagedSystem (User of the Service)
				$UserRef = $Connection.NewReference("SELECT * from CIM_ComputerSystem WHERE Name='ManagedSystem'")

				#Get the Instance of CIM_AssociatedPowerManagmentService for the ManagedSystem
				$ObjRef = $Connection.NewReference("CIM_AssociatedPowerManagementService");
				$ObjRef.AddSelector("UserOfService", $userRef);
				$AssociatedObj = $ObjRef.Get()

				#Get current power state
				$PowerState = $AssociatedObj.GetProperty("PowerState").ToString()

				#If client in S4/S5 then poweron operation otherwise reset operation
				if (($PowerState -eq 8) -or ($PowerState -eq 7)) {
					$PowerInt = 2
				} 
				else {
					$PowerInt = 10
				}

				#Now associate to the Provider of the Service (CIM_PowerManagementService)
				$ServiceRef = $AssociatedObj.GetProperty("ServiceProvided").Ref

				#Now invoke the RequestPowerStateChange
				$InputObj = $ServiceRef.CreateMethodInput("RequestPowerStateChange")
				$InputObj.SetProperty("PowerState", "$PowerInt")
				$InputObj.SetProperty("ManagedElement", $UserRef)
				$OutObj = $ServiceRef.InvokeMethod($InputObj);

				#Note Return Status
				$ReturnValue = $OutObj.GetProperty("ReturnValue").ToString()


				#Display SOL console
				if (($console -eq "SOL" ) -and ($device -ne "HardDrive") -and ($device -ne "PXE") -and ($device -ne "Optical"))  {					
					$argumentlist = $SOLTerminalArgList.replace("%Port", $link.Port)
					
					start-process -FilePath $SOLTerminalPath -ArgumentList $argumentlist -Wait
				
				}


				#Identify if redirection library required
				if (($console -eq "SOL")-and ($link) ) {
					$link.Close()

					#close the session
					$imrResult = $imrLib.SOLCloseSession($clientId)
					#remove client
					$imrResult = $imrLib.RemoveClient($clientId)
					#close the library
					$imrResult = $imrLib.Close()
				}

				if ($device -eq "IDER") {
					write-host -NoNewLine "Press any key to end the IDER session with" $Comp
					$null = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
					write-host ""

					#close the session
					$imrResult = $imrLib.IDERCloseSession($clientId)
					#remove client
					$imrResult = $imrLib.RemoveClient($clientId)
					#close the library
					$imrResult = $imrLib.Close()
				}
	 
				#Add Member to Object. Include Computer Name and Operation success 
				$Obj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
				$Obj | Add-Member -MemberType noteproperty -Name Port -value $Port
				$Obj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
				$Obj | Add-Member -MemberType noteproperty -Name Device -value $Device
				if ($ReturnValue -eq 0) {
					$Obj | Add-Member -MemberType noteproperty -Name Status -value "Successful"
				} else {
					$Obj | Add-Member -MemberType noteproperty -Name Status -value "Failed"
					$Obj | Add-Member -MemberType noteproperty -Name Reason -value $_.Exception.Message
				}

			} catch {
				#Identify if redirection library required
				if ($console -eq "SOL")  {
					if($link)
					{
						try
						{
							$link.Close()
						}
						catch
						{
							
						}
					}
					#close the session
					$imrResult = $imrLib.SOLCloseSession($clientId)
					}
				
				if ($device -eq "IDER") {
					write-host -NoNewLine "Press any key to end the IDER session with" $Comp
					$null = $Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown")
					write-host ""
					#close the session
					$imrResult = $imrLib.IDERCloseSession($clientId)
				}
				
				#remove client
					$imrResult = $imrLib.RemoveClient($clientId)
				#Close the library
				if ($imrLib) {
					$imrResult = $imrLib.Close()
					
			
	 
				#Add Member to Object. Include Computer Name and Operation success 
				$Obj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
				$Obj | Add-Member -MemberType noteproperty -Name Port -value $Port
				$Obj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
				$Obj | Add-Member -MemberType noteproperty -Name Device -value $Device
				$Obj | Add-Member -MemberType noteproperty -Name Status -value "Failed"
				$Obj | Add-Member -MemberType noteproperty -Name Reason -value $_.Exception.Message
					
				}
			}

			#Append to Result 
			$Results += $Obj
		}
	}
	catch
	{
		$Obj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
		$Obj | Add-Member -MemberType noteproperty -Name Port -value $Port
		$Obj | Add-Member -MemberType noteproperty -Name Operation -value $Operation
		$Obj | Add-Member -MemberType noteproperty -Name Device -value $Device
		$Obj | Add-Member -MemberType noteproperty -Name Status -value "Failed"
		$Obj | Add-Member -MemberType noteproperty -Name Reason -value $_.Exception.Message
		
		#Append to Result 
		$Results += $Obj
	}
	
	# Set back to false for the next cmdlet, if PS is still open
	$Connection.Options.AcceptSelfSignedCertificate=$false;

	#Write out Results
	Write-Output $Results

}

}
# SIG # Begin signature block
# MIIogwYJKoZIhvcNAQcCoIIodDCCKHACAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDvfvAJlb6Ra9dI
# 8eoXESo+EPPrkrb/gM/ds3ZNIZO9OaCCEfIwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYcMIIEBKADAgECAhAz1wio
# kUBTGeKlu9M5ua1uMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLjAs
# BgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBFViBSMzYwggGi
# MA0GCSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQC70f4et0JbePWQp64sg/GNIdMw
# hoV739PN2RZLrIXFuwHP4owoEXIEdiyBxasSekBKxRDogRQ5G19PB/YwMDB/NSXl
# wHM9QAmU6Kj46zkLVdW2DIseJ/jePiLBv+9l7nPuZd0o3bsffZsyf7eZVReqskmo
# PBBqOsMhspmoQ9c7gqgZYbU+alpduLyeE9AKnvVbj2k4aOqlH1vKI+4L7bzQHkND
# brBTjMJzKkQxbr6PuMYC9ruCBBV5DFIg6JgncWHvL+T4AvszWbX0w1Xn3/YIIq62
# 0QlZ7AGfc4m3Q0/V8tm9VlkJ3bcX9sR0gLqHRqwG29sEDdVOuu6MCTQZlRvmcBME
# Jd+PuNeEM4xspgzraLqVT3xE6NRpjSV5wyHxNXf4T7YSVZXQVugYAtXueciGoWnx
# G06UE2oHYvDQa5mll1CeHDOhHu5hiwVoHI717iaQg9b+cYWnmvINFD42tRKtd3V6
# zOdGNmqQU8vGlHHeBzoh+dYyZ+CcblSGoGSgg8sCAwEAAaOCAWMwggFfMB8GA1Ud
# IwQYMBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBSBMpJBKyjNRsjE
# osYqORLsSKk/FDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADAT
# BgNVHSUEDDAKBggrBgEFBQcDAzAaBgNVHSAEEzARMAYGBFUdIAAwBwYFZ4EMAQMw
# SwYDVR0fBEQwQjBAoD6gPIY6aHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UHVibGljQ29kZVNpZ25pbmdSb290UjQ2LmNybDB7BggrBgEFBQcBAQRvMG0wRgYI
# KwYBBQUHMAKGOmh0dHA6Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0Nv
# ZGVTaWduaW5nUm9vdFI0Ni5wN2MwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMA0GCSqGSIb3DQEBDAUAA4ICAQBfNqz7+fZyWhS38Asd3tj9lwHS
# /QHumS2G6Pa38Dn/1oFKWqdCSgotFZ3mlP3FaUqy10vxFhJM9r6QZmWLLXTUqwj3
# ahEDCHd8vmnhsNufJIkD1t5cpOCy1rTP4zjVuW3MJ9bOZBHoEHJ20/ng6SyJ6UnT
# s5eWBgrh9grIQZqRXYHYNneYyoBBl6j4kT9jn6rNVFRLgOr1F2bTlHH9nv1HMePp
# GoYd074g0j+xUl+yk72MlQmYco+VAfSYQ6VK+xQmqp02v3Kw/Ny9hA3s7TSoXpUr
# OBZjBXXZ9jEuFWvilLIq0nQ1tZiao/74Ky+2F0snbFrmuXZe2obdq2TWauqDGIgb
# MYL1iLOUJcAhLwhpAuNMu0wqETDrgXkG4UGVKtQg9guT5Hx2DJ0dJmtfhAH2KpnN
# r97H8OQYok6bLyoMZqaSdSa+2UA1E2+upjcaeuitHFFjBypWBmztfhj24+xkc6Zt
# CDaLrw+ZrnVrFyvCTWrDUUZBVumPwo3/E3Gb2u2e05+r5UWmEsUUWlJBl6MGAAjF
# 5hzqJ4I8O9vmRsTvLQA1E802fZ3lqicIBczOwDYOSxlP0GOabb/FKVMxItt1UHeG
# 0PL4au5rBhs+hSMrl8h+eplBDN1Yfw6owxI9OjWb4J0sjBeBVESoeh2YnZZ/WVim
# VGX/UUIL+Efrz/jlvzCCBlswggTDoAMCAQICEHAQMvP0AIEdrcUef3h12LwwDQYJ
# KoZIhvcNAQELBQAwVzELMAkGA1UEBhMCR0IxGDAWBgNVBAoTD1NlY3RpZ28gTGlt
# aXRlZDEuMCwGA1UEAxMlU2VjdGlnbyBQdWJsaWMgQ29kZSBTaWduaW5nIENBIEVW
# IFIzNjAeFw0yMzA0MDQwMDAwMDBaFw0yNDA0MDMyMzU5NTlaMIG7MRAwDgYDVQQF
# EwcyMTg5MDc0MRMwEQYLKwYBBAGCNzwCAQMTAlVTMRkwFwYLKwYBBAGCNzwCAQIT
# CERlbGF3YXJlMR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjELMAkGA1UE
# BhMCVVMxEzARBgNVBAgMCkNhbGlmb3JuaWExGjAYBgNVBAoMEUludGVsIENvcnBv
# cmF0aW9uMRowGAYDVQQDDBFJbnRlbCBDb3Jwb3JhdGlvbjCCAaIwDQYJKoZIhvcN
# AQEBBQADggGPADCCAYoCggGBANO3BjDeeK28E+zFxToyp77/6fB+Mp6913C+9t1U
# JguD/sBVnxljfvQScWzRrvF0chAOz5rjlnvUTXUFsgvJ4XFIZQtG/Rfyb4qvQbFE
# vdoS1kFlMAtXnMuuDSMbncPju4R/5gLfMzHFLRpHOoUmeRFlwEkyAYVwjDvH+OcR
# WW2Vm72ggaVeqgW5kIx3R4VQSXgJ3ToPP/49/9YjEwhM0yLccTPJKVB/6unAJaY6
# /j15gi4owbo6K88625JlZTy7drEhZvvI9vlStkPUOxbHQmU72QG7bF1ofao/4lTH
# pE6RAXTppoWlu24kRhLcXaIIAitMDqs9E0SFdV2qcAxc7d38v0l2sds/sy2EWMU9
# RZAjJD4fH/Bm0hJVm5937cq/sf20lucUQnu96+5zVHXSIuD65dkQ/6aoGxwZty49
# rdTFY494ri++yfw6NWrbQkIoc2c1HG0jHRAV6m3dgzwY0104g/BpZkLogT4EEEO5
# XRm6k1PSMxoM194ytHU+pJ2B2QIDAQABo4IBvDCCAbgwHwYDVR0jBBgwFoAUgTKS
# QSsozUbIxKLGKjkS7EipPxQwHQYDVR0OBBYEFJLGD4aF5QaeHQosntNMcCm3IDAZ
# MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMEkGA1UdIARCMEAwNQYMKwYBBAGyMQECAQYBMCUwIwYIKwYBBQUHAgEWF2h0
# dHBzOi8vc2VjdGlnby5jb20vQ1BTMAcGBWeBDAEDMEsGA1UdHwREMEIwQKA+oDyG
# Omh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVTaWduaW5n
# Q0FFVlIzNi5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsGAQUFBzAChjpodHRwOi8v
# Y3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmluZ0NBRVZSMzYu
# Y3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTAuBgNVHREE
# JzAloCMGCCsGAQUFBwgDoBcwFQwTVVMtREVMQVdBUkUtMjE4OTA3NDANBgkqhkiG
# 9w0BAQsFAAOCAYEAttRv5ICWB9YKIXk6xtQ385RQks/95gKJP+T62x8e2rHfQXY4
# GQChplzrceWnolz+TBJF1Dt8HYLUjpj0tgQHcU1mT54Lz4vt8eRHfldjRUC3I+Sy
# FxfWncIWYru27DPKAUEVmCHNF6++myfE7FLTFsL2ogZhZF1sUIBAvDd7d3j0wtz4
# N4d9MhjIfncOOqTOctwEbLtSTX9xIOry4ZgsDVs/QWiqyMyUaG649bcWWXDWxIq+
# 6jtY4WJn4KQUhPVzP53e45Y7Leb+69GCSkfQ5cEZlSOZq2PeFRqP3PwVQ946hbm8
# saXEl3iaZ8s5YY2JorFvstkqYXiQ9O55wAZ4fTeyhWjVNpkqDiIHpO+UYREJ5AHG
# nsZEJJkx8E/Y+zX7YuDMJmV7+WLI1CvIyCLoNhUMupfReykiQKD1P+CCe6UzSOey
# mSrh0X/XpOcDz/+sSgUIziD5HUuMYp0czD4Up/FFrl3qX3te9N4hQt74Ifh35DLR
# g/DdRlqPI70u7jl5MYIV5zCCFeMCAQEwazBXMQswCQYDVQQGEwJHQjEYMBYGA1UE
# ChMPU2VjdGlnbyBMaW1pdGVkMS4wLAYDVQQDEyVTZWN0aWdvIFB1YmxpYyBDb2Rl
# IFNpZ25pbmcgQ0EgRVYgUjM2AhBwEDLz9ACBHa3FHn94ddi8MA0GCWCGSAFlAwQC
# AQUAoHwwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIE
# IKGNpK40zuCv5Qv67zunWl3HjdNfrrzxF6gzeTu7N3dmMA0GCSqGSIb3DQEBAQUA
# BIIBgFo2sLufUoYWYlF/J19SN/N6xblb7GvUVy5jRYb7gJ0IWCcOJn9/0DrOCQIN
# vVw4xo0rOX2ZrHyzeJ9kzhoBFPaOAlq0DnxgOZEjsBCLjZEnXx3HvOPCsi+ilz33
# xqpXL/7gtm2Aqi4fR7BmdhtXbXYGqXReRXVHbl2TZs1onAXE9W9rXpKJ1m1KWFmt
# tUfebAzPf7rYhxufd41Y8nRmVovRDiXuB0zYLLL4NlxoPyWJtZauE7QRLE/N/7Ll
# CQYoK/k37Yyub51Kq8uracbIek1G2Ik8VXSy3GptcibsDSREgcRucaQzwH017227
# OQ5wo6Y02LeeorE1MI8z4X8kMAevur86yi+weXJ0pZg2r1qbXpSog1XdeEgQ7F4v
# pVn9hS88sj15lQnRbX+fVHj7F0j0JDkybiAUnBKsq1AvaNjp1Go1EgijpYT+b8W+
# 4+7opBBGCEGAJtbh8jVDS2aocCII13fz4XJrEF4A8YIGKrA4bCKX8iweZHasH8KV
# n1+42KGCE08wghNLBgorBgEEAYI3AwMBMYITOzCCEzcGCSqGSIb3DQEHAqCCEygw
# ghMkAgEDMQ8wDQYJYIZIAWUDBAICBQAwgfAGCyqGSIb3DQEJEAEEoIHgBIHdMIHa
# AgEBBgorBgEEAbIxAgEBMDEwDQYJYIZIAWUDBAIBBQAEIDTgrZtwRAM7IGmvMUaG
# kwpp0E+X/nFb8ijFNCsKku7kAhUAjoopn4sba+6C6I7XxUOsRNvuN9AYDzIwMjMw
# ODEzMDczMzI0WqBupGwwajELMAkGA1UEBhMCR0IxEzARBgNVBAgTCk1hbmNoZXN0
# ZXIxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDEsMCoGA1UEAwwjU2VjdGlnbyBS
# U0EgVGltZSBTdGFtcGluZyBTaWduZXIgIzSggg3pMIIG9TCCBN2gAwIBAgIQOUwl
# 4XygbSeoZeI72R0i1DANBgkqhkiG9w0BAQwFADB9MQswCQYDVQQGEwJHQjEbMBkG
# A1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUg
# U3RhbXBpbmcgQ0EwHhcNMjMwNTAzMDAwMDAwWhcNMzQwODAyMjM1OTU5WjBqMQsw
# CQYDVQQGEwJHQjETMBEGA1UECBMKTWFuY2hlc3RlcjEYMBYGA1UEChMPU2VjdGln
# byBMaW1pdGVkMSwwKgYDVQQDDCNTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIFNp
# Z25lciAjNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKSTKFJLzyeH
# dqQpHJk4wOcO1NEc7GjLAWTkis13sHFlgryf/Iu7u5WY+yURjlqICWYRFFiyuiJb
# 5vYy8V0twHqiDuDgVmTtoeWBIHIgZEFsx8MI+vN9Xe8hmsJ+1yzDuhGYHvzTIAhC
# s1+/f4hYMqsws9iMepZKGRNcrPznq+kcFi6wsDiVSs+FUKtnAyWhuzjpD2+pWpqR
# KBM1uR/zPeEkyGuxmegN77tN5T2MVAOR0Pwtz1UzOHoJHAfRIuBjhqe+/dKDcxIU
# m5pMCUa9NLzhS1B7cuBb/Rm7HzxqGXtuuy1EKr48TMysigSTxleGoHM2K4GX+hub
# foiH2FJ5if5udzfXu1Cf+hglTxPyXnypsSBaKaujQod34PRMAkjdWKVTpqOg7RmW
# ZRUpxe0zMCXmloOBmvZgZpBYB4DNQnWs+7SR0MXdAUBqtqgQ7vaNereeda/TpUsY
# oQyfV7BeJUeRdM11EtGcb+ReDZvsdSbu/tP1ki9ShejaRFEqoswAyodmQ6MbAO+i
# tZadYq0nC/IbSsnDlEI3iCCEqIeuw7ojcnv4VO/4ayewhfWnQ4XYKzl021p3AtGk
# +vXNnD3MH65R0Hts2B0tEUJTcXTC5TWqLVIS2SXP8NPQkUMS1zJ9mGzjd0HI/x8k
# VO9urcY+VXvxXIc6ZPFgSwVP77kv7AkTAgMBAAGjggGCMIIBfjAfBgNVHSMEGDAW
# gBQaofhhGSAPw0F3RSiO0TVfBhIEVTAdBgNVHQ4EFgQUAw8xyJEqk71j89FdTaQ0
# D9KVARgwDgYDVR0PAQH/BAQDAgbAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwSgYDVR0gBEMwQTA1BgwrBgEEAbIxAQIBAwgwJTAjBggrBgEF
# BQcCARYXaHR0cHM6Ly9zZWN0aWdvLmNvbS9DUFMwCAYGZ4EMAQQCMEQGA1UdHwQ9
# MDswOaA3oDWGM2h0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVT
# dGFtcGluZ0NBLmNybDB0BggrBgEFBQcBAQRoMGYwPwYIKwYBBQUHMAKGM2h0dHA6
# Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVTdGFtcGluZ0NBLmNydDAj
# BggrBgEFBQcwAYYXaHR0cDovL29jc3Auc2VjdGlnby5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAEybZVj64HnP7xXDMm3eM5Hrd1ji673LSjx13n6UbcMixwSV32VpYRMM
# 9gye9YkgXsGHxwMkysel8Cbf+PgxZQ3g621RV6aMhFIIRhwqwt7y2opF87739i7E
# fu347Wi/elZI6WHlmjl3vL66kWSIdf9dhRY0J9Ipy//tLdr/vpMM7G2iDczD8W69
# IZEaIwBSrZfUYngqhHmo1z2sIY9wwyR5OpfxDaOjW1PYqwC6WPs1gE9fKHFsGV7C
# g3KQruDG2PKZ++q0kmV8B3w1RB2tWBhrYvvebMQKqWzTIUZw3C+NdUwjwkHQepY7
# w0vdzZImdHZcN6CaJJ5OX07Tjw/lE09ZRGVLQ2TPSPhnZ7lNv8wNsTow0KE9SK16
# ZeTs3+AB8LMqSjmswaT5qX010DJAoLEZKhghssh9BXEaSyc2quCYHIN158d+S4RD
# zUP7kJd2KhKsQMFwW5kKQPqAbZRhe8huuchnZyRcUI0BIN4H9wHU+C4RzZ2D5fjK
# JRxEPSflsIZHKgsbhHZ9e2hPjbf3E7TtoC3ucw/ZELqdmSx813UfjxDElOZ+JOWV
# SoiMJ9aFZh35rmR2kehI/shVCu0pwx/eOKbAFPsyPfipg2I2yMO+AIccq/pKQhyJ
# A9z1XHxw2V14Tu6fXiDmCWp8KwijSPUV/ARP380hHHrl9Y4a1LlAMIIG7DCCBNSg
# AwIBAgIQMA9vrN1mmHR8qUY2p3gtuTANBgkqhkiG9w0BAQwFADCBiDELMAkGA1UE
# BhMCVVMxEzARBgNVBAgTCk5ldyBKZXJzZXkxFDASBgNVBAcTC0plcnNleSBDaXR5
# MR4wHAYDVQQKExVUaGUgVVNFUlRSVVNUIE5ldHdvcmsxLjAsBgNVBAMTJVVTRVJU
# cnVzdCBSU0EgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNMTkwNTAyMDAwMDAw
# WhcNMzgwMTE4MjM1OTU5WjB9MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3JlYXRl
# ciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0aWdv
# IExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgQ0Ew
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDIGwGv2Sx+iJl9AZg/IJC9
# nIAhVJO5z6A+U++zWsB21hoEpc5Hg7XrxMxJNMvzRWW5+adkFiYJ+9UyUnkuyWPC
# E5u2hj8BBZJmbyGr1XEQeYf0RirNxFrJ29ddSU1yVg/cyeNTmDoqHvzOWEnTv/M5
# u7mkI0Ks0BXDf56iXNc48RaycNOjxN+zxXKsLgp3/A2UUrf8H5VzJD0BKLwPDU+z
# kQGObp0ndVXRFzs0IXuXAZSvf4DP0REKV4TJf1bgvUacgr6Unb+0ILBgfrhN9Q0/
# 29DqhYyKVnHRLZRMyIw80xSinL0m/9NTIMdgaZtYClT0Bef9Maz5yIUXx7gpGaQp
# L0bj3duRX58/Nj4OMGcrRrc1r5a+2kxgzKi7nw0U1BjEMJh0giHPYla1IXMSHv2q
# yghYh3ekFesZVf/QOVQtJu5FGjpvzdeE8NfwKMVPZIMC1Pvi3vG8Aij0bdonigbS
# lofe6GsO8Ft96XZpkyAcSpcsdxkrk5WYnJee647BeFbGRCXfBhKaBi2fA179g6JT
# Z8qx+o2hZMmIklnLqEbAyfKm/31X2xJ2+opBJNQb/HKlFKLUrUMcpEmLQTkUAx4p
# +hulIq6lw02C0I3aa7fb9xhAV3PwcaP7Sn1FNsH3jYL6uckNU4B9+rY5WDLvbxhQ
# iddPnTO9GrWdod6VQXqngwIDAQABo4IBWjCCAVYwHwYDVR0jBBgwFoAUU3m/Wqor
# Ss9UgOHYm8Cd8rIDZsswHQYDVR0OBBYEFBqh+GEZIA/DQXdFKI7RNV8GEgRVMA4G
# A1UdDwEB/wQEAwIBhjASBgNVHRMBAf8ECDAGAQH/AgEAMBMGA1UdJQQMMAoGCCsG
# AQUFBwMIMBEGA1UdIAQKMAgwBgYEVR0gADBQBgNVHR8ESTBHMEWgQ6BBhj9odHRw
# Oi8vY3JsLnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQ2VydGlmaWNhdGlvbkF1
# dGhvcml0eS5jcmwwdgYIKwYBBQUHAQEEajBoMD8GCCsGAQUFBzAChjNodHRwOi8v
# Y3J0LnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQWRkVHJ1c3RDQS5jcnQwJQYI
# KwYBBQUHMAGGGWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAG1UgaUzXRbhtVOBkXXfA3oyCy0lhBGysNsqfSoF9bw7J/RaoLlJWZAp
# bGHLtVDb4n35nwDvQMOt0+LkVvlYQc/xQuUQff+wdB+PxlwJ+TNe6qAcJlhc87QR
# D9XVw+K81Vh4v0h24URnbY+wQxAPjeT5OGK/EwHFhaNMxcyyUzCVpNb0llYIuM1c
# fwGWvnJSajtCN3wWeDmTk5SbsdyybUFtZ83Jb5A9f0VywRsj1sJVhGbks8VmBvbz
# 1kteraMrQoohkv6ob1olcGKBc2NeoLvY3NdK0z2vgwY4Eh0khy3k/ALWPncEvAQ2
# ted3y5wujSMYuaPCRx3wXdahc1cFaJqnyTdlHb7qvNhCg0MFpYumCf/RoZSmTqo9
# CfUFbLfSZFrYKiLCS53xOV5M3kg9mzSWmglfjv33sVKRzj+J9hyhtal1H3G/W0Nd
# ZT1QgW6r8NDT/LKzH7aZlib0PHmLXGTMze4nmuWgwAxyh8FuTVrTHurwROYybxzr
# F06Uw3hlIDsPQaof6aFBnf6xuKBlKjTg3qj5PObBMLvAoGMs/FwWAKjQxH/qEZ0e
# BsambTJdtDgJK0kHqv3sMNrxpy/Pt/360KOE2See+wFmd7lWEOEgbsausfm2usg1
# XTN2jvF8IAwqd661ogKGuinutFoAsYyr4/kKyVRd1LlqdJ69SK6YMYIELDCCBCgC
# AQEwgZEwfTELMAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3Rl
# cjEQMA4GA1UEBxMHU2FsZm9yZDEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSUw
# IwYDVQQDExxTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIENBAhA5TCXhfKBtJ6hl
# 4jvZHSLUMA0GCWCGSAFlAwQCAgUAoIIBazAaBgkqhkiG9w0BCQMxDQYLKoZIhvcN
# AQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTIzMDgxMzA3MzMyNFowPwYJKoZIhvcNAQkE
# MTIEMDlWrOhDL5cm4NEox7NrVFcDVH9MTCXpzen+z1c3oObzmJfkrZfR1JgPTtlV
# m6POXzCB7QYLKoZIhvcNAQkQAgwxgd0wgdowgdcwFgQUrmKvdQoMvUfWRh91aOK8
# jOfKT5QwgbwEFALWW5Xig3DBVwCV+oj5I92Tf62PMIGjMIGOpIGLMIGIMQswCQYD
# VQQGEwJVUzETMBEGA1UECBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENp
# dHkxHjAcBgNVBAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNF
# UlRydXN0IFJTQSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eQIQMA9vrN1mmHR8qUY2
# p3gtuTANBgkqhkiG9w0BAQEFAASCAgAfSxueb6tXyVf7vC+11z5mdpJI7KII45wL
# Ao+ha+1ykUto+7NdC+1bCt/VYztVNBUllUBgcpWVsSnovfiLRcST6yiXWl9CDlHO
# MiMz4fwoT56K//7dVabu6/LR5Ld5xlWmenHqpxiUpKGTBvmKITh0fjWy4KaOfZQ3
# N8WtbqgMsxc15lI7h2PsqjG+BdJPNOlEPXr/ac3EXqbllLXUmXqwH7WtF8d1LrwF
# fM+/iwCv8ES1kIlIM1VDGc1ku89r1cDFUF7qy5G1rMunqPfs2dwgGB0aQz1SL6sf
# r0pvsJmbKKHjKvR/CiqSa2wutcVdgfs68wZyUeFQQM4bWcPtDt8iLDgJR8kcUWye
# yIDyNJ3uvQD7oXCvenLsIEGQRT3jAACj7Jkgpv9j0bFOyDLGjcwwe80fdKX3f1Nz
# cCY4b5MOllh81ATIcHGyznggN9XhR4/w5hHSbyNhVWtbSxoyHYl1ymDzk9oGfMku
# VqSltC9Tz1tBYIp9r+K2tMui0WErMQdxpGB2yOZmfnxXxjPrAxM3L8JbMRt3JEeI
# bgnSiMxo9uETOASmyEZXw4Z7SERZxxA6gO93elR24ddIrkU6DZlB+q8qwyORGuaG
# StDtfQjxPWY4Gp0rWqomPTDjqmVl31xIWCMtRVAGLxsAqoZGdxLbqZQUaALKN4do
# 3RmCaup1cQ==
# SIG # End signature block
