# Copyright (C) 2021 Intel Corporation

<#
  .Synopsis 
    Invokes the Intel Active Management Technology force boot command using the One-Click Recovery boot options
  .Description
    This cmdlet invokes the Intel Active Management Technology force boot command using the One-Click Recovery boot options. 
	The available OCR boot options are: HTTPS server, PBA, and WinRE.
	The OCR feature is available on vPro SKUs, starting with Tiger Lake platforms using Intel CSME 15 firmware.
	This cmdlet flow includes:
		1.	Checks if selected OCR boot capability is supported.
		2.	Checks if selected OCR boot capability is enabled.
		3.	Displays available OCR sources for the selected option.
		4.	Sets next boot to OCR selected boot source.
		5.	Performs remote-control reset/ power on, to actually trigger the OCR boot.

  .Notes
    Supported AMT Firmware Versions: 15.0 or higher.
	The OCR commands are supported only over TLS connection. For running this cmdlet -TLS parameter is mandatory
    
   AMT Provisioning:
      The vPro client AMT firmware must be provisioned prior to accessing AMT functionality. This Cmdlet will fail if it is run against a vPro client that has not been provisioned.
        
    AMT Client Authentication:
      To invoke commands against AMT enabled clients credentials to authenticate must be specified. 
      When no credential is provided as a parameter, the script will use the local logged on Kerberos credential.
      When only the username (Kerberos or Digest) parameter is included the user will be prompted to provide the associated password.
      Credentials should be stored as a PowerShell variable then passed into the Cmdlet with the credential parameter.
      $AMTCredential = get-credential
     
    AMT Client Encryption:
      If the Intel vPro client has been configured to use TLS (a web server certificate has been issued to the Intel Management Engine) the Cmdlet must be called with a -TLS switch.

      When managing an Intel vPro client over TLS (Port 16993) it is important that the computername parameter matchs the primary subject name of the issued TLS certificate. Typically this is the fully qualified domain name (FQDN).

	  If Mutual TLS is desired, the Cmdlet must be called with -TLS switch and with a valid certificate name from the certificate store in the -CertificateName parameter.
    Status:
      Status output designates if the Cmdlet was run successfully. For failed attempts additional status may be provided.
  .Link
    http:\\vproexpert.com
    http:\\www.intel.com\vpro
    http:\\www.intel.com

  .Example
	Invoke-AMTForceBoot_OCR -ComputerName dut.vprodemo.com -Username admin -Password Admin!98 -TLS -BootSource WinRE
	
	PBA and WinRE Boot Sources:
	
	Index				 : 1
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 1
	BIOSBootString		 : OEM PBA
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:1
	
	Index				 : 2
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 2
	BIOSBootString		 : OEM PBA 1
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:2
	
	Index				 : 3
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 3
	BIOSBootString		 : OEM PBA 2
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:3
	
	Index				 : 4
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 4
	BIOSBootString		 : OEM PBA 3
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:4
	
	Index				 : 5
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 5
	BIOSBootString		 : OEM PBA 4
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:5
	
	Index				 : 6
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 6
	BIOSBootString		 : OEM PBA 5
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:6
	
	Index				 : 7
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 7
	BIOSBootString		 : OEM PBA 6
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:7
	
	Index				 : 8
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 8
	BIOSBootString		 : OEM PBA 7
	BootString			 : \netboot.xyz-snp.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:8
	
	Index				 : 9
	InstanceID			 : Intel(r) AMT: Force OCR UEFI Boot Option 9
	BIOSBootString		 : One Click RecoveryWinRe
	BootString			 : PciRoot(0x0)/Pci(0x6,0x0)/Pci(0x0, 0x0)/Sata(0x0,0xFFFF,0x0)/HD(1,GPT,476CEECB-C5B7-42A3-B628-73BOECF2222F,0x800,0x32000)/\EFI\Microsoft\Boot\bootmgfw.efi
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option:9
	
	Please enter selected boot source index: 9
	
	======================================================================
	Force Boot OCR Configuration is completed, performing reset/ Power On.
	======================================================================
	
	Performing Reset
	
	Power Action executed successfully. Return Value: 0
	
  .Example
	Invoke-AMTForceBoot_OCR -ComputerName dut.vprodemo.com -Username admin -Password Admin!98 -TLS -BootSource HTTPS -ParametersArray_path "C:\Users\Administrator\HTTPParamsArray.txt" -NumberOfParameters 4
	
	HTTP Boot sources:
	
	Index				 : 1
	InstanceID			 : Intel(r) AMT: Force OCR UEFI HTTPS Boot
	BIOSBootString		 :
	BootString			 :
	StructuredBootString : Intel(r)AMT:OCR-UEFI-Boot-Option-HTTPS:1
	
	Please enter selected boot source index: 1
	
	======================================================================
	Force Boot OCR Configuration is completed, performing reset/ Power On.
	======================================================================
	
	Performing Reset
	
	Power Action executed successfully. Return Value: 0
#>

Function Invoke-AMTForceBoot_OCR.ps1
{
	[CmdletBinding()]
	Param (
		[Parameter(Mandatory = $true, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $true, position = 0, HelpMessage = "Hostname, FQDN, or IP Address")] [string] $ComputerName,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 1, HelpMessage = "Valid Ports are 16992 (non-TLS) or 16993 (TLS)")][ValidateSet("16992", "16993")] [String] $Port,
		[Parameter(Mandatory = $true, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 2, HelpMessage = "Use TLS (Port 16993)")] [switch] $TLS,  
		[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Accept self-signed certificate for TLS connection (skip any certificate checks.)")] [switch] $AcceptSelfSignedCert,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 3, HelpMessage = "Name of certificate. (Use for mutual TLS)")] [string] $CertificateName,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 4, HelpMessage = "Digest or Kerberos User")] [string] $Username,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 5, HelpMessage = "Digest or Kerberos Password")] [string] $Password,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 6, HelpMessage = "PS Credential")] [System.Management.Automation.PSCredential] $Credential,
		[Parameter(Mandatory = $true, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 7, HelpMessage = "Source Boot can be one of the following: WinRE, HTTPS, PBA")][ValidateSet("WinRE", "HTTPS", "PBA")] [string] $BootSource,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 8, HelpMessage = "Path to a file which contains the UEFI boot parameter array encoded to base64")] [string] $ParametersArray_path,
		[Parameter(Mandatory = $false, ValueFromPipelineByPropertyName = $true, ValueFromPipeline = $false, position = 9, HelpMessage = "The number of parameters included in the UEFI boot parameter array")] [string] $NumberOfParameters

	)

	PROCESS
	{
		$Results = @()
		
		# Create a connection object
		$Connection = New-Object Intel.Management.Wsman.WsmanConnection 

		if ($Credential.username.Length -gt 0) {
			$Connection.SetCredentials($Credential.Username, $Credential.Password)  
		}
		elseif ($Username.length -gt 0) {
			if ($Password.length -gt 0) {
				$Connection.SetCredentials($Username, $Password)  
			}
			else {
				$Cred = Get-Credential $Username
				$Connection.SetCredentials($Cred.Username, $Cred.Password)
			}
		} 
			   
		if ($Port.Length -lt 1) {
			if ($TLS.IsPresent) {
				$Port = 16993;
			}
			else {
				$Port = 16992;
			}
		}

		if ($CertificateName.Length -gt 0) {
			$Connection.Options.SetClientCertificateByCertificateName($CertificateName)
		}
		
		if($AcceptSelfSignedCert.IsPresent)
		{
			$Connection.Options.AcceptSelfSignedCertificate=$true;			
		}

		# Attempt Connection with Client
		$Connection.SetHost($ComputerName, $Port)
		
		# Set local member with selected boot source type
		switch($BootSource)
		{
			"WinRE"
			{
				$BootSourceProperty = "ForceWinREBoot"
				$BootSourceEnabledProperty = "WinREBootEnabled"
				Break;
			}
			"HTTPS"
			{
				$BootSourceProperty = "ForceUEFIHTTPSBoot"
				$BootSourceEnabledProperty = "UEFIHTTPSBootEnabled"
				Break;
			}
			"PBA"
			{
				$BootSourceProperty = "ForceUEFIPBABoot"
				$BootSourceEnabledProperty = "UEFILocalPBABootEnabled"
				Break;
			}
		}
		

		try
		{	
			$Obj = new-object psobject
			
			# Check if boot capability is supported.
			$BootCapabilities_ref = $Connection.NewReference("SELECT * From AMT_BootCapabilities").Get()
			$BootSourceStatus = $BootCapabilities_ref.GetProperty($BootSourceProperty).ToString()
			
			if($BootSourceStatus -eq "false") # Boot capability unsupported
			{
				$Results += "$BootSource boot option unsupported.`n"
				$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Selected Boot option is not supported."
				$Results += $Obj
				Write-Output $Results
				return
			}
			else # Boot capability supported
			{
				# Check if boot option is enabled.
				$BootSettingData_ref = $Connection.NewReference("SELECT * From AMT_BootSettingData").Get()
				$BootOptionStatus = $BootSettingData_ref.GetProperty($BootSourceEnabledProperty).ToString()
				
				if($BootOptionStatus -eq "false") # Selected Boot option disabled.
				{
					$Results += "$BootSource boot option disabled.`n"
					$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Selected Boot option is disabled."
					$Results += $Obj
					Write-Output $Results
					return
				}
				else # Selected Boot option enabled.
				{
					#----------------------------------------
					#-------------- Start Boot --------------
					#----------------------------------------
					
					# Get the AMT boot settings
					$cimRef = $Connection.NewReference("SELECT * FROM AMT_BootSettingData")
					$cimObj = $cimRef.Get()
					
					# Set important parameters for the OCR boot
					$cimObj.SetProperty("UseIDER", "false")
					$cimObj.SetProperty("BIOSSetup", "false")
					$cimObj.SetProperty("UseSOL", "false")
					$cimObj.SetProperty("EnforceSecureBoot", "true")
					$cimObj.SetProperty("BootMediaIndex", "0")
					$cimObj.SetProperty("UserPasswordBypass", "false")
					
					if (-not $NumberOfParameters)
					{
						$NumberOfParameters = 0
					}
					# Set Parameter array
					
					if($ParametersArray_path){
						if(-not (Test-Path $ParametersArray_path))
						{
							Write-Output "Failed to read Parameter Array file, please check path.`n"
							return
						}
						
						$UefiBootParametersArray = Get-Content $ParametersArray_path
						
						# Validate array size is up to 1500 bytes
						# 1.333 is the factor because of the base64 encoding, (1.333 * 1024) + buffer
						if($UefiBootParametersArray.length -gt 1500){
							Write-Output "Array Length is too long. Array can contain up to 1500 bytes`n"
							return
						}
						
						# If array was inserted, number of parameters must be greater than 0
						if($UefiBootParametersArray -and ($NumberOfParameters -le 0))
						{
							Write-Output "If array was inserted, number of parameters must be greater than 0.`n"
							return
						}
						
						$cimObj.SetProperty("UefiBootParametersArray", "$UefiBootParametersArray") # Set UefiBootParametersArray
					}
					$cimObj.SetProperty("UefiBootNumberOfParams", "$numberOfParameters") # Number of OCR Parameters
					
					# Set the AMT boot settings
					$putResult = $cimRef.Put($cimObj)
										
					# Set the boot source
					$Sources = @()
					$i=0
					switch($BootSource)
					{
						"HTTPS" # HTTPS can only have 1 instance.
						{
							Write-Output "HTTPS Boot sources:`n"
							$InstanceObj = new-object psobject
							$SourceRef = $Connection.NewReference("SELECT * FROM CIM_BootSourceSetting")
							$SourceRef.AddSelector("InstanceID", "Intel(r) AMT: Force OCR UEFI HTTPS Boot")
							$SourceObj = $SourceRef.Get()
							$i++
							if(-not $SourceObj) # No boot source found fot HTTPS
							{
								Write-Output "No instances for $BootSource boot option.`n"
								return
							}
							
							$BIOSBootString = $SourceObj.GetProperty("BIOSBootString")
							$BootString = $SourceObj.GetProperty("BootString")
							$StructuredBootString = $SourceObj.GetProperty("StructuredBootString")
							
							$Sources += "Intel(r) AMT: Force OCR UEFI HTTPS Boot"
							$InstanceObj | Add-Member -MemberType noteproperty -Name "Index" -value "$i"
							$InstanceObj | Add-Member -MemberType noteproperty -Name "InstanceID" -value "Intel(r) AMT: Force OCR UEFI HTTPS Boot"
							$InstanceObj | Add-Member -MemberType noteproperty -Name "BIOSBootString" -value $BIOSBootString.ToString()
							$InstanceObj | Add-Member -MemberType noteproperty -Name "BootString" -value $BootString.ToString()
							$InstanceObj | Add-Member -MemberType noteproperty -Name "StructuredBootString" -value $StructuredBootString.ToString()
							
							Write-Output $InstanceObj
							$SelectedBootSourceIndex = Read-Host -Prompt 'Please enter selected boot source index'
							$SelectedBootSourceIndex = $SelectedBootSourceIndex -as[int]
							if(($SelectedBootSourceIndex -gt $i) -or ($SelectedBootSourceIndex -le 0))
							{
								Write-Output "Invalid index inserted.`n"
								return
							}
							$SelectedBootSource = $Sources[$SelectedBootSourceIndex -1]
						}
						{($_ -eq "WinRE") -or ($_ -eq "PBA")} # PBA and WinRE can have 1-10 instances
						{
							Write-Output "PBA and WinRE Boot sources:`n"
							
							$SourceRef_Enum = $Connection.NewReference("CIM_BootSourceSetting").Enumerate()
							foreach ($BootSettingInstance in $SourceRef_Enum) {
								$InstanceObj = new-object psobject
								$BootSetting_InstanceId = $BootSettingInstance.Object.GetProperty("InstanceID").ToString()
								if($BootSetting_InstanceId -match '[a-zA-Z]*: Force OCR UEFI Boot Option \d[0]?$'){
									$i++
									$Sources += $BootSetting_InstanceId
									
									$SourceObj = $BootSettingInstance.Object
									$BIOSBootString = $SourceObj.GetProperty("BIOSBootString")
									$BootString = $SourceObj.GetProperty("BootString")
									$StructuredBootString = $SourceObj.GetProperty("StructuredBootString")
							
									$InstanceObj | Add-Member -MemberType noteproperty -Name "Index" -value $i
									$InstanceObj | Add-Member -MemberType noteproperty -Name "InstanceID" -value $BootSetting_InstanceId
									$InstanceObj | Add-Member -MemberType noteproperty -Name "BIOSBootString" -value $BIOSBootString.ToString()
									$InstanceObj | Add-Member -MemberType noteproperty -Name "BootString" -value $BootString.ToString()
									$InstanceObj | Add-Member -MemberType noteproperty -Name "StructuredBootString" -value $StructuredBootString.ToString()
									
									Write-Output $InstanceObj
								}
							}
							
							if(-not $Sources) # No boot source found for PBA or WinRE
							{
								Write-Output "No instance for $BootSource boot option.`n"
								return
							}
							
							# Get boot source from user
							$SelectedBootSourceIndex = Read-Host -Prompt 'Please enter selected boot source index'
							$SelectedBootSourceIndex = $SelectedBootSourceIndex -as[int]
							if(($SelectedBootSourceIndex -gt $i) -or ($SelectedBootSourceIndex -le 0))
							{
								Write-Output "Invalid index inserted.`n"
								return
							}
							$SelectedBootSource = $Sources[$SelectedBootSourceIndex-1]
						}
					}
					
					# Change boot order
					$SourceRef = $Connection.NewReference("SELECT * FROM CIM_BootSourceSetting")
					$SourceRef.AddSelector("InstanceID", $SelectedBootSource)
					
					$objRef = $Connection.NewReference("SELECT * FROM CIM_BootConfigSetting")
					$objRef.AddSelector("InstanceID", 'Intel(r) AMT: Boot Configuration 0')
					
					$InputObj = $objRef.CreateMethodInput("ChangeBootOrder")
					$InputObj.SetProperty("Source", $SourceRef)
					
					
					$OutObj = $objRef.InvokeMethod($InputObj)
					if(-not ($OutObj.GetProperty("ReturnValue") -like "0"))
					{
						Write-Output "Change Boot Order - Failed`n" 
						return
					}
					
					# Set Boot Config Role
					$BootConfigSettingRef = $Connection.NewReference("SELECT * FROM CIM_BootConfigSetting")
					$BootConfigSettingRef.AddSelector("InstanceID", 'Intel(r) AMT: Boot Configuration 0')

					$objRef = $Connection.NewReference("CIM_BootService")
					$InputObj = $objref.CreateMethodInput("SetBootConfigRole")
					$InputObj.SetProperty("BootConfigSetting", $BootConfigSettingRef)
					$InputObj.SetProperty("Role", "1")
					$OutObj = $objRef.InvokeMethod($InputObj)
					if(-not ($OutObj.GetProperty("ReturnValue") -like "0"))
					{
						Write-Output "Set Boot Config Role - Failed.`n" 
						return
					}
					
					# Configuration complete, performing reset/ Power on.
					Write-Output "`n"$("=" * 70)
					Write-Output "Force Boot OCR Configuration is completed, performing reset/ Power On."
					Write-Output $("=" * 70)"`n"
					
					#----------------------------------------
					#------ Performing Reset/ Power On ------
					#----------------------------------------
					
					# Get a reference to the ManagedSystem (User of the Service)
					$UserRef = $Connection.NewReference("SELECT * from CIM_ComputerSystem WHERE Name='ManagedSystem'")

					# Get the Instance of CIM_AssociatedPowerManagmentService for the ManagedSystem
					$ObjRef = $Connection.NewReference("CIM_AssociatedPowerManagementService");
					$ObjRef.AddSelector("UserOfService", $userRef);
					$AssociatedObj = $ObjRef.Get()

					# Get current power state
					$PowerState = $AssociatedObj.GetProperty("PowerState").ToString()

					# If client in S4/S5 then power-on operation otherwise reset operation
					if (($PowerState -eq 8) -or ($PowerState -eq 7)) {
						$PowerInt = 2
						Write-Output "Performing Power ON`n"
					} 
					else {
						$PowerInt = 10
						Write-Output "Performing Reset`n"
					}
					
					# Now associate to the Provider of the Service (CIM_PowerManagementService)
					$ServiceRef = $AssociatedObj.GetProperty("ServiceProvided").Ref

					# Now invoke the RequestPowerStateChange
					$InputObj = $ServiceRef.CreateMethodInput("RequestPowerStateChange")
					$InputObj.SetProperty("PowerState", "$PowerInt")
					$InputObj.SetProperty("ManagedElement", $UserRef)
					$OutObj = $ServiceRef.InvokeMethod($InputObj);

					# Note Return Status
					$ReturnValue = $OutObj.GetProperty("ReturnValue").ToString()
					
					if ($ReturnValue -eq 0) {
						Write-Output "Power Action executed successfully. Return Value: $ReturnValue`n"
					}
					else {
						Write-Output "Power Action Failed. Return Value: $ReturnValue`n"
					}
				}

			}
		}
		catch {
			Write-Output "Exception thrown."
			Write-Output $_.ToString()
		}
		
		# Set back to false for the next cmdlet, if PS is still open
		$Connection.Options.AcceptSelfSignedCertificate=$false;

	}
}
# SIG # Begin signature block
# MIIogwYJKoZIhvcNAQcCoIIodDCCKHACAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDCR7upwnM/wEqN
# AaVLAKsE99w02fEYdVf4eujkzHQgpqCCEfIwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYcMIIEBKADAgECAhAz1wio
# kUBTGeKlu9M5ua1uMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLjAs
# BgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBFViBSMzYwggGi
# MA0GCSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQC70f4et0JbePWQp64sg/GNIdMw
# hoV739PN2RZLrIXFuwHP4owoEXIEdiyBxasSekBKxRDogRQ5G19PB/YwMDB/NSXl
# wHM9QAmU6Kj46zkLVdW2DIseJ/jePiLBv+9l7nPuZd0o3bsffZsyf7eZVReqskmo
# PBBqOsMhspmoQ9c7gqgZYbU+alpduLyeE9AKnvVbj2k4aOqlH1vKI+4L7bzQHkND
# brBTjMJzKkQxbr6PuMYC9ruCBBV5DFIg6JgncWHvL+T4AvszWbX0w1Xn3/YIIq62
# 0QlZ7AGfc4m3Q0/V8tm9VlkJ3bcX9sR0gLqHRqwG29sEDdVOuu6MCTQZlRvmcBME
# Jd+PuNeEM4xspgzraLqVT3xE6NRpjSV5wyHxNXf4T7YSVZXQVugYAtXueciGoWnx
# G06UE2oHYvDQa5mll1CeHDOhHu5hiwVoHI717iaQg9b+cYWnmvINFD42tRKtd3V6
# zOdGNmqQU8vGlHHeBzoh+dYyZ+CcblSGoGSgg8sCAwEAAaOCAWMwggFfMB8GA1Ud
# IwQYMBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBSBMpJBKyjNRsjE
# osYqORLsSKk/FDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADAT
# BgNVHSUEDDAKBggrBgEFBQcDAzAaBgNVHSAEEzARMAYGBFUdIAAwBwYFZ4EMAQMw
# SwYDVR0fBEQwQjBAoD6gPIY6aHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UHVibGljQ29kZVNpZ25pbmdSb290UjQ2LmNybDB7BggrBgEFBQcBAQRvMG0wRgYI
# KwYBBQUHMAKGOmh0dHA6Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0Nv
# ZGVTaWduaW5nUm9vdFI0Ni5wN2MwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMA0GCSqGSIb3DQEBDAUAA4ICAQBfNqz7+fZyWhS38Asd3tj9lwHS
# /QHumS2G6Pa38Dn/1oFKWqdCSgotFZ3mlP3FaUqy10vxFhJM9r6QZmWLLXTUqwj3
# ahEDCHd8vmnhsNufJIkD1t5cpOCy1rTP4zjVuW3MJ9bOZBHoEHJ20/ng6SyJ6UnT
# s5eWBgrh9grIQZqRXYHYNneYyoBBl6j4kT9jn6rNVFRLgOr1F2bTlHH9nv1HMePp
# GoYd074g0j+xUl+yk72MlQmYco+VAfSYQ6VK+xQmqp02v3Kw/Ny9hA3s7TSoXpUr
# OBZjBXXZ9jEuFWvilLIq0nQ1tZiao/74Ky+2F0snbFrmuXZe2obdq2TWauqDGIgb
# MYL1iLOUJcAhLwhpAuNMu0wqETDrgXkG4UGVKtQg9guT5Hx2DJ0dJmtfhAH2KpnN
# r97H8OQYok6bLyoMZqaSdSa+2UA1E2+upjcaeuitHFFjBypWBmztfhj24+xkc6Zt
# CDaLrw+ZrnVrFyvCTWrDUUZBVumPwo3/E3Gb2u2e05+r5UWmEsUUWlJBl6MGAAjF
# 5hzqJ4I8O9vmRsTvLQA1E802fZ3lqicIBczOwDYOSxlP0GOabb/FKVMxItt1UHeG
# 0PL4au5rBhs+hSMrl8h+eplBDN1Yfw6owxI9OjWb4J0sjBeBVESoeh2YnZZ/WVim
# VGX/UUIL+Efrz/jlvzCCBlswggTDoAMCAQICEHAQMvP0AIEdrcUef3h12LwwDQYJ
# KoZIhvcNAQELBQAwVzELMAkGA1UEBhMCR0IxGDAWBgNVBAoTD1NlY3RpZ28gTGlt
# aXRlZDEuMCwGA1UEAxMlU2VjdGlnbyBQdWJsaWMgQ29kZSBTaWduaW5nIENBIEVW
# IFIzNjAeFw0yMzA0MDQwMDAwMDBaFw0yNDA0MDMyMzU5NTlaMIG7MRAwDgYDVQQF
# EwcyMTg5MDc0MRMwEQYLKwYBBAGCNzwCAQMTAlVTMRkwFwYLKwYBBAGCNzwCAQIT
# CERlbGF3YXJlMR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjELMAkGA1UE
# BhMCVVMxEzARBgNVBAgMCkNhbGlmb3JuaWExGjAYBgNVBAoMEUludGVsIENvcnBv
# cmF0aW9uMRowGAYDVQQDDBFJbnRlbCBDb3Jwb3JhdGlvbjCCAaIwDQYJKoZIhvcN
# AQEBBQADggGPADCCAYoCggGBANO3BjDeeK28E+zFxToyp77/6fB+Mp6913C+9t1U
# JguD/sBVnxljfvQScWzRrvF0chAOz5rjlnvUTXUFsgvJ4XFIZQtG/Rfyb4qvQbFE
# vdoS1kFlMAtXnMuuDSMbncPju4R/5gLfMzHFLRpHOoUmeRFlwEkyAYVwjDvH+OcR
# WW2Vm72ggaVeqgW5kIx3R4VQSXgJ3ToPP/49/9YjEwhM0yLccTPJKVB/6unAJaY6
# /j15gi4owbo6K88625JlZTy7drEhZvvI9vlStkPUOxbHQmU72QG7bF1ofao/4lTH
# pE6RAXTppoWlu24kRhLcXaIIAitMDqs9E0SFdV2qcAxc7d38v0l2sds/sy2EWMU9
# RZAjJD4fH/Bm0hJVm5937cq/sf20lucUQnu96+5zVHXSIuD65dkQ/6aoGxwZty49
# rdTFY494ri++yfw6NWrbQkIoc2c1HG0jHRAV6m3dgzwY0104g/BpZkLogT4EEEO5
# XRm6k1PSMxoM194ytHU+pJ2B2QIDAQABo4IBvDCCAbgwHwYDVR0jBBgwFoAUgTKS
# QSsozUbIxKLGKjkS7EipPxQwHQYDVR0OBBYEFJLGD4aF5QaeHQosntNMcCm3IDAZ
# MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMEkGA1UdIARCMEAwNQYMKwYBBAGyMQECAQYBMCUwIwYIKwYBBQUHAgEWF2h0
# dHBzOi8vc2VjdGlnby5jb20vQ1BTMAcGBWeBDAEDMEsGA1UdHwREMEIwQKA+oDyG
# Omh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVTaWduaW5n
# Q0FFVlIzNi5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsGAQUFBzAChjpodHRwOi8v
# Y3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmluZ0NBRVZSMzYu
# Y3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTAuBgNVHREE
# JzAloCMGCCsGAQUFBwgDoBcwFQwTVVMtREVMQVdBUkUtMjE4OTA3NDANBgkqhkiG
# 9w0BAQsFAAOCAYEAttRv5ICWB9YKIXk6xtQ385RQks/95gKJP+T62x8e2rHfQXY4
# GQChplzrceWnolz+TBJF1Dt8HYLUjpj0tgQHcU1mT54Lz4vt8eRHfldjRUC3I+Sy
# FxfWncIWYru27DPKAUEVmCHNF6++myfE7FLTFsL2ogZhZF1sUIBAvDd7d3j0wtz4
# N4d9MhjIfncOOqTOctwEbLtSTX9xIOry4ZgsDVs/QWiqyMyUaG649bcWWXDWxIq+
# 6jtY4WJn4KQUhPVzP53e45Y7Leb+69GCSkfQ5cEZlSOZq2PeFRqP3PwVQ946hbm8
# saXEl3iaZ8s5YY2JorFvstkqYXiQ9O55wAZ4fTeyhWjVNpkqDiIHpO+UYREJ5AHG
# nsZEJJkx8E/Y+zX7YuDMJmV7+WLI1CvIyCLoNhUMupfReykiQKD1P+CCe6UzSOey
# mSrh0X/XpOcDz/+sSgUIziD5HUuMYp0czD4Up/FFrl3qX3te9N4hQt74Ifh35DLR
# g/DdRlqPI70u7jl5MYIV5zCCFeMCAQEwazBXMQswCQYDVQQGEwJHQjEYMBYGA1UE
# ChMPU2VjdGlnbyBMaW1pdGVkMS4wLAYDVQQDEyVTZWN0aWdvIFB1YmxpYyBDb2Rl
# IFNpZ25pbmcgQ0EgRVYgUjM2AhBwEDLz9ACBHa3FHn94ddi8MA0GCWCGSAFlAwQC
# AQUAoHwwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIE
# IAZXEBOKbRYpszvcg5rHI+fvw9P3QNFLe4DjfV5b11FfMA0GCSqGSIb3DQEBAQUA
# BIIBgKFH3s/uq5Nmj0/kFkqqkpndUUrXuZO/zNYiHObtObsjvtjektrrH4PcsPA7
# rb6Wo/zINZK32jOAlZW77hmRx7PrZCr18ZFGnDT6hEPlBNMksLILXCHx5+IyCWOL
# NHTQZoMUSvX2IlLn4r+mcjmh4NoCBVinSHf4r4BgWSRXyiJFzDwGpj4dYWSPD/pU
# R+IcfAeF3+FCUVcCFj2axiKcG68CY0RAX+QWX/Cy1v24xkgupNxSI3IbFRwKZw2L
# DQVS/ZIjZZKq6xstFLCljMiE/fmbvDW6+nXVT+jsX55xv0E4XhvL2+ZTRlpUGCQB
# N8BCIW27mhN0mJE68W6Uj2yOcklM/EVsnKWv/RXEOi2M8KrJPrRRI5wD1nbyKh0R
# 1h0NAW1JfZL6Orgnst49JFVm5HuvdhsmO7snoW5OU0yd1+WNasK4TKs3lQvsptU8
# pcaoQ0hdWY16OEu4NaYcDVJ9CfwtRJeOfdmoogVQnwtysCdoYXVWdvm/fdSYeg4I
# E6bRtKGCE08wghNLBgorBgEEAYI3AwMBMYITOzCCEzcGCSqGSIb3DQEHAqCCEygw
# ghMkAgEDMQ8wDQYJYIZIAWUDBAICBQAwgfAGCyqGSIb3DQEJEAEEoIHgBIHdMIHa
# AgEBBgorBgEEAbIxAgEBMDEwDQYJYIZIAWUDBAIBBQAEILOHlHDTmBufLxc8qrM8
# /6qBvltWUSTtbBNf4SDv0G5lAhUAlSZg6tbNYSWlkjkkZ7q0AFZNJAsYDzIwMjMw
# ODEzMDczMzI4WqBupGwwajELMAkGA1UEBhMCR0IxEzARBgNVBAgTCk1hbmNoZXN0
# ZXIxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDEsMCoGA1UEAwwjU2VjdGlnbyBS
# U0EgVGltZSBTdGFtcGluZyBTaWduZXIgIzSggg3pMIIG9TCCBN2gAwIBAgIQOUwl
# 4XygbSeoZeI72R0i1DANBgkqhkiG9w0BAQwFADB9MQswCQYDVQQGEwJHQjEbMBkG
# A1UECBMSR3JlYXRlciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUg
# U3RhbXBpbmcgQ0EwHhcNMjMwNTAzMDAwMDAwWhcNMzQwODAyMjM1OTU5WjBqMQsw
# CQYDVQQGEwJHQjETMBEGA1UECBMKTWFuY2hlc3RlcjEYMBYGA1UEChMPU2VjdGln
# byBMaW1pdGVkMSwwKgYDVQQDDCNTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIFNp
# Z25lciAjNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAKSTKFJLzyeH
# dqQpHJk4wOcO1NEc7GjLAWTkis13sHFlgryf/Iu7u5WY+yURjlqICWYRFFiyuiJb
# 5vYy8V0twHqiDuDgVmTtoeWBIHIgZEFsx8MI+vN9Xe8hmsJ+1yzDuhGYHvzTIAhC
# s1+/f4hYMqsws9iMepZKGRNcrPznq+kcFi6wsDiVSs+FUKtnAyWhuzjpD2+pWpqR
# KBM1uR/zPeEkyGuxmegN77tN5T2MVAOR0Pwtz1UzOHoJHAfRIuBjhqe+/dKDcxIU
# m5pMCUa9NLzhS1B7cuBb/Rm7HzxqGXtuuy1EKr48TMysigSTxleGoHM2K4GX+hub
# foiH2FJ5if5udzfXu1Cf+hglTxPyXnypsSBaKaujQod34PRMAkjdWKVTpqOg7RmW
# ZRUpxe0zMCXmloOBmvZgZpBYB4DNQnWs+7SR0MXdAUBqtqgQ7vaNereeda/TpUsY
# oQyfV7BeJUeRdM11EtGcb+ReDZvsdSbu/tP1ki9ShejaRFEqoswAyodmQ6MbAO+i
# tZadYq0nC/IbSsnDlEI3iCCEqIeuw7ojcnv4VO/4ayewhfWnQ4XYKzl021p3AtGk
# +vXNnD3MH65R0Hts2B0tEUJTcXTC5TWqLVIS2SXP8NPQkUMS1zJ9mGzjd0HI/x8k
# VO9urcY+VXvxXIc6ZPFgSwVP77kv7AkTAgMBAAGjggGCMIIBfjAfBgNVHSMEGDAW
# gBQaofhhGSAPw0F3RSiO0TVfBhIEVTAdBgNVHQ4EFgQUAw8xyJEqk71j89FdTaQ0
# D9KVARgwDgYDVR0PAQH/BAQDAgbAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAww
# CgYIKwYBBQUHAwgwSgYDVR0gBEMwQTA1BgwrBgEEAbIxAQIBAwgwJTAjBggrBgEF
# BQcCARYXaHR0cHM6Ly9zZWN0aWdvLmNvbS9DUFMwCAYGZ4EMAQQCMEQGA1UdHwQ9
# MDswOaA3oDWGM2h0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVT
# dGFtcGluZ0NBLmNybDB0BggrBgEFBQcBAQRoMGYwPwYIKwYBBQUHMAKGM2h0dHA6
# Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1JTQVRpbWVTdGFtcGluZ0NBLmNydDAj
# BggrBgEFBQcwAYYXaHR0cDovL29jc3Auc2VjdGlnby5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAEybZVj64HnP7xXDMm3eM5Hrd1ji673LSjx13n6UbcMixwSV32VpYRMM
# 9gye9YkgXsGHxwMkysel8Cbf+PgxZQ3g621RV6aMhFIIRhwqwt7y2opF87739i7E
# fu347Wi/elZI6WHlmjl3vL66kWSIdf9dhRY0J9Ipy//tLdr/vpMM7G2iDczD8W69
# IZEaIwBSrZfUYngqhHmo1z2sIY9wwyR5OpfxDaOjW1PYqwC6WPs1gE9fKHFsGV7C
# g3KQruDG2PKZ++q0kmV8B3w1RB2tWBhrYvvebMQKqWzTIUZw3C+NdUwjwkHQepY7
# w0vdzZImdHZcN6CaJJ5OX07Tjw/lE09ZRGVLQ2TPSPhnZ7lNv8wNsTow0KE9SK16
# ZeTs3+AB8LMqSjmswaT5qX010DJAoLEZKhghssh9BXEaSyc2quCYHIN158d+S4RD
# zUP7kJd2KhKsQMFwW5kKQPqAbZRhe8huuchnZyRcUI0BIN4H9wHU+C4RzZ2D5fjK
# JRxEPSflsIZHKgsbhHZ9e2hPjbf3E7TtoC3ucw/ZELqdmSx813UfjxDElOZ+JOWV
# SoiMJ9aFZh35rmR2kehI/shVCu0pwx/eOKbAFPsyPfipg2I2yMO+AIccq/pKQhyJ
# A9z1XHxw2V14Tu6fXiDmCWp8KwijSPUV/ARP380hHHrl9Y4a1LlAMIIG7DCCBNSg
# AwIBAgIQMA9vrN1mmHR8qUY2p3gtuTANBgkqhkiG9w0BAQwFADCBiDELMAkGA1UE
# BhMCVVMxEzARBgNVBAgTCk5ldyBKZXJzZXkxFDASBgNVBAcTC0plcnNleSBDaXR5
# MR4wHAYDVQQKExVUaGUgVVNFUlRSVVNUIE5ldHdvcmsxLjAsBgNVBAMTJVVTRVJU
# cnVzdCBSU0EgQ2VydGlmaWNhdGlvbiBBdXRob3JpdHkwHhcNMTkwNTAyMDAwMDAw
# WhcNMzgwMTE4MjM1OTU5WjB9MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3JlYXRl
# ciBNYW5jaGVzdGVyMRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0aWdv
# IExpbWl0ZWQxJTAjBgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgQ0Ew
# ggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDIGwGv2Sx+iJl9AZg/IJC9
# nIAhVJO5z6A+U++zWsB21hoEpc5Hg7XrxMxJNMvzRWW5+adkFiYJ+9UyUnkuyWPC
# E5u2hj8BBZJmbyGr1XEQeYf0RirNxFrJ29ddSU1yVg/cyeNTmDoqHvzOWEnTv/M5
# u7mkI0Ks0BXDf56iXNc48RaycNOjxN+zxXKsLgp3/A2UUrf8H5VzJD0BKLwPDU+z
# kQGObp0ndVXRFzs0IXuXAZSvf4DP0REKV4TJf1bgvUacgr6Unb+0ILBgfrhN9Q0/
# 29DqhYyKVnHRLZRMyIw80xSinL0m/9NTIMdgaZtYClT0Bef9Maz5yIUXx7gpGaQp
# L0bj3duRX58/Nj4OMGcrRrc1r5a+2kxgzKi7nw0U1BjEMJh0giHPYla1IXMSHv2q
# yghYh3ekFesZVf/QOVQtJu5FGjpvzdeE8NfwKMVPZIMC1Pvi3vG8Aij0bdonigbS
# lofe6GsO8Ft96XZpkyAcSpcsdxkrk5WYnJee647BeFbGRCXfBhKaBi2fA179g6JT
# Z8qx+o2hZMmIklnLqEbAyfKm/31X2xJ2+opBJNQb/HKlFKLUrUMcpEmLQTkUAx4p
# +hulIq6lw02C0I3aa7fb9xhAV3PwcaP7Sn1FNsH3jYL6uckNU4B9+rY5WDLvbxhQ
# iddPnTO9GrWdod6VQXqngwIDAQABo4IBWjCCAVYwHwYDVR0jBBgwFoAUU3m/Wqor
# Ss9UgOHYm8Cd8rIDZsswHQYDVR0OBBYEFBqh+GEZIA/DQXdFKI7RNV8GEgRVMA4G
# A1UdDwEB/wQEAwIBhjASBgNVHRMBAf8ECDAGAQH/AgEAMBMGA1UdJQQMMAoGCCsG
# AQUFBwMIMBEGA1UdIAQKMAgwBgYEVR0gADBQBgNVHR8ESTBHMEWgQ6BBhj9odHRw
# Oi8vY3JsLnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQ2VydGlmaWNhdGlvbkF1
# dGhvcml0eS5jcmwwdgYIKwYBBQUHAQEEajBoMD8GCCsGAQUFBzAChjNodHRwOi8v
# Y3J0LnVzZXJ0cnVzdC5jb20vVVNFUlRydXN0UlNBQWRkVHJ1c3RDQS5jcnQwJQYI
# KwYBBQUHMAGGGWh0dHA6Ly9vY3NwLnVzZXJ0cnVzdC5jb20wDQYJKoZIhvcNAQEM
# BQADggIBAG1UgaUzXRbhtVOBkXXfA3oyCy0lhBGysNsqfSoF9bw7J/RaoLlJWZAp
# bGHLtVDb4n35nwDvQMOt0+LkVvlYQc/xQuUQff+wdB+PxlwJ+TNe6qAcJlhc87QR
# D9XVw+K81Vh4v0h24URnbY+wQxAPjeT5OGK/EwHFhaNMxcyyUzCVpNb0llYIuM1c
# fwGWvnJSajtCN3wWeDmTk5SbsdyybUFtZ83Jb5A9f0VywRsj1sJVhGbks8VmBvbz
# 1kteraMrQoohkv6ob1olcGKBc2NeoLvY3NdK0z2vgwY4Eh0khy3k/ALWPncEvAQ2
# ted3y5wujSMYuaPCRx3wXdahc1cFaJqnyTdlHb7qvNhCg0MFpYumCf/RoZSmTqo9
# CfUFbLfSZFrYKiLCS53xOV5M3kg9mzSWmglfjv33sVKRzj+J9hyhtal1H3G/W0Nd
# ZT1QgW6r8NDT/LKzH7aZlib0PHmLXGTMze4nmuWgwAxyh8FuTVrTHurwROYybxzr
# F06Uw3hlIDsPQaof6aFBnf6xuKBlKjTg3qj5PObBMLvAoGMs/FwWAKjQxH/qEZ0e
# BsambTJdtDgJK0kHqv3sMNrxpy/Pt/360KOE2See+wFmd7lWEOEgbsausfm2usg1
# XTN2jvF8IAwqd661ogKGuinutFoAsYyr4/kKyVRd1LlqdJ69SK6YMYIELDCCBCgC
# AQEwgZEwfTELMAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3Rl
# cjEQMA4GA1UEBxMHU2FsZm9yZDEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSUw
# IwYDVQQDExxTZWN0aWdvIFJTQSBUaW1lIFN0YW1waW5nIENBAhA5TCXhfKBtJ6hl
# 4jvZHSLUMA0GCWCGSAFlAwQCAgUAoIIBazAaBgkqhkiG9w0BCQMxDQYLKoZIhvcN
# AQkQAQQwHAYJKoZIhvcNAQkFMQ8XDTIzMDgxMzA3MzMyOFowPwYJKoZIhvcNAQkE
# MTIEMAEOQtDsbW0BqeIdl90+KM+/+7CJjga8LBKsFq/ATTwxSNN+ZZsRZY4sr550
# 0zcyHDCB7QYLKoZIhvcNAQkQAgwxgd0wgdowgdcwFgQUrmKvdQoMvUfWRh91aOK8
# jOfKT5QwgbwEFALWW5Xig3DBVwCV+oj5I92Tf62PMIGjMIGOpIGLMIGIMQswCQYD
# VQQGEwJVUzETMBEGA1UECBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENp
# dHkxHjAcBgNVBAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNF
# UlRydXN0IFJTQSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eQIQMA9vrN1mmHR8qUY2
# p3gtuTANBgkqhkiG9w0BAQEFAASCAgCY/3UTWmTd8xkQHXjkioX94MwtxyReEk9+
# yVRbvElsW0jK4qZeJO4HNyeRmxTk/M6lmQdMeUxwgya3nsVcHU/eQieNGzspJvoY
# cYA/dFW9xLQ7GrXCH3Q7aSPq41KuPQc2KMegUpJllsS2guFqo5uU/zUXXrD2Ib6x
# bUymQr32/lB4c/c7llaZcqNbZ2OWoo8hakjBavywZa2KtHnofirMnXGtvzVdasju
# S+u6T7/buOWXL+ZSSwyMI97HVCXmQAyYGCnkT5s+G8mFmrOX3lhLHvjaCQak16FK
# DySkcfJFySf2djyikdEKIuUEQTRT9ezU+I0jzAXiMbgSi0wnkWy3Qmy4pkY6Bp3I
# mX+NMXuPoWqpP7w2YxArEw1LYc5u6cw5lW3hg1fWu+OLxVee0Yon/TYwn9HXGMUy
# o/wJChu+fGqf9loN54u8VLyGEKGI0W8la2aMq5dlZ97ZfnhyUX2Tb1tQUXW5oHVx
# gaBh7Hmysvlejqvkyigjl1YkGs1e4SArarVV/a98AxynOv3sLKkUZoa0mnqBVTZf
# Y8I/jAzmE52gsb9bYX7FTGGSTd0aIchriwVK5N5d0mq7Ox6M7Ie3tLI4YwpybTdS
# ikMAtUe9wn/uxflJz4d1iQXtGp6Em+ok5MQoDLv4fWbPVg777SCWHxcd2+NY3Vok
# eeJDvC3OKA==
# SIG # End signature block
