﻿# Copyright (C) 2007 Intel Corporation

Function Set-AMTSystemDefense {
<#
  .Synopsis 
    Enables Intel Active Management Technology System Defense 
  .Description
    This CmdLet disables network access using Intel Active Management Technology (AMT) System Defense from clients that have Intel AMT Firmware version 3.0 and Higher.
  .Notes
    Supported AMT Firmware Versions: 3.0 and higher

   AMT Provisioning:
      The vPro client AMT firmware must be provisioned prior to accessing AMT functionality. This Cmdlet will fail if it is run against a vPro client that has not been provisioned.
        
    AMT Client Authentication:
      To invoke commands against AMT enabled clients credentials to authenticate must be specified. 
      When no credential is provided as a parameter, the script will use the local logged on Kerberos credential.
      When only the username (Kerberos or Digest) parameter is included the user will be prompted to provide the associated password.
      Credentials should be stored as a PowerShell variable then passed into the Cmdlet with the credential parameter.
      $AMTCredential = get-credential
     
    AMT Client Encryption:
      If the Intel vPro client has been configured to use TLS (a web server certificate has been issued to the Intel Management Engine) the Cmdlet must be called with a -TLS switch.

      When managing an Intel vPro client over TLS (Port 16993) it is important that the computername parameter matchs the primary subject name of the issued TLS certificate. Typically this is the fully qualified domain name (FQDN).

    Status:
      Status output designates if the Cmdlet was run successfully. For failed attempts additional status may be provided.
  .Link
    http:\\vproexpert.com
    http:\\www.intel.com\vpro
    http:\\www.intel.com

  .Example
    Set-AMTSystemDefense vproclient.vprodemo.com -TLS

    ComputerName                  Port                          Status                        EnabledOnInterfaces
    ------------                  ----                          ------                        -------------------
    vproclient.vprodemo.com       16993                         Successful                    Wireless1, Wired                 
  .Example
    Set-AMTSystemDefense vproclient 16992  -username:amtuser

    Will prompt for digest username password.

    ComputerName                  Port                          Status                        EnabledOnInterfaces
    ------------                  ----                          ------                        -------------------
    vproclient                    16992                         Successful                    Wireless1, Wired0
  .Example
    Get-Content computers.txt | Set-AMTSystemDefense -TLS

    Will pull the list of amt clients from a text file and pipe them into Set-AMTSystemDefense

    ComputerName                  Port                          Status                        EnabledOnInterfaces
    ------------                  ----                          ------                        -------------------
    Computer1.vprodemo.com        16993                         Successful                    Wireless1, Wired0
    Computer2.vprodemo.com        16993                         Successful                    Wired0
  .Example
    Set-AMTSystemDefense vproclient.vprodemo.com -TLS -XMLConfig xmlfile.xml


    Here is the outline of the xml format, see 'System Defense' in the AMT SDK for more details.
    http://software.intel.com/sites/manageability/AMT_Implementation_and_Reference_Guide/default.htm
    
    <?xml version="1.0"?>
<SystemDefensePolicySet>
<ArrayOfFilter xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <Filter xsi:type="AMT_Hdr8021Filter">
    <Name/>
    <PolicyName/>
    <CreationClassName/>
    <SystemName/>
    <SystemCreationClassName/>
    <FilterProfile/>
    <FilterDirection/>
    <ActionEventOnMatch/>
    <HdrProtocolID8021/>
  </Filter>
  <Filter xsi:type="AMT_IPHeadersFilter">
    <Name/>
    <PolicyName/>
    <CreationClassName/>
    <SystemName/>
    <SystemCreationClassName/>
    <HdrIPVersion/>
    <HdrProtocolID/>
    <FilterProfile/>
    <FilterDirection/>
    <ActionEventOnMatch/>
    <HdrDestAddress/>
    <HdrDestMask/>
    <HdrDestPortStart/>
    <HdrDestPortEnd/>
  </Filter>
</ArrayOfFilter>
<ArrayOfPolicies xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
  <Policy xsi:type="AMT_systemDefensePolicy">
    <PolicyName/>
      <PolicyPrecedence/>
      <AntiSpoofingSupport/>
      <TxDefaultDrop/>
      <TxDefaultMatchEvent/>
      <TxDefaultCount/>
      <RxDefaultDrop/>
      <RxDefaultMatchEvent/>
      <RxDefaultCount/>
      <Active/>
   </Policy>
</ArrayOfPolicies>
</SystemDefensePolicySet>

#>

[CmdletBinding()]
Param (
	[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$true, position=0, HelpMessage="Hostname, FQDN, or IP Address")] [String[]] $ComputerName,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Valid Ports are 16992 (non-TLS) or 16993 (TLS)")][ValidateSet("16992", "16993")] [String] $Port,
    [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Use TLS (Port 16993)")] [switch] $TLS,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Accept self-signed certificate for TLS connection (skip any certificate checks.)")] [switch] $AcceptSelfSignedCert,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest of Kerberos User")] [string] $Username,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Digest of Kerberos Password")] [string] $Password,
	[Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="PS Credential")] [System.Management.Automation.PSCredential] $Credential,
    [Parameter(Mandatory=$false,ValueFromPipelineByPropertyName=$true,ValueFromPipeline=$false, HelpMessage="Path to XMl configuration file")] [string] $XMLConfig
) 

PROCESS {
$defaultXML = @'
<?xml version="1.0"?>
<SystemDefensePolicySet>
<ArrayOfFilter xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
 <Filter xsi:type="AMT_Hdr8021Filter">
    <Name>defaultBlock</Name>
    <PolicyName>defaultPolicy</PolicyName>
    <CreationClassName>n/a</CreationClassName>
    <SystemName>n/a</SystemName>
    <SystemCreationClassName>n/a</SystemCreationClassName>
    <FilterProfile>1</FilterProfile>
    <FilterDirection>0</FilterDirection>
    <ActionEventOnMatch>false</ActionEventOnMatch>
    <HdrProtocolID8021>2048</HdrProtocolID8021>
  </Filter> 
</ArrayOfFilter>
<ArrayOfPolicies xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
	<Policy xsi:type="AMT_systemDefensePolicy">
    	<PolicyName>defaultPolicy</PolicyName>
		<PolicyPrecedence>0</PolicyPrecedence>
		<AntiSpoofingSupport>3</AntiSpoofingSupport>
		<TxDefaultDrop>false</TxDefaultDrop>
		<TxDefaultMatchEvent>false</TxDefaultMatchEvent>
		<TxDefaultCount>true</TxDefaultCount>
		<RxDefaultDrop>false</RxDefaultDrop>
		<RxDefaultMatchEvent>true</RxDefaultMatchEvent>
		<RxDefaultCount>false</RxDefaultCount>
		<Active>true</Active>
	</Policy>
</ArrayOfPolicies>
</SystemDefensePolicySet>
'@    

	if($XMLConfig) {
		[xml]$data = get-content $XMLConfig
	} else {
	  [xml]$data = $defaultXML
	}

	$filterList = $data.selectnodes("/SystemDefensePolicySet/ArrayOfFilter/Filter")


	$Results = @()
	
	try
	{
		$Obj = new-object psobject
		$Connection = New-Object Intel.Management.Wsman.WsmanConnection 

		if ($credential.username.Length -gt 0)
		{
		  $Connection.SetCredentials($credential.Username, $credential.Password)  
		}
		elseif ($username.length -gt 0)
		{
		  if ($password.length -gt 0)
		  {
			$Connection.SetCredentials($username, $password)  
		  }
		  else
		  {
			$cred = Get-Credential $username
			$Connection.SetCredentials($cred.Username, $cred.Password)
		  }
		}
	 
		if ($Port.Length -lt 1) {
			if ($TLS.IsPresent) {
			  $Port = 16993;
			}
			else {
			  $Port = 16992;
			}
		}
		
		if($AcceptSelfSignedCert.IsPresent)
		{
			$Connection.Options.AcceptSelfSignedCertificate=$true;			
		}


		Foreach ($comp in $ComputerName)
		{
		  $Connection.SetHost($comp, $port)

		  
		  # Enable  
		  $Obj | Add-Member -MemberType noteproperty -Name ComputerName -value $Comp
		  $Obj | Add-Member -MemberType noteproperty -Name Port -value $port
		  $EnabledInterfaces = ""  
			
		  $cport = $Connection.NewReference("http://schemas.dmtf.org/wbem/wscim/1/cim-schema/2/CIM_EthernetPort")
		  try
		  {
			$cportEnumeration = $cport.Enumerate($null, $null)
		  }
		  catch
		  {
			$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Failed"
			$Obj | Add-Member -MemberType noteproperty -Name "EnabledOnInterfaces" -value ""
			$Results += $Obj  
			continue 
		  }
		   
		  Write-Output $Results

		  $filterTable = @{}

		  foreach($filter in $filterList)
		  {
			if($filter.type -eq "AMT_Hdr8021Filter")
			{
			  $AMT_Hdr8021Filter = $Connection.NewInstance("http://intel.com/wbem/wscim/1/amt-schema/1/AMT_Hdr8021Filter")
			  $AMT_Hdr8021Filter.AddProperty("InstanceID","0")
			  if($filter.Name)
			  {
				$AMT_Hdr8021Filter.AddProperty("Name",$filter.Name)
			  }
			  if($filter.CreationClassName)
			  {
				$AMT_Hdr8021Filter.AddProperty("CreationClassName",$filter.CreationClassName)
			  }
			  if($filter.SystemName)
			  {
				$AMT_Hdr8021Filter.AddProperty("SystemName", $filter.SystemName)
			  }
			  if($filter.SystemCreationClassName)
			  {
				$AMT_Hdr8021Filter.AddProperty("SystemCreationClassName", $filter.SystemCreationClassName)
			  }
			  if($filter.HdrProtocolID8021)
			  {
				$AMT_Hdr8021Filter.AddProperty("HdrProtocolID8021", $filter.HdrProtocolID8021)
			  }
			  if($filter.FilterProfile)
			  {
				$AMT_Hdr8021Filter.AddProperty("FilterProfile", $filter.FilterProfile)
			  }
			  if($filter.FilterDirection)
			  {
				$AMT_Hdr8021Filter.AddProperty("FilterDirection", $filter.FilterDirection)
			  }
			  if($filter.ActionEventOnMatch)
			  {
				$AMT_Hdr8021Filter.AddProperty("ActionEventOnMatch", $filter.ActionEventOnMatch)
			  }
			  if($filter.FilterProfileData)
			  {
				$AMT_Hdr8021Filter.AddProperty("FilterProfileData", $filter.FilterProfileData)
			  }

			  $ID = $AMT_Hdr8021Filter.Create().Get().GetProperty("InstanceID").ToString()
			  $filterTable[$ID] = $filter.PolicyName
			}


			if($filter.type -eq "AMT_IPHeadersFilter")
			{
			  $AMT_IPHeadersFilter = $Connection.NewInstance("http://intel.com/wbem/wscim/1/amt-schema/1/AMT_IPHeadersFilter")
			  $AMT_IPHeadersFilter.AddProperty("InstanceID","0")
			  if($filter.Name)
			  {
				$AMT_IPHeadersFilter.AddProperty("Name",$filter.Name)
			  }

			  if($filter.CreationClassName)
			  {
				$AMT_IPHeadersFilter.AddProperty("CreationClassName",$filter.CreationClassName)
			  }
			  if($filter.SystemName)
			  {
				$AMT_IPHeadersFilter.AddProperty("SystemName", $filter.SystemName)
			  }
			  if($filter.SystemCreationClassName)
			  {
				$AMT_IPHeadersFilter.AddProperty("SystemCreationClassName", $filter.SystemCreationClassName)
			  }
			  if($filter.FilterProfile)
			  {
				$AMT_IPHeadersFilter.AddProperty("FilterProfile", $filter.FilterProfile)
			  }
			  if($filter.FilterDirection)
			  {
				$AMT_IPHeadersFilter.AddProperty("FilterDirection", $filter.FilterDirection)
			  }
			  if($filter.ActionEventOnMatch)
			  {
				$AMT_IPHeadersFilter.AddProperty("ActionEventOnMatch", $filter.ActionEventOnMatch)
			  }
			  if($filter.HdrIPVersion)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrIPVersion", $filter.HdrIPVersion)
			  }
			  if($filter.HdrSrcAddress)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestAddress", $filter.HdrDestAddress)
			  }
			  if($filter.HdrSrcMask)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestMask", $filter.HdrDestMask)
			  }

			  if($filter.HdrDestAddress)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestAddress", $filter.HdrDestAddress)
			  }
			  if($filter.HdrDestMask)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestMask", $filter.HdrDestMask)
			  }

			  if($filter.HdrProtocolID)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrProtocolID", $filter.HdrProtocolID)
			  }

			  if($filter.HdrSrcPortStart)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrSrcPortStart", $filter.HdrSrcPortStart)
			  }
			  if($filter.HdrSrcPortEnd)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrSrcPortEnd", $filter.HdrSrcPortEnd)
			  }
			  if($filter.HdrDestPortStart)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestPortStart", $filter.HdrDestPortStart)
			  }
			  if($filter.HdrDestPortEnd)
			  {
				$AMT_IPHeadersFilter.AddProperty("HdrDestPortEnd", $filter.HdrDestPortEnd)
			  }
			  if($filter.TCPFlagsOn)
			  {
				$AMT_IPHeadersFilter.AddProperty("TCPFlagsOn ", $filter.TCPFlagsOn)
			  }
			  if($filter.TCPFlagsOff)
			  {
				$AMT_IPHeadersFilter.AddProperty("TCPFlagsOff ", $filter.TCPFlagsOff)
			  }
			  if($filter.FilterProfileData)
			  {
				$AMT_IPHeadersFilter.AddProperty("FilterProfileData  ", $filter.FilterProfileData)
			  }
			  $ID = $AMT_IPHeadersFilter.Create().Get().GetProperty("InstanceID").ToString()
			  $filterTable[$ID] = $filter.PolicyName
			}	
		  }


		  $policyList= $data.selectnodes("/SystemDefensePolicySet/ArrayOfPolicies/Policy")

		  foreach($policy in $policyList)
		  {
			  
			$AMT_SystemDefensePolicy = $Connection.NewInstance("http://intel.com/wbem/wscim/1/amt-schema/1/AMT_SystemDefensePolicy")
			$AMT_SystemDefensePolicy.AddProperty("InstanceID","n/a")
			if($policy.PolicyName)
			{
			  $AMT_SystemDefensePolicy.AddProperty("PolicyName", $policy.PolicyName)
			}
			if($policy.PolicyPrecedence)
			{
			  $AMT_SystemDefensePolicy.AddProperty("PolicyPrecedence", $policy.PolicyPrecedence)
			}
			if($policy.AntiSpoofingSupport)
			{
			  $AMT_SystemDefensePolicy.AddProperty("AntiSpoofingSupport", $policy.AntiSpoofingSupport)
			}
			if($policy.TxDefaultDrop)
			{
			  $AMT_SystemDefensePolicy.AddProperty("TxDefaultDrop", $policy.TxDefaultDrop)
			}
			if($policy.TxDefaultMatchEvent)
			{
			  $AMT_SystemDefensePolicy.AddProperty("TxDefaultMatchEvent", $policy.TxDefaultMatchEvent)
			}
			if($policy.TxDefaultCount)
			{
			  $AMT_SystemDefensePolicy.AddProperty("TxDefaultCount", $policy.TxDefaultCount)
			}
			if($policy.RxDefaultDrop)
			{
			  $AMT_SystemDefensePolicy.AddProperty("RxDefaultDrop", $policy.RxDefaultDrop)
			}
			if($policy.RxDefaultMatchEvent)
			{
			  $AMT_SystemDefensePolicy.AddProperty("RxDefaultMatchEvent", $policy.RxDefaultMatchEvent)
			}
			if($policy.RxDefaultCount)
			{
			  $AMT_SystemDefensePolicy.AddProperty("RxDefaultCount", $policy.RxDefaultCount)
			}
			
			foreach($filterID in $filterTable.GetEnumerator())
			{
			  if($filterID.Value -eq $policy.policyName)
			  {
				$AMT_SystemDefensePolicy.AddProperty("FilterCreationHandles", $FilterID.Name)
			  }
			}

			$Policy_EPR = $AMT_SystemDefensePolicy.Create()
			if($policy.Active)
			{
			  $ActivePolicy_EPR = $Policy_EPR 
			}
		  }

		  if($ActivePolicy_EPR)
		  {

		  while($cportEnumeration.HasNext)
		  {
			$portReference = $cportEnumeration.Next().Object 
			[System.String[]] $EthernetPort_KeyNames = "CreationClassName", "DeviceID", "SystemCreationClassName", "SystemName"
			$EthernetPort_EPR = $portReference.ToReference($EthernetPort_KeyNames)
			$NetworkPolicy = $Connection.NewInstance("http://intel.com/wbem/wscim/1/amt-schema/1/AMT_NetworkPortSystemDefensePolicy")
			$NetworkPolicy.AddProperty("Antecedent",$EthernetPort_EPR)
			$NetworkPolicy.AddProperty("Dependent",$ActivePolicy_EPR)
			$NetworkPolicy.AddProperty("Enabled","true")
			$NetworkPolicy.AddProperty("Active","true")
			$NetoworkPolicyObject = $NetworkPolicy.Create()    
			$EthernetPortName = $portReference.GetProperty("Description").ToString()

			if($EnabledInterfaces -eq "")
			{
			  $EnabledInterfaces = $EthernetPortName
			}
			else
			{
			  $EnabledInterfaces = $EnabledInterfaces +", " + $EthernetPortName
			} 
		  }
			$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Successful"
			$Obj | Add-Member -MemberType noteproperty -Name "EnabledOnInterfaces" -value $EnabledInterfaces
			$Results += $Obj      
		  }
		}
	}
	catch
	{
		$Obj | Add-Member -MemberType noteproperty -Name "Status" -value "Failed"
		$Obj | Add-Member -MemberType noteproperty -Name "EnabledOnInterfaces" -value $EnabledInterfaces
		$Obj | Add-Member -MemberType noteproperty -Name "Reason" -value $_.Exception.Message
		$Results += $Obj 
	}
	
	# Set back to false for the next cmdlet, if PS is still open
	$Connection.Options.AcceptSelfSignedCertificate=$false;

	Write-Output $Results
}
}
# SIG # Begin signature block
# MIIoggYJKoZIhvcNAQcCoIIoczCCKG8CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCDKPyMDMrxpe2tq
# xRtn6NSZ6Nd5mYkE3/lBRdfFTyoUKqCCEfIwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggYcMIIEBKADAgECAhAz1wio
# kUBTGeKlu9M5ua1uMA0GCSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYD
# VQQKEw9TZWN0aWdvIExpbWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENv
# ZGUgU2lnbmluZyBSb290IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5
# NTlaMFcxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxLjAs
# BgNVBAMTJVNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBFViBSMzYwggGi
# MA0GCSqGSIb3DQEBAQUAA4IBjwAwggGKAoIBgQC70f4et0JbePWQp64sg/GNIdMw
# hoV739PN2RZLrIXFuwHP4owoEXIEdiyBxasSekBKxRDogRQ5G19PB/YwMDB/NSXl
# wHM9QAmU6Kj46zkLVdW2DIseJ/jePiLBv+9l7nPuZd0o3bsffZsyf7eZVReqskmo
# PBBqOsMhspmoQ9c7gqgZYbU+alpduLyeE9AKnvVbj2k4aOqlH1vKI+4L7bzQHkND
# brBTjMJzKkQxbr6PuMYC9ruCBBV5DFIg6JgncWHvL+T4AvszWbX0w1Xn3/YIIq62
# 0QlZ7AGfc4m3Q0/V8tm9VlkJ3bcX9sR0gLqHRqwG29sEDdVOuu6MCTQZlRvmcBME
# Jd+PuNeEM4xspgzraLqVT3xE6NRpjSV5wyHxNXf4T7YSVZXQVugYAtXueciGoWnx
# G06UE2oHYvDQa5mll1CeHDOhHu5hiwVoHI717iaQg9b+cYWnmvINFD42tRKtd3V6
# zOdGNmqQU8vGlHHeBzoh+dYyZ+CcblSGoGSgg8sCAwEAAaOCAWMwggFfMB8GA1Ud
# IwQYMBaAFDLrkpr/NZZILyhAQnAgNpFcF4XmMB0GA1UdDgQWBBSBMpJBKyjNRsjE
# osYqORLsSKk/FDAOBgNVHQ8BAf8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADAT
# BgNVHSUEDDAKBggrBgEFBQcDAzAaBgNVHSAEEzARMAYGBFUdIAAwBwYFZ4EMAQMw
# SwYDVR0fBEQwQjBAoD6gPIY6aHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdv
# UHVibGljQ29kZVNpZ25pbmdSb290UjQ2LmNybDB7BggrBgEFBQcBAQRvMG0wRgYI
# KwYBBQUHMAKGOmh0dHA6Ly9jcnQuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0Nv
# ZGVTaWduaW5nUm9vdFI0Ni5wN2MwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNl
# Y3RpZ28uY29tMA0GCSqGSIb3DQEBDAUAA4ICAQBfNqz7+fZyWhS38Asd3tj9lwHS
# /QHumS2G6Pa38Dn/1oFKWqdCSgotFZ3mlP3FaUqy10vxFhJM9r6QZmWLLXTUqwj3
# ahEDCHd8vmnhsNufJIkD1t5cpOCy1rTP4zjVuW3MJ9bOZBHoEHJ20/ng6SyJ6UnT
# s5eWBgrh9grIQZqRXYHYNneYyoBBl6j4kT9jn6rNVFRLgOr1F2bTlHH9nv1HMePp
# GoYd074g0j+xUl+yk72MlQmYco+VAfSYQ6VK+xQmqp02v3Kw/Ny9hA3s7TSoXpUr
# OBZjBXXZ9jEuFWvilLIq0nQ1tZiao/74Ky+2F0snbFrmuXZe2obdq2TWauqDGIgb
# MYL1iLOUJcAhLwhpAuNMu0wqETDrgXkG4UGVKtQg9guT5Hx2DJ0dJmtfhAH2KpnN
# r97H8OQYok6bLyoMZqaSdSa+2UA1E2+upjcaeuitHFFjBypWBmztfhj24+xkc6Zt
# CDaLrw+ZrnVrFyvCTWrDUUZBVumPwo3/E3Gb2u2e05+r5UWmEsUUWlJBl6MGAAjF
# 5hzqJ4I8O9vmRsTvLQA1E802fZ3lqicIBczOwDYOSxlP0GOabb/FKVMxItt1UHeG
# 0PL4au5rBhs+hSMrl8h+eplBDN1Yfw6owxI9OjWb4J0sjBeBVESoeh2YnZZ/WVim
# VGX/UUIL+Efrz/jlvzCCBlswggTDoAMCAQICEHAQMvP0AIEdrcUef3h12LwwDQYJ
# KoZIhvcNAQELBQAwVzELMAkGA1UEBhMCR0IxGDAWBgNVBAoTD1NlY3RpZ28gTGlt
# aXRlZDEuMCwGA1UEAxMlU2VjdGlnbyBQdWJsaWMgQ29kZSBTaWduaW5nIENBIEVW
# IFIzNjAeFw0yMzA0MDQwMDAwMDBaFw0yNDA0MDMyMzU5NTlaMIG7MRAwDgYDVQQF
# EwcyMTg5MDc0MRMwEQYLKwYBBAGCNzwCAQMTAlVTMRkwFwYLKwYBBAGCNzwCAQIT
# CERlbGF3YXJlMR0wGwYDVQQPExRQcml2YXRlIE9yZ2FuaXphdGlvbjELMAkGA1UE
# BhMCVVMxEzARBgNVBAgMCkNhbGlmb3JuaWExGjAYBgNVBAoMEUludGVsIENvcnBv
# cmF0aW9uMRowGAYDVQQDDBFJbnRlbCBDb3Jwb3JhdGlvbjCCAaIwDQYJKoZIhvcN
# AQEBBQADggGPADCCAYoCggGBANO3BjDeeK28E+zFxToyp77/6fB+Mp6913C+9t1U
# JguD/sBVnxljfvQScWzRrvF0chAOz5rjlnvUTXUFsgvJ4XFIZQtG/Rfyb4qvQbFE
# vdoS1kFlMAtXnMuuDSMbncPju4R/5gLfMzHFLRpHOoUmeRFlwEkyAYVwjDvH+OcR
# WW2Vm72ggaVeqgW5kIx3R4VQSXgJ3ToPP/49/9YjEwhM0yLccTPJKVB/6unAJaY6
# /j15gi4owbo6K88625JlZTy7drEhZvvI9vlStkPUOxbHQmU72QG7bF1ofao/4lTH
# pE6RAXTppoWlu24kRhLcXaIIAitMDqs9E0SFdV2qcAxc7d38v0l2sds/sy2EWMU9
# RZAjJD4fH/Bm0hJVm5937cq/sf20lucUQnu96+5zVHXSIuD65dkQ/6aoGxwZty49
# rdTFY494ri++yfw6NWrbQkIoc2c1HG0jHRAV6m3dgzwY0104g/BpZkLogT4EEEO5
# XRm6k1PSMxoM194ytHU+pJ2B2QIDAQABo4IBvDCCAbgwHwYDVR0jBBgwFoAUgTKS
# QSsozUbIxKLGKjkS7EipPxQwHQYDVR0OBBYEFJLGD4aF5QaeHQosntNMcCm3IDAZ
# MA4GA1UdDwEB/wQEAwIHgDAMBgNVHRMBAf8EAjAAMBMGA1UdJQQMMAoGCCsGAQUF
# BwMDMEkGA1UdIARCMEAwNQYMKwYBBAGyMQECAQYBMCUwIwYIKwYBBQUHAgEWF2h0
# dHBzOi8vc2VjdGlnby5jb20vQ1BTMAcGBWeBDAEDMEsGA1UdHwREMEIwQKA+oDyG
# Omh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVTaWduaW5n
# Q0FFVlIzNi5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsGAQUFBzAChjpodHRwOi8v
# Y3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmluZ0NBRVZSMzYu
# Y3J0MCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTAuBgNVHREE
# JzAloCMGCCsGAQUFBwgDoBcwFQwTVVMtREVMQVdBUkUtMjE4OTA3NDANBgkqhkiG
# 9w0BAQsFAAOCAYEAttRv5ICWB9YKIXk6xtQ385RQks/95gKJP+T62x8e2rHfQXY4
# GQChplzrceWnolz+TBJF1Dt8HYLUjpj0tgQHcU1mT54Lz4vt8eRHfldjRUC3I+Sy
# FxfWncIWYru27DPKAUEVmCHNF6++myfE7FLTFsL2ogZhZF1sUIBAvDd7d3j0wtz4
# N4d9MhjIfncOOqTOctwEbLtSTX9xIOry4ZgsDVs/QWiqyMyUaG649bcWWXDWxIq+
# 6jtY4WJn4KQUhPVzP53e45Y7Leb+69GCSkfQ5cEZlSOZq2PeFRqP3PwVQ946hbm8
# saXEl3iaZ8s5YY2JorFvstkqYXiQ9O55wAZ4fTeyhWjVNpkqDiIHpO+UYREJ5AHG
# nsZEJJkx8E/Y+zX7YuDMJmV7+WLI1CvIyCLoNhUMupfReykiQKD1P+CCe6UzSOey
# mSrh0X/XpOcDz/+sSgUIziD5HUuMYp0czD4Up/FFrl3qX3te9N4hQt74Ifh35DLR
# g/DdRlqPI70u7jl5MYIV5jCCFeICAQEwazBXMQswCQYDVQQGEwJHQjEYMBYGA1UE
# ChMPU2VjdGlnbyBMaW1pdGVkMS4wLAYDVQQDEyVTZWN0aWdvIFB1YmxpYyBDb2Rl
# IFNpZ25pbmcgQ0EgRVYgUjM2AhBwEDLz9ACBHa3FHn94ddi8MA0GCWCGSAFlAwQC
# AQUAoHwwEAYKKwYBBAGCNwIBDDECMAAwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcC
# AQQwHAYKKwYBBAGCNwIBCzEOMAwGCisGAQQBgjcCARUwLwYJKoZIhvcNAQkEMSIE
# IMKjxMjCrHQFagc33U7h6IxqfH4cAuv5gxOtqRMCyNMuMA0GCSqGSIb3DQEBAQUA
# BIIBgHhltoRnjZ0WkAkByhpIRdJdy3+uHHbMRz2mH/w0rIRnNSG8D6E9LH44Ag8g
# CukMJOd7rc5R5aJPQzsw7ieGoyzT06ooRJobzYYbuQ0BY8Im7oJBsaMAqqAFV2Os
# H5gbULF6NHrkPPNDuzJ9zkuuCxQ+4J88j+BSYvCreAULePvEH9EYs+Sfd6ED+zy2
# OpNJNSG4ek0uLGWaTrtmlqitXO8p15An5cNSGxJu8oeCjV+6lu/7HCZ8bt3n0Qsi
# TpdvJH0o+f43TWtU7ASt1kDu1I+52T4MrS/GTBKh8qBh7z2MPq9bXbxK1W+MkODX
# 1M7iIeYicN56WP8vX8hGncfU76z5nbDIQso02EZdOuMRQX1vyhSKtnXHjukZXYiF
# VH9qno53ZELTxo8ziUFL6I933wu4zX3+4Xb3eUBy4j3tDod3tYg2Tbk+LsLwXhij
# 1/sDwAThMoBlGI0iXPtDNTKKqnITydhPvP/qS91KwX0CLxklXfCTEmrHd8MzC/wZ
# JBgStaGCE04wghNKBgorBgEEAYI3AwMBMYITOjCCEzYGCSqGSIb3DQEHAqCCEycw
# ghMjAgEDMQ8wDQYJYIZIAWUDBAICBQAwge8GCyqGSIb3DQEJEAEEoIHfBIHcMIHZ
# AgEBBgorBgEEAbIxAgEBMDEwDQYJYIZIAWUDBAIBBQAEICVyVYTbYzjUmNkA1b+3
# okAuYtzuCxm5VFx+WhDvvp5AAhQ+XTywoSv4NNFEv0SezwCDCm1f2BgPMjAyMzA4
# MTMwNzM0MjRaoG6kbDBqMQswCQYDVQQGEwJHQjETMBEGA1UECBMKTWFuY2hlc3Rl
# cjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMSwwKgYDVQQDDCNTZWN0aWdvIFJT
# QSBUaW1lIFN0YW1waW5nIFNpZ25lciAjNKCCDekwggb1MIIE3aADAgECAhA5TCXh
# fKBtJ6hl4jvZHSLUMA0GCSqGSIb3DQEBDAUAMH0xCzAJBgNVBAYTAkdCMRswGQYD
# VQQIExJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGDAWBgNV
# BAoTD1NlY3RpZ28gTGltaXRlZDElMCMGA1UEAxMcU2VjdGlnbyBSU0EgVGltZSBT
# dGFtcGluZyBDQTAeFw0yMzA1MDMwMDAwMDBaFw0zNDA4MDIyMzU5NTlaMGoxCzAJ
# BgNVBAYTAkdCMRMwEQYDVQQIEwpNYW5jaGVzdGVyMRgwFgYDVQQKEw9TZWN0aWdv
# IExpbWl0ZWQxLDAqBgNVBAMMI1NlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgU2ln
# bmVyICM0MIICIjANBgkqhkiG9w0BAQEFAAOCAg8AMIICCgKCAgEApJMoUkvPJ4d2
# pCkcmTjA5w7U0RzsaMsBZOSKzXewcWWCvJ/8i7u7lZj7JRGOWogJZhEUWLK6Ilvm
# 9jLxXS3AeqIO4OBWZO2h5YEgciBkQWzHwwj6831d7yGawn7XLMO6EZge/NMgCEKz
# X79/iFgyqzCz2Ix6lkoZE1ys/Oer6RwWLrCwOJVKz4VQq2cDJaG7OOkPb6lampEo
# EzW5H/M94STIa7GZ6A3vu03lPYxUA5HQ/C3PVTM4egkcB9Ei4GOGp7790oNzEhSb
# mkwJRr00vOFLUHty4Fv9GbsfPGoZe267LUQqvjxMzKyKBJPGV4agczYrgZf6G5t+
# iIfYUnmJ/m53N9e7UJ/6GCVPE/JefKmxIFopq6NCh3fg9EwCSN1YpVOmo6DtGZZl
# FSnF7TMwJeaWg4Ga9mBmkFgHgM1Cdaz7tJHQxd0BQGq2qBDu9o16t551r9OlSxih
# DJ9XsF4lR5F0zXUS0Zxv5F4Nm+x1Ju7+0/WSL1KF6NpEUSqizADKh2ZDoxsA76K1
# lp1irScL8htKycOUQjeIIISoh67DuiNye/hU7/hrJ7CF9adDhdgrOXTbWncC0aT6
# 9c2cPcwfrlHQe2zYHS0RQlNxdMLlNaotUhLZJc/w09CRQxLXMn2YbON3Qcj/HyRU
# 726txj5Ve/Fchzpk8WBLBU/vuS/sCRMCAwEAAaOCAYIwggF+MB8GA1UdIwQYMBaA
# FBqh+GEZIA/DQXdFKI7RNV8GEgRVMB0GA1UdDgQWBBQDDzHIkSqTvWPz0V1NpDQP
# 0pUBGDAOBgNVHQ8BAf8EBAMCBsAwDAYDVR0TAQH/BAIwADAWBgNVHSUBAf8EDDAK
# BggrBgEFBQcDCDBKBgNVHSAEQzBBMDUGDCsGAQQBsjEBAgEDCDAlMCMGCCsGAQUF
# BwIBFhdodHRwczovL3NlY3RpZ28uY29tL0NQUzAIBgZngQwBBAIwRAYDVR0fBD0w
# OzA5oDegNYYzaHR0cDovL2NybC5zZWN0aWdvLmNvbS9TZWN0aWdvUlNBVGltZVN0
# YW1waW5nQ0EuY3JsMHQGCCsGAQUFBwEBBGgwZjA/BggrBgEFBQcwAoYzaHR0cDov
# L2NydC5zZWN0aWdvLmNvbS9TZWN0aWdvUlNBVGltZVN0YW1waW5nQ0EuY3J0MCMG
# CCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTANBgkqhkiG9w0BAQwF
# AAOCAgEATJtlWPrgec/vFcMybd4zket3WOLrvctKPHXefpRtwyLHBJXfZWlhEwz2
# DJ71iSBewYfHAyTKx6XwJt/4+DFlDeDrbVFXpoyEUghGHCrC3vLaikXzvvf2LsR+
# 7fjtaL96VkjpYeWaOXe8vrqRZIh1/12FFjQn0inL/+0t2v++kwzsbaINzMPxbr0h
# kRojAFKtl9RieCqEeajXPawhj3DDJHk6l/ENo6NbU9irALpY+zWAT18ocWwZXsKD
# cpCu4MbY8pn76rSSZXwHfDVEHa1YGGti+95sxAqpbNMhRnDcL411TCPCQdB6ljvD
# S93NkiZ0dlw3oJoknk5fTtOPD+UTT1lEZUtDZM9I+GdnuU2/zA2xOjDQoT1IrXpl
# 5Ozf4AHwsypKOazBpPmpfTXQMkCgsRkqGCGyyH0FcRpLJzaq4Jgcg3Xnx35LhEPN
# Q/uQl3YqEqxAwXBbmQpA+oBtlGF7yG65yGdnJFxQjQEg3gf3AdT4LhHNnYPl+Mol
# HEQ9J+WwhkcqCxuEdn17aE+Nt/cTtO2gLe5zD9kQup2ZLHzXdR+PEMSU5n4k5ZVK
# iIwn1oVmHfmuZHaR6Ej+yFUK7SnDH944psAU+zI9+KmDYjbIw74Ahxyr+kpCHIkD
# 3PVcfHDZXXhO7p9eIOYJanwrCKNI9RX8BE/fzSEceuX1jhrUuUAwggbsMIIE1KAD
# AgECAhAwD2+s3WaYdHypRjaneC25MA0GCSqGSIb3DQEBDAUAMIGIMQswCQYDVQQG
# EwJVUzETMBEGA1UECBMKTmV3IEplcnNleTEUMBIGA1UEBxMLSmVyc2V5IENpdHkx
# HjAcBgNVBAoTFVRoZSBVU0VSVFJVU1QgTmV0d29yazEuMCwGA1UEAxMlVVNFUlRy
# dXN0IFJTQSBDZXJ0aWZpY2F0aW9uIEF1dGhvcml0eTAeFw0xOTA1MDIwMDAwMDBa
# Fw0zODAxMTgyMzU5NTlaMH0xCzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVy
# IE1hbmNoZXN0ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGDAWBgNVBAoTD1NlY3RpZ28g
# TGltaXRlZDElMCMGA1UEAxMcU2VjdGlnbyBSU0EgVGltZSBTdGFtcGluZyBDQTCC
# AiIwDQYJKoZIhvcNAQEBBQADggIPADCCAgoCggIBAMgbAa/ZLH6ImX0BmD8gkL2c
# gCFUk7nPoD5T77NawHbWGgSlzkeDtevEzEk0y/NFZbn5p2QWJgn71TJSeS7JY8IT
# m7aGPwEFkmZvIavVcRB5h/RGKs3EWsnb111JTXJWD9zJ41OYOioe/M5YSdO/8zm7
# uaQjQqzQFcN/nqJc1zjxFrJw06PE37PFcqwuCnf8DZRSt/wflXMkPQEovA8NT7OR
# AY5unSd1VdEXOzQhe5cBlK9/gM/REQpXhMl/VuC9RpyCvpSdv7QgsGB+uE31DT/b
# 0OqFjIpWcdEtlEzIjDzTFKKcvSb/01Mgx2Bpm1gKVPQF5/0xrPnIhRfHuCkZpCkv
# RuPd25Ffnz82Pg4wZytGtzWvlr7aTGDMqLufDRTUGMQwmHSCIc9iVrUhcxIe/arK
# CFiHd6QV6xlV/9A5VC0m7kUaOm/N14Tw1/AoxU9kgwLU++Le8bwCKPRt2ieKBtKW
# h97oaw7wW33pdmmTIBxKlyx3GSuTlZicl57rjsF4VsZEJd8GEpoGLZ8DXv2DolNn
# yrH6jaFkyYiSWcuoRsDJ8qb/fVfbEnb6ikEk1Bv8cqUUotStQxykSYtBORQDHin6
# G6UirqXDTYLQjdprt9v3GEBXc/Bxo/tKfUU2wfeNgvq5yQ1TgH36tjlYMu9vGFCJ
# 10+dM70atZ2h3pVBeqeDAgMBAAGjggFaMIIBVjAfBgNVHSMEGDAWgBRTeb9aqitK
# z1SA4dibwJ3ysgNmyzAdBgNVHQ4EFgQUGqH4YRkgD8NBd0UojtE1XwYSBFUwDgYD
# VR0PAQH/BAQDAgGGMBIGA1UdEwEB/wQIMAYBAf8CAQAwEwYDVR0lBAwwCgYIKwYB
# BQUHAwgwEQYDVR0gBAowCDAGBgRVHSAAMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6
# Ly9jcmwudXNlcnRydXN0LmNvbS9VU0VSVHJ1c3RSU0FDZXJ0aWZpY2F0aW9uQXV0
# aG9yaXR5LmNybDB2BggrBgEFBQcBAQRqMGgwPwYIKwYBBQUHMAKGM2h0dHA6Ly9j
# cnQudXNlcnRydXN0LmNvbS9VU0VSVHJ1c3RSU0FBZGRUcnVzdENBLmNydDAlBggr
# BgEFBQcwAYYZaHR0cDovL29jc3AudXNlcnRydXN0LmNvbTANBgkqhkiG9w0BAQwF
# AAOCAgEAbVSBpTNdFuG1U4GRdd8DejILLSWEEbKw2yp9KgX1vDsn9FqguUlZkCls
# Ycu1UNviffmfAO9Aw63T4uRW+VhBz/FC5RB9/7B0H4/GXAn5M17qoBwmWFzztBEP
# 1dXD4rzVWHi/SHbhRGdtj7BDEA+N5Pk4Yr8TAcWFo0zFzLJTMJWk1vSWVgi4zVx/
# AZa+clJqO0I3fBZ4OZOTlJux3LJtQW1nzclvkD1/RXLBGyPWwlWEZuSzxWYG9vPW
# S16toytCiiGS/qhvWiVwYoFzY16gu9jc10rTPa+DBjgSHSSHLeT8AtY+dwS8BDa1
# 53fLnC6NIxi5o8JHHfBd1qFzVwVomqfJN2Udvuq82EKDQwWli6YJ/9GhlKZOqj0J
# 9QVst9JkWtgqIsJLnfE5XkzeSD2bNJaaCV+O/fexUpHOP4n2HKG1qXUfcb9bQ11l
# PVCBbqvw0NP8srMftpmWJvQ8eYtcZMzN7iea5aDADHKHwW5NWtMe6vBE5jJvHOsX
# TpTDeGUgOw9Bqh/poUGd/rG4oGUqNODeqPk85sEwu8CgYyz8XBYAqNDEf+oRnR4G
# xqZtMl20OAkrSQeq/eww2vGnL8+3/frQo4TZJ577AWZ3uVYQ4SBuxq6x+ba6yDVd
# M3aO8XwgDCp3rrWiAoa6Ke60WgCxjKvj+QrJVF3UuWp0nr1IrpgxggQsMIIEKAIB
# ATCBkTB9MQswCQYDVQQGEwJHQjEbMBkGA1UECBMSR3JlYXRlciBNYW5jaGVzdGVy
# MRAwDgYDVQQHEwdTYWxmb3JkMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxJTAj
# BgNVBAMTHFNlY3RpZ28gUlNBIFRpbWUgU3RhbXBpbmcgQ0ECEDlMJeF8oG0nqGXi
# O9kdItQwDQYJYIZIAWUDBAICBQCgggFrMBoGCSqGSIb3DQEJAzENBgsqhkiG9w0B
# CRABBDAcBgkqhkiG9w0BCQUxDxcNMjMwODEzMDczNDI0WjA/BgkqhkiG9w0BCQQx
# MgQwiD+OdWCAhWdBbJfJAALgZGcX3U8JbfqYcI6zMYQHy1ik03/iCWuJ9vYE+d48
# mh58MIHtBgsqhkiG9w0BCRACDDGB3TCB2jCB1zAWBBSuYq91Cgy9R9ZGH3Vo4ryM
# 58pPlDCBvAQUAtZbleKDcMFXAJX6iPkj3ZN/rY8wgaMwgY6kgYswgYgxCzAJBgNV
# BAYTAlVTMRMwEQYDVQQIEwpOZXcgSmVyc2V5MRQwEgYDVQQHEwtKZXJzZXkgQ2l0
# eTEeMBwGA1UEChMVVGhlIFVTRVJUUlVTVCBOZXR3b3JrMS4wLAYDVQQDEyVVU0VS
# VHJ1c3QgUlNBIENlcnRpZmljYXRpb24gQXV0aG9yaXR5AhAwD2+s3WaYdHypRjan
# eC25MA0GCSqGSIb3DQEBAQUABIICAEys6uz6/uqs4sHss0YQWMZsfV/J82icILb+
# ufqLLanYwKuo//31RK5Xmjc+FMYJiqcpkNkaALGgfNblvUKMz51QW2FPrMQtSoev
# DFeaSx3FZXNXi3lDjyW8MkxYbqzQ7fiNomJf8uX9nNOvNSJxkYlzzDPSvBbp37Vm
# H3b6I+A6qTRqEWs+I8B85QyocIvNbmpv07+BLpwUiNwuxuTqpGUwmOGNLGrGqrlM
# 3g8FQLl0b7PHOd/aST6hw6/h9w0FoVVYIdJVeXbF4DlASvMmmCw7s4Jh/lN5/+Db
# FILDAZn6Jnv7PkhweAfZ7ImGek4CjbGZ+9QF0GDDFP+bvC1vRI5vSlghUaknWhR9
# xytE/6WucaDsVPBM7oVrrkMc6lwvnyukUJb/AVJOEyk98iIV0Dy7YIATVPORLDvU
# yymMctxuK/WelV4BiAI7Sgsrm3Dj5RZcERlvxMLJ9miw7OyipIf5GN0DX3o9XZ13
# sbucygmmICVm2Wgkmu46oLKz7LVD9ObYhS3yIoOPKoZZgyqAa/tmiG3iQVryNBuh
# tj/fzVAtPAVO/g1zXsMvTfkVuT+Kzj1TVElnal4+dccKlEsxpi7T6ucfHu/8kXEt
# p5tUNAmSjdCtwDDlHNVWkQzoLOzqnPu5vEvmRE6tmnsJeIu5yBfXwdJaN1jWeZau
# 2+McC9gA
# SIG # End signature block
