// Copyright (C) 2003 Intel Corporation

//////////////////////////////////////////////////////////////////////////
// TimerManager.cpp
//
// This file contains the implementation of the TimerManager class
//////////////////////////////////////////////////////////////////////////
#include "TimerManager.h"
#include "Lock.h"


TimerManager* TimerManager::_instance = NULL;

TimerManager* TimerManager::instance() 
{
	if (_instance == NULL) {
		_instance = new TimerManager();
		//_destroyer.setTimerManager (_instance);
	}
	return _instance;
}

void TimerManager::stop()
{
	if( _instance ){
		delete _instance;
		_instance = 0;
	}
}

TimerManager::TimerManager() : Thread(), _timerId(0), _exitThread(false),_callbacks() 
{
	start(); // run the thread;
}

TimerManager::~TimerManager()
{
	_exitThread = true;
	_registration.set(); // to exit the main loop
	wait(); // wait for the thread to exit
}

int TimerManager::registerCallback(long interval, CallbackFunction func, SPtr<CallbackParam> param, bool recurring)
{
	_Callback cb;
	cb.function = func;
	cb.interval = interval;
	cb.param = param;
	cb.nextExpiration = elapsedTime() + interval;
	cb.recurring = recurring;
	cb.runLock = SPtr<Semaphore>(new Semaphore());
	{
		Lock l(_sem);
		cb.timerId = _timerId++;
		_callbacks.push_back(cb);
		_registration.set();
	}
	return cb.timerId;
}

void TimerManager::unregisterCallback(int timer_id)
{
	Lock l(_sem);
    vector<_Callback>::iterator it;
	for (it = _callbacks.begin(); it != _callbacks.end(); ++it) {
		if (it->timerId == timer_id) {
			it->runLock->acquire();
			_callbacks.erase(it);
			_registration.set();
			break;
		}
	}
}

void TimerManager::run()
{
	while (!_exitThread) {
		// calculate how long to wait
		unsigned long msecs_to_wait = WAIT_INFINITE;
		unsigned long current_msecs = elapsedTime();
		{
			Lock l(_sem);
			for (vector<_Callback>::iterator it = _callbacks.begin(); it != _callbacks.end(); ++it) {
				if( (unsigned long)it->nextExpiration < current_msecs ){
					msecs_to_wait = 0;
				} else if (it->nextExpiration - current_msecs <= msecs_to_wait) {
					msecs_to_wait = it->nextExpiration - current_msecs;
				}
			}
			_registration.reset();
		}
		// wait
		_registration.wait(msecs_to_wait);

		// identify the expired callbacks
		current_msecs = elapsedTime();
		vector<_Callback> expired;
		{
			Lock l(_sem);
			for (vector<_Callback>::iterator it = _callbacks.begin(); it != _callbacks.end(); ++it) {
				if( (unsigned long)it->nextExpiration <= current_msecs) {
					it->nextExpiration = current_msecs + it->interval;
					(*it).runLock->acquire();
					expired.push_back(*it);
				}
			}
		}

		// call the callbacks
		for (vector<_Callback>::iterator it = expired.begin(); it != expired.end(); ++it) {
			it->function(it->param);
			it->runLock->release();
			if (!it->recurring) {
				unregisterCallback(it->timerId);
			}
		}
	}
}

