//  Copyright (C) 2007 Intel Corporation
#ifndef _SOCKET_LIB__H_
#define _SOCKET_LIB__H_

#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>

#ifndef SECURITY_WIN32
#define SECURITY_WIN32
#endif
#include "schannel.h"
#include "sspi.h"
#include "Logger.h"

#else
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <vector>
#include <SPtr.h>

#define INVALID_SOCKET -1
typedef unsigned short USHORT;
typedef unsigned char BYTE;
#endif

#include <string>
#include "LibDefs.h"
#include "Lock.h"

#ifndef _WIN32
#include "openssl/ssl.h"
#include "openssl/err.h"
#endif

using namespace std;

#define MAX_IP_LEN			256
#define MAX_NAME_LEN		256
#define MAX_PSWD_LEN		256

typedef struct {
	char user_name[MAX_NAME_LEN]; // user name - nul terminated string 
	char user_pswd[MAX_PSWD_LEN]; // user password - nul terminated string 
} TCPSessionParams;

typedef enum {PROXY_NO_PROXY, PROXY_SOCKS_V5} PROXY_TYPE;

typedef struct {
		PROXY_TYPE type;
		char server[MAX_IP_LEN]; // hostname or IPv4 (IPv6 not supported yet) �hostname�, �a.b.c.d�
		INT32 port;
		char user[MAX_NAME_LEN]; // user name - nul terminated string 
		char password[MAX_PSWD_LEN]; // user password - nul terminated string 
} ProxySettings;

#define UDP_PORT_RMCP_CLEAR		0x26F
#define UDP_PORT_RMCP_SECURE	0x298
#define SOCKET_BUF_SIZE			1512

//ICMP defines
#define ICMP_HDR 8 // minimum 8 byte icmp packet (just header)
#define UDP_HDR 8

#define ICMP_ECHOREPLY          0
#define ICMP_DEST_UNREACH       3
#define ICMP_FRAG_NEEDED        4

typedef enum {
	SOCKET_LIB_RES_OK, 
	SOCKET_LIB_RES_ERROR, 
	SOCKET_LIB_RES_SOCKET_ERROR,
	SOCKET_LIB_RES_TLS_CONNECTION_FAILED,
	SOCKET_LIB_RES_SOCKS_ERROR,
	SOCKET_LIB_RES_SOCKS_AUTH_FAIL
} SOCKET_LIB_RESULT;


#pragma pack(1)
/* IP header */
struct _IpHdr {
	UINT8	ver_len:4;			// length of the header(4bits) & Version of IP (4bits)
	UINT8	tos;				// Type of service
	UINT16	total_len;			// total length of the packet
	UINT16	ident;				// unique identifier
	UINT16	frag_and_flags;		// flags
	UINT8	ttl;				// TTL
	UINT8	proto;				// protocol (TCP, UDP etc)
	UINT16	checksum;			// IP checksum
	UINT32	sourceIP;
	UINT32	destIP;
};

/* ICMP header */
struct _IcmpHdr {
  UINT8		i_type;
  UINT8		i_code; /* type sub code */
  UINT16	i_cksum;
  UINT16	unused;  
  UINT16	next_mtu;
};

struct _IcmpEchoHdr {
  UINT8		i_type;
  UINT8		i_code; /* type sub code */
  UINT16	i_cksum;
  UINT16	i_id;  
  UINT16	i_seq;
};


/* UDP header */
struct _UdpHdr {
  UINT16     source;
  UINT16     dest;
  UINT16     len;
  UINT16     check;
};

#pragma pack()

#define IPH_LEN(x) ( ((_IpHdr*)x)->ver_len & 0x0f ) 
#define IPH_VER(x) ( ((_IpHdr*)x)->ver_len & 0xf0) >> 4)

typedef enum _IP_VERSION
{
	IP_VERSION_NOT_SET = 0,
	IPv4 = 4,
	IPv6 = 6
}IP_VERSION;

class Socket{
public:
	Socket(){}
	virtual ~Socket(){}

	virtual SOCKET_LIB_RESULT connect(const string& ipAddr, const UINT16 port)=0;
	virtual INT32 send(const UINT8 * data, UINT32 dataLen)=0;
	virtual INT32 sendTo(const UINT8 * data, UINT32 dataLen, const string& ipAddr, const UINT16& port)=0;
	virtual INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout = INFINITE)=0;
	virtual INT32 receiveFrom(UINT8 *data, UINT32 dataLen, string& ipAddr, UINT16& port)=0;
	virtual INT32 close()=0;
	virtual Socket *accept()=0;

	virtual INT32 getType()=0;
	virtual SOCKET getSocket()=0;
	virtual UINT16 getPort()=0;
	virtual bool setNonBlocking( bool enable = true )=0;
#ifdef _WIN32
	virtual bool setIPFragment( bool enable = false )=0;
#endif
	virtual bool setRecvBuf(const INT32& recvBufSize = SOCKET_BUF_SIZE)=0;
	virtual bool setSendBuf(const INT32& sendBufSize = SOCKET_BUF_SIZE)=0;
	// Get internal buffer date was received in the last received call. Usefull just for TLS-SSPI functionality.
	virtual INT32 getInternalBUffer(UINT8 *&data, UINT32 dataLen) = 0;

	static string GetErrorMessage(SOCKET_LIB_RESULT res);
protected:
};

typedef INT32 (*Open)(UINT32 clientID, UINT32 conID);
typedef INT32 (*Close)(UINT32 conID);
typedef INT32 (*Receive)(UINT8* buffer, UINT32* length, UINT32 conID);
typedef INT32 (*Send)(UINT8* buffer, UINT32 length, UINT32 conID);
typedef struct _CallbacksSocketsFunction
{
	Open   		open_connection;
	Close  		close_connection;
	Receive		receive_data;
	Send		send_data;
}CallbacksSocketsFunction;

class CallbacksSocket: public Socket{
public:
	CallbacksSocket(CallbacksSocketsFunction* callbacks)
	{
		_callbacks = new CallbacksSocketsFunction();
		_callbacks->open_connection = callbacks->open_connection;
		_callbacks->close_connection = callbacks->close_connection;
		_callbacks->send_data = callbacks->send_data;
		_callbacks->receive_data = callbacks->receive_data;
		_conID++;
	}

	INT32 send(const UINT8 * data, UINT32 dataLen)
	{
		return (_callbacks->send_data((UINT8*)data, dataLen, _conID) == 0) ? SOCKET_LIB_RES_OK : SOCKET_LIB_RES_ERROR;
	}
	
	INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout)
	{
		return (_callbacks->receive_data(data, (UINT32*)&dataLen, _conID) == 0) ? SOCKET_LIB_RES_OK : SOCKET_LIB_RES_ERROR;
	}
	
	SOCKET_LIB_RESULT connect(UINT32 ClientID)
	{
		return (_callbacks->open_connection(ClientID, _conID) == 0) ? SOCKET_LIB_RES_OK : SOCKET_LIB_RES_ERROR;
	}

	virtual ~CallbacksSocket()
	{
		if(_callbacks)
			delete _callbacks;
	}
protected:
	static UINT32 _conID;
	CallbacksSocketsFunction* _callbacks;
	CallbacksSocket(const CallbacksSocket& rhs);
	virtual CallbacksSocket& operator=(const CallbacksSocket& rhs);
};


class IPSocket: public Socket{
public:
	IPSocket();
	IPSocket(const INT32& type, const INT32& protocol =0,  const UINT16& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET);
	IPSocket(SOCKET bindedSocket, IP_VERSION ipVersion = IP_VERSION_NOT_SET);
	virtual ~IPSocket();

	virtual SOCKET_LIB_RESULT connect(const string& ipAddr, const UINT16 port);
	virtual INT32 send(const UINT8 * data, UINT32 dataLen);
	virtual INT32 sendTo(const UINT8 * data, UINT32 dataLen, const string& ipAddr, const UINT16& port);
	virtual INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout);
	virtual INT32 receiveFrom(UINT8 *data, UINT32 dataLen, string& ipAddr, UINT16& port);
	virtual INT32 close();
	virtual Socket *accept(){ return 0; }
	// Get internal buffer date was received in the last received call. Usefull just for TLS-SSPI functionality.
	virtual INT32 getInternalBUffer(UINT8 *&data, UINT32 dataLen){return 0;}
		
	
	virtual INT32 getType();
	virtual SOCKET getSocket();
	virtual UINT16 getPort();
	virtual bool setNonBlocking( bool enable = true );
	static IP_VERSION SupportedIPStack(const char* hostname = "localhost");
#ifdef _WIN32
	virtual bool setIPFragment( bool enable = false );
#endif
	virtual bool setRecvBuf(const INT32& recvBufSize );
	virtual bool setSendBuf(const INT32& sendBufSize );
protected: 
	SOCKET_LIB_RESULT connect_v6(const string& ipAddr, const UINT16& port);
	SOCKET_LIB_RESULT connect_v4(const string& ipAddr, const UINT16& port);
	virtual INT32 prepareDataToSend(const BYTE*& data, int dataLen,const BYTE*& preparedData, int &envelopedDataLen);
	virtual INT32 prepareReceivedData(BYTE*&data, int bytesReceived);
	virtual INT32 copyTempBuffer(UINT8 *&data, UINT32 dataLen, int totalReceived){ return totalReceived;}

	friend class IPServerSocket;
	bool   _isNonBlocking;
	SOCKET _socket;
	UINT16 _port;
	Semaphore _txSem;
	IP_VERSION _ipVersion;
	std::vector<BYTE> _tempReceiveBuffer;


private:
	// These functions are called for WinSSLSocket usage
	virtual bool _readingCompleted(){ return false;}
	virtual int _reservedPlace(int dataLen){ return 0;}
	virtual bool _isTLS(){return false;}
};



class UDPSocket: public IPSocket{
public:
	UDPSocket(const UINT16& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET):IPSocket(SOCK_DGRAM,0,localPort, localIP, ipVersion){}
	virtual ~UDPSocket(){}
	virtual INT32 getType(){return SOCK_DGRAM;}
protected: 
	UDPSocket(const UDPSocket& ths);
	virtual UDPSocket& operator=(const UDPSocket& rhs);
};

class TCPSocket: public IPSocket{
public:
	TCPSocket(const UINT16& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET):IPSocket(SOCK_STREAM,0,localPort, localIP, ipVersion){}
	TCPSocket(SOCKET bindedSocket, IP_VERSION ipVersion = IP_VERSION_NOT_SET):IPSocket(bindedSocket, ipVersion){}
	virtual ~TCPSocket(){}
	virtual INT32 getType(){ return SOCK_STREAM;}
	virtual INT32 receiveFrom(UINT8 *data, UINT32 dataLen, string& ipAddr, UINT16& port){ return SOCKET_ERROR;}
	virtual INT32 sendTo(const UINT8 * data, UINT32 dataLen, const string& ipAddr, UINT16 port){ return SOCKET_ERROR;}
protected: 
	TCPSocket(const TCPSocket& rhs);
	/*virtual*/ TCPSocket& operator=(const TCPSocket& rhs);
};

class ICMPSocket: public IPSocket{
public:
	struct ParsedPacket {
		UINT16	type;
		string  ipAddr;
		UINT16	port;
		UINT32	dataSize;
		UINT8 *	data;
	};

	ICMPSocket(const UINT16& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET):IPSocket(SOCK_RAW, IPPROTO_ICMP,localPort, localIP, ipVersion){}
	virtual ~ICMPSocket(){}
	virtual INT32 getType(){return SOCK_RAW;}

#ifdef _WIN32
	bool parseDFSet(UINT8 *packet, UINT32 packetLen, ParsedPacket& parsedPacket);
#endif

protected: 
	ICMPSocket(const ICMPSocket& ths);
	virtual ICMPSocket& operator=(const ICMPSocket& rhs);
};


class IPServerSocket: public IPSocket{
public:
	IPServerSocket(const INT32& type, const INT32& protocol =0, const UINT16& localPort=0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET);
	virtual ~IPServerSocket(){}

	virtual Socket *accept();
protected:
	IPServerSocket(const IPServerSocket& ths);
	/*virtual*/ IPServerSocket& operator=(const IPServerSocket& rhs);
};

class TCPServerSocket: public IPServerSocket{
public:
	TCPServerSocket(const UINT16& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET):IPServerSocket(SOCK_STREAM,0,localPort, localIP, ipVersion){}
	virtual ~TCPServerSocket(){}

	virtual INT32 getType(){ return SOCK_STREAM;}
	virtual INT32 receiveFrom(UINT8 *data, UINT32 dataLen, string& ipAddr, UINT16& port){ return SOCKET_ERROR;}
	virtual INT32 sendTo(const UINT8 * data, UINT32 dataLen, const string& ipAddr, UINT16 port){ return SOCKET_ERROR;}
protected: 

};

// SSLSocket is used as TLS interface.
class SSLSocket: public TCPSocket{

public:
	static SPtr<Socket> createSSLSocket(string caFile, string caPath, string privCert, string cert_pwd, string client_cert, IP_VERSION ipVersion, const string &cipher, bool enhancedTls, SOCKET bindedSocket = NULL);
	virtual SOCKET_LIB_RESULT handshake(const string& ipAddr , const UINT16)=0;
	virtual ~SSLSocket();

protected:
	SSLSocket(const UINT16 localPort=0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET);
	SSLSocket(SOCKET bindedSocket, IP_VERSION ipVersion= IP_VERSION_NOT_SET);


};

#ifdef _WIN32
class WinSSLSocket: public SSLSocket{
		
public:
	WinSSLSocket(const USHORT& localPort = 0, const string& localIP = "", IP_VERSION ipVersion = IP_VERSION_NOT_SET, string clientCert = "", bool enhancedTls = false);
	WinSSLSocket(SOCKET bindedSocket, IP_VERSION ipVersion = IP_VERSION_NOT_SET, string clientCert = "", bool enhancedTls = false);
	
	virtual ~WinSSLSocket();	
	SOCKET_LIB_RESULT WinSSLSocket::connect(const string& ipAddr, const UINT16 port);
	virtual SOCKET_LIB_RESULT handshake(const string& ipAddr, const UINT16 port);
	//virtual INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout);
	// Get internal buffer date was received in the last received call. Usefull just for TLS-SSPI functionality.
	virtual INT32 getInternalBUffer(UINT8 *&data, UINT32 dataLen);
	
	virtual int close();
	
	static bool getIsMutual();
	static void setIsMutual(bool isMutual);

	using SSLSocket::handshake;

protected:
	virtual int prepareDataToSend(const BYTE*& data, int dataLen,const BYTE*& preparedData, int &envelopedDataLen);
	virtual int prepareReceivedData(BYTE*& data, int bytesReceived);
	virtual INT32 copyTempBuffer(UINT8 *&data, UINT32 dataLen, int totalReceived);
	virtual bool _isTLS(){ return true;}
	virtual bool _readingCompleted();
	virtual int _reservedPlace(int dataLen);

private:
	SOCKET_LIB_RESULT _buildClientCredentials();
	SOCKET_LIB_RESULT _getContextAttributes();
	SOCKET_LIB_RESULT _handshakeProcess();
	string _clientCertName;
	bool _enhancedTls;
	int _getClientCerts();

	void _printLogMsg(string functionName, SECURITY_STATUS errCode);


	// Client credentials
	SCHANNEL_CRED _clientCred;
	CredHandle _credsHandle;
	PCCERT_CONTEXT _clientCerts;
	ALG_ID _alg;

	// Context handle
	CtxtHandle _ctxtHandle;

	// Message attributes
	SecPkgContext_StreamSizes _msgSizes; 
	std::vector<BYTE> _outStream, _inStream;
	int _maxMsgLen, _inStreamIndex;

	Semaphore _decSem;
	LPSTR _ipAddr;
	bool _connectionEstablished;
	bool _encryptMessage;

	// There are special requirements for the Intel AMT client certificates. The certificate needs to contain the following OIDs:
	// 1.3.6.1.5.5.7.3.2 OID - marks the certificate as a TLS client certificate.
	// 2.16.840.1.113741.1.2.1 OID - used by the Intel AMT device to authenticate the Redirection Library.
	static const LPSTR _TLSClientOID;
	static const DWORD _TLSClientOIDLen;
	static const LPSTR _AMTClientOID;
	static const DWORD _AMTClientOIDLen;
	static bool _isMutual;

	std::vector<BYTE> _internalBuffer;
	int _internalBufferIndex;
};

#else

class OpenSSLSocket: public SSLSocket{
public:
	OpenSSLSocket(string ca_file, string ca_path, string priv_cert, string cert_pass, IP_VERSION ipVersion = IP_VERSION_NOT_SET, const bool tls = true, const string &cipher = "", const USHORT& localPort = 0, const string& localIP = "");
	OpenSSLSocket(SOCKET bindedSocket, string ca_file, string ca_path, string priv_cert, string cert_pass, IP_VERSION ipVersion = IP_VERSION_NOT_SET, const bool tls = true, const string &cipher = "");
	virtual ~OpenSSLSocket();
	virtual SOCKET_LIB_RESULT connect(const string& ipAddr, const UINT16 port);
	virtual INT32 send(const UINT8 * data, UINT32 dataLen);
	virtual INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout);
	virtual INT32 close();

	virtual SOCKET_LIB_RESULT handshake(const string& ipAddr, const UINT16 port);
	class SharedLocks
	{
	public:
		SharedLocks();
		~SharedLocks();
		static void Init();
		Semaphore& operator [] (int index) { return _sharedLocks[index]; }
		const Semaphore& operator [] (int index) const { return _sharedLocks[index]; }

	private:
		SharedLocks(SharedLocks& proxy);
		SharedLocks& operator=(const SharedLocks& proxy);
		static Semaphore* _sharedLocks;
	};
protected:

	SOCKET_LIB_RESULT _verifyCert(string ipAddr);

	SSL_CTX *_ctx; // SSL contex
	SSL *_con;  // SSL connection, using the context m_ctx
	bool _tls;  // are we TLS(SSLv3.1) or SSLv3
	string _cipher; // cipher string
	string _caFile;
	string _caPath;
	string _privCert;
	string _certPass;
	X509 * _peerCert;

	// wrapper that initialize on the first time openssl
	static SharedLocks _sharedLocks;
	static void _locking_callback(INT32 mode, INT32 n, const char *file, INT32 line);
	static INT32 _passwd_callback(char *buf, INT32 size, INT32 rwflag, void *userdata);
	virtual bool _isTLS(){return true;}
};

#endif	//_WIN32

class SOCKSSocket: public TCPSocket{
public:
	SOCKSSocket(const ProxySettings &proxy_settings, string ca_file, string ca_path, string priv_cert, string cert_pass, string clientCert, IP_VERSION ipVersion = IP_VERSION_NOT_SET, const bool enhancedTls = false, const bool tls = true, const string &cipher = "", const UINT16& localPort = 0, const string& localIP = "");
	SOCKSSocket(const ProxySettings &proxy_settings,IP_VERSION ipVersion = IP_VERSION_NOT_SET, const UINT16& localPort = 0, const string& localIP = "");
	virtual ~SOCKSSocket();
	virtual INT32 getType();
	virtual SOCKET_LIB_RESULT connect(const string& ipAddr, const UINT16 port);
	virtual INT32 send(const UINT8 * data, UINT32 dataLen);
	virtual INT32 receive(UINT8 *data, UINT32 dataLen, INT32 timeout);
	virtual INT32 close();
	virtual bool setNonBlocking( bool enable = true );

protected:
	bool _userPassAuth(string proxyUser, string proxyPass);

	const ProxySettings _proxy_settings;
	bool _ssl; // are we SSL or TCP
	SPtr<Socket> _internal_socket;

	// SSL parameters
	bool _tls;  // are we TLS(SSLv3.1) or SSLv3
	bool _enhancedTls;
	string _cipher; // cipher string
	string _caFile;
	string _caPath;
	string _privCert;
	string _certPass;
	// SSL_SSPI parameter
	string _clientCert;
};

#endif //_SOCKET_LIB__H_

