//  Copyright (C) 2007 Intel Corporation
#include "AuthenticationManager.h"
#include "AuthenticatorDigest.h"
#include "AuthenticatorBasic.h"
#include "Logger.h"
#ifdef _WIN32
#include "AuthenticatorKerberos.h"
#endif

AuthenticationManager::AuthenticationManager(SPtr<Socket> socket, 
											 AuthenticationInfo& info)
{
	m_socket = socket;
	m_username = info.UserName;
	m_password = info.Password;
	m_ip = info.Ip;
	m_domain = info.Domain;
	m_port = info.Port;
}

AuthenticationManager::~AuthenticationManager(void)
{
}

AuthenticationResult	AuthenticationManager::OpenSession()
{
	AuthenticationResult status = AuthenticationResultFailure;
	do
	{
		// start redirection messages
		if(AuthenticationResultSuccess != (status = startRedirection()))
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "start redirection failed with status: " << status << endl;
			break;
		}
		// start authentication negotiation support DIGEST, KERBEROS
		if(AuthenticationResultSuccess != (status = authenticate()))
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "authenticate failed with status: " << status << endl;
			break;
		}
		// start KVM Redirection messages
		if(AuthenticationResultSuccess != (status = startKVMRedirection()))
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "start KVM redirection failed with status: " << status << endl;
			break;
		}
	}while(0);

	return status;
}

AuthenticationResult	AuthenticationManager::startRedirection()
{
	PacketBuffer buffer;
	prepareStartRedirection(buffer);
	// send start redirection
	if (m_socket->send(buffer.data, buffer.length) != buffer.length)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticationManager::startRedirection> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		

	// receive start redirection reply
	buffer.Memset();
	buffer.length = m_socket->receive(buffer.data, sizeof(MSG_StartRedirectionSessionReply), RECEIVE_TIMEOUT);
	if (buffer.length != sizeof(MSG_StartRedirectionSessionReply))
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticationManager::startRedirection> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}

	MSG_StartRedirectionSessionReply* reply = (MSG_StartRedirectionSessionReply*)buffer.data;
	//Note that here the returned status is NOT an authentication status:
	switch (reply->hdr.status)
	{
	case STATUS_SUCCESS: return AuthenticationResultSuccess;
	case STATUS_BUSY: return AuthenticationResultClientBusy;
	case STATUS_UNSUPPORTED: return AuthenticationResultUnsupported;
	}
	
	return AuthenticationResultFailure;
}

AuthenticationResult	AuthenticationManager::authenticate()
{
	AuthenticationResult  res = AuthenticationResultFailure;
	vector<UINT8> authQuery;

	// send QUERY auth message
	res = authenticationQuery(authQuery);
	if (AuthenticationResultSuccess != res) 
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "Failed query supported authentication types" << res << endl;
		return AuthenticationResultFailure;
	}

	// try each supported authentication type in order
	for (UINT32 i = 0; i < authQuery.size(); i++) 
	{
		// assume an error
		res = AuthenticationResultFailure;

		// skip kerberos authentication when user name and password are specified
		if (AUTH_TYPE_KERBEROS == authQuery[i] && !m_username.empty() && m_domain.empty() && !m_password.empty())
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "User name and password are specified. Skipping kerberos " << res << endl;
			continue;
		}
				

		if(AUTH_TYPE_KERBEROS == authQuery[i])
		{
			#ifdef _WIN32
			AuthenticatorKerberos kerberos(m_socket, m_ip, m_port, m_username, m_password, m_domain);
			res = kerberos.Authenticate();
			Logger::GetInstance() << "Trying kerberos Authentication result: " << res << endl;
			#else
			Logger::GetInstance() << "Authentication kerberos not supported under non Windows systems" << endl;
			#endif
			break;
		}
		// Digest Authentication should be added
		else if(AUTH_TYPE_DIGEST2 == authQuery[i])
		{
			// TODO: workaround for Linux, currently we do not support Digest authentication on Linux so skip to Basic Auth only on Linux
			AuthenticatorDigest digest(m_socket, m_username, m_password, AUTH_TYPE_DIGEST2);
			res = digest.Authenticate();
			Logger::GetInstance() << "Trying digest2 Authentication result: " << res << endl;
		}

		// Digest Authentication should be added
		else if(AUTH_TYPE_DIGEST == authQuery[i])
		{
			// TODO: workaround for Linux, currently we do not support Digest authentication on Linux so skip to Basic Auth only on Linux
			AuthenticatorDigest digest(m_socket, m_username, m_password);
			res = digest.Authenticate();
			Logger::GetInstance() << "Trying digest Authentication result: " << res << endl;
		}
		else
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "Authentication not supported : " << authQuery[i] << endl;
			continue;
		}
		if(AuthenticationResultSuccess == res || AuthenticationResultAuditLogFailure == res)
		{
			break;
		}
	}

	return res;
}

AuthenticationResult	AuthenticationManager::startKVMRedirection()
{
	AuthenticationResult status = AuthenticationResultFailure;
	PacketBuffer buffer;
	do
	{
		buffer.Memset();
		// send start kvm redirection
		MSG_StartKVMRedirection* send = (MSG_StartKVMRedirection*)buffer.data;
		send->KVMMessageType = StartKVMRedirection;
		buffer.length = sizeof(MSG_StartKVMRedirection);
		if(m_socket->send(buffer.data, buffer.length) != buffer.length)
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticationManager::startKVMRedirection> Socket fail send" << endl;
			status = AuthenticationResultFailure;
			break;
		}
		
		buffer.Memset();
		// receive start kvm redirection reply
		buffer.length = m_socket->receive(buffer.data, sizeof(MSG_StartKVMRedirectionReply), RECEIVE_TIMEOUT);
		if(buffer.length != sizeof(MSG_StartKVMRedirectionReply))
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticationManager::startKVMRedirection> Socket fail receive" << endl;
			status = AuthenticationResultFailure;
			break;
		}
		// verify that the kvm redirection reply status is success
		MSG_StartKVMRedirectionReply* reply = (MSG_StartKVMRedirectionReply*)buffer.data;
		if(STATUS_SUCCESS != reply->u.Status)
		{
			status = AuthenticationResultFailure;
			break;
		}

		status = AuthenticationResultSuccess;

	}while(0);

	return status;
}

AuthenticationResult	AuthenticationManager::authenticationQuery(vector<UINT8>& authQuery)
{
	AuthenticationResult status = AuthenticationResultFailure;
	PacketBuffer buffer;
	do
	{
		// send authentication query
		prepareAuthMsg(buffer, AUTH_TYPE_QUERY);
		if(m_socket->send(buffer.data, buffer.length) != buffer.length)
		{
			status = AuthenticationResultFailure;
			break;
		}

		buffer.Memset();

		// receive authentication query reply
		buffer.length = m_socket->receive(buffer.data, sizeof(MSG_AuthenticateSessionReply), RECEIVE_TIMEOUT);
		if(buffer.length != sizeof(MSG_AuthenticateSessionReply))
		{
			status = AuthenticationResultFailure;
			break;
		}
		
		MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)buffer.data;
		if(AUTH_STATUS_SUCCESS == reply->hdr.status)
		{
			UINT32 tmp = m_socket->receive(buffer.data + buffer.length, reply->dataLength, RECEIVE_TIMEOUT);
			if(tmp != reply->dataLength)
			{
				status = AuthenticationResultFailure;
				break;
			}
			buffer.length += reply->dataLength;

			for(UINT32 i=0; i<reply->dataLength; i++)
				authQuery.push_back(reply->data[i]);
		}
		status = AuthenticationResultSuccess;
	}while(0);

	return status;
}

void	AuthenticationManager::prepareAuthMsg(PacketBuffer& packet, int authType)
{
	MSG_AuthenticateSession *authMsg = (MSG_AuthenticateSession *)packet.data;	
	authMsg->hdr.type = AuthenticateSession;
	authMsg->authType = authType;
	packet.length = sizeof(MSG_AuthenticateSession);
}

void	AuthenticationManager::prepareStartRedirection(PacketBuffer& packet)
{
	MSG_StartRedirectionSession *StartMsg = (MSG_StartRedirectionSession *)packet.data;
	packet.length = sizeof(MSG_StartRedirectionSession);
	memset(StartMsg, 0, sizeof(MSG_StartRedirectionSession));
	StartMsg->hdr.type = StartRedirectionSession;
	// turn on the LINK PREFERENCE SUPPORT flag
	StartMsg->hdr.linkPrefernce = LINK_PREFERENCE_SUPPORT_EXISTS;
	if (memcpy_s(StartMsg->SessionType, ARRAYSIZE(StartMsg->SessionType), START_REDIRECTION_PROTOCOL.c_str(), START_REDIRECTION_PROTOCOL.size()))
	{
		printf("Error: Can't Copy Memory");
	}
}
