//  Copyright (C) 2007 Intel Corporation
#include "AuthenticatorDigest.h"
#include <string>
#include <iostream>
#include "digcalc.h"
#include <time.h>
#ifndef _WIN32
#include <math.h>
#include <openssl/md5.h>
#endif

using namespace std;

static const char* const DigestMethod = "POST";
static const char* const DigestNC = "00000002";
static const char* const Digest2Qop = "auth";
static const UINT32 const DataSize = 8192; // packetbuffer initalize it's data field with size 8192

AuthenticatorDigest::AuthenticatorDigest(SPtr<Socket> socket, string user, string pwd, UINT8 digestType) : Authenticator(socket)
{
	m_user = user;
	m_pwd = pwd;
	m_digestType = digestType;
}

AuthenticatorDigest::~AuthenticatorDigest(void)
{
}

/**
	Authenticate function will perform the Digest Authentication
*/
AuthenticationResult AuthenticatorDigest::Authenticate()
{
	if (AUTH_TYPE_DIGEST2 == m_digestType)
	{
		return Authenticate2();
	}
	AuthenticationResult status;
	// perform first time authentication only with the username
	// the response should include structure with details
	MSG_DigestAuthentication digestResponse;
	memset(&digestResponse, 0, sizeof(MSG_DigestAuthentication));
	status = authFirst(digestResponse);
	if(AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::Authenticate> First authentication (only user name) failed" << endl;
		return status;
	}

	// fill the MSG_DigestAuthentication struct with the digest and authenticate
	status = authSecond(digestResponse);
	if(AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::Authenticate> Second authentication failed" << endl;
		return status;
	}

	//cout << "Console Send" << endl;
	//printDigestResponse(digestResponse);

	// prepare the second attempt with credentials
	PacketBuffer buffer;
	prepareAuthMsg(&buffer, AUTH_TYPE_DIGEST);

	// fill the packet buffer with the MSG_DigestAuthentication struct
	fillBuffer(buffer, digestResponse);
	
	// update the data length field
	MSG_AuthenticateSession* authMsg = (MSG_AuthenticateSession*)buffer.data;
	authMsg->dataLength = buffer.length - sizeof(MSG_AuthenticateSession);

	// send the buffer
	if (m_socket->send(buffer.data, buffer.length) != buffer.length)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::Authenticate2> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		

	// wait for a response
	buffer.Memset();

	buffer.length = m_socket->receive(buffer.data, sizeof(MSG_AuthenticateSessionReply), RECEIVE_TIMEOUT);
	if (buffer.length != sizeof(MSG_AuthenticateSessionReply))
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorDigest2::Authenticate2> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}

	
	// make sure AUTH_STATUS_SUCCESS returned in the response
	MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)buffer.data;

	return AMTRes2AuthenticationResult(reply->hdr.status);
}
/**
	Authenticate function will perform the Digest Authentication
*/
AuthenticationResult AuthenticatorDigest::Authenticate2()
{
	AuthenticationResult status;
	// perform first time authentication only with the username
	// the response should include structure with details
	MSG_DigestAuthentication2 digestResponse;
	memset(&digestResponse, 0, sizeof(MSG_DigestAuthentication2));
	status = authFirst(digestResponse);
	if(AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::Authenticate2> First authentication (only user name) failed" << endl;
		return status;
	}

	// fill the MSG_DigestAuthentication2 struct with the digest and authenticate
	status = authSecond(digestResponse);
	if(AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::Authenticate2> Second authentication failed" << endl;
		return status;
	}

	//cout << "Console Send" << endl;
	//printDigestResponse(digestResponse);

	// prepare the second attempt with credentials
	PacketBuffer buffer;
	prepareAuthMsg(&buffer, AUTH_TYPE_DIGEST2);

	// fill the packet buffer with the MSG_DigestAuthentication2 struct
	fillBuffer(buffer, digestResponse);
	
	// update the data length field
	MSG_AuthenticateSession* authMsg = (MSG_AuthenticateSession*)buffer.data;
	authMsg->dataLength = buffer.length - sizeof(MSG_AuthenticateSession);

	// send the buffer
	if (m_socket->send(buffer.data, buffer.length) != buffer.length)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::Authenticate2> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		

	// wait for a response
	buffer.Memset();

	buffer.length = m_socket->receive(buffer.data, sizeof(MSG_AuthenticateSessionReply), RECEIVE_TIMEOUT);
	if (buffer.length != sizeof(MSG_AuthenticateSessionReply))
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::Authenticate2> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}
		

	
	// make sure AUTH_STATUS_SUCCESS returned in the response
	MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)buffer.data;

	return AMTRes2AuthenticationResult(reply->hdr.status);
}

void AuthenticatorDigest::printDigestResponse(MSG_DigestAuthentication& digestResponse)
{
	cout << "Printing digest reply" << endl;
	cout << "-----------------------" << endl;
	
	string s1((const char*)digestResponse.UserName, digestResponse.UserNameLength);
	cout << "Username: " << s1 << endl;

	string s2((const char*)digestResponse.Service, digestResponse.ServiceLength);
	cout << "Service: " << s2 << endl;

	cout << "Cnonce: " ;
	for(int i=0; i<digestResponse.CNonceLength; i++)
	{
		cout << hex << (unsigned int)digestResponse.CNonce[i];
	}
	cout << endl;

	cout << "NC: " ;
	for(int i=0; i<digestResponse.NCLength; i++)
	{
		cout << hex << (unsigned int)digestResponse.NC[i];
	}
	cout << endl;

	cout << "Nonce: " ;
	for(int i=0; i<digestResponse.NonceLength  ; i++)
	{
		cout << hex << (unsigned int)digestResponse.Nonce[i];
	}
	cout << endl;

	cout << "Realm: " ;
	for(int i=0; i<digestResponse.RealmLength; i++)
	{
		cout << hex << (unsigned int)digestResponse.Realm[i];
	}
	cout << endl;

	cout << "Response " ;
	for(int i=0; i<digestResponse.ResponseLength  ; i++)
	{
		cout << hex << (unsigned int)digestResponse.Response[i];
	}
	cout << endl;
}

/**
	the function will calculate the MD5 hash based on the fields of MSG_DigestAuthentication
	and will set the result in the field Response in addition it will set all the
	other neccessary fields
*/
AuthenticationResult AuthenticatorDigest::authSecond(MSG_DigestAuthentication& digestResponse, char* qop)
{
	HASHHEX HA1;
    HASHHEX HA2 = "";
    HASHHEX response;

	char realm[MAX_LENGTH_REALM_DIGEST + 1];
	memset(realm, 0, ARRAYSIZE(realm));
	if (memcpy_s(realm, ARRAYSIZE(realm), digestResponse.Realm, ARRAYSIZE(digestResponse.Realm)))
	{
		return AuthenticationResultFailure;		
	}

	char nonce[MAX_LENGTH_NONCE_DIGEST + 1];
	memset(nonce, 0, ARRAYSIZE(nonce));
	if (memcpy_s(nonce, ARRAYSIZE(nonce), digestResponse.Nonce, ARRAYSIZE(digestResponse.Nonce)))
	{
		return AuthenticationResultFailure;
	}

	char cnonce[MAX_LENGTH_CNONCE_DIGEST + 1];
	memset(cnonce, 0, ARRAYSIZE(cnonce));
	GenerateCNonce(cnonce);
	digestResponse.CNonceLength = (UINT8)strlen(cnonce);
	if (memcpy_s(digestResponse.CNonce, ARRAYSIZE(digestResponse.CNonce), cnonce, ARRAYSIZE(cnonce)))
	{
		return AuthenticationResultFailure;
	}

	char serviceUri[MAX_LENGTH_SERVICE_DIGEST + 1];
	memset(serviceUri, 0, ARRAYSIZE(serviceUri));
	if (memcpy_s(serviceUri, ARRAYSIZE(serviceUri), digestResponse.Service, ARRAYSIZE(digestResponse.Service)))
	{
		return AuthenticationResultFailure;
	}

	

	// calculate the Digest response (notice the method=POST, and the qop-value="" or "auth")
	DigestCalcHA1("MD5", (char*)m_user.c_str(), realm, (char*)m_pwd.c_str(), nonce, cnonce, HA1);
	DigestCalcResponse(HA1, nonce, (char*)DigestNC, cnonce, qop, (char*)DigestMethod, serviceUri,
                       HA2, response);
	
	digestResponse.ResponseLength = (UINT8)strlen(response);
	if (memcpy_s(digestResponse.Response, ARRAYSIZE(digestResponse.Response), response, ARRAYSIZE(response)))
	{
		return AuthenticationResultFailure;
	}

	UINT8 digestNcLen = (UINT8)strlen(DigestNC);
	digestResponse.NCLength = digestNcLen;
	if (digestNcLen >= sizeof(digestResponse.NC))
	{
		return AuthenticationResultFailure;
	}
	digestResponse.NCLength = digestNcLen;
	if (memcpy_s(digestResponse.NC, ARRAYSIZE(digestResponse.NC), DigestNC, digestResponse.NCLength))
	{
		return AuthenticationResultFailure;
	}

	return AuthenticationResultSuccess;
}

/**
	the function will calculate the MD5 hash based on the fields of MSG_DigestAuthentication2
	and will set the result in the field Response in addition it will set all the
	other neccessary fields
*/
AuthenticationResult AuthenticatorDigest::authSecond(MSG_DigestAuthentication2& digestResponse)
{
	if (strncmp((char*)digestResponse.Qop, Digest2Qop, digestResponse.QopLength) != 0)
	{
		return AuthenticationResultFailure;
	}
	char qop[MAX_LENGTH_QOP_DIGEST + 1];
	memset(qop, 0, ARRAYSIZE(qop));
	if (memcpy_s(qop, ARRAYSIZE(qop), digestResponse.Qop, ARRAYSIZE(digestResponse.Qop)))
	{
		return AuthenticationResultFailure;
	}
	return authSecond(digestResponse.DigestAuth, (char*) qop);
}

/**
		The function receive a buffer with format of <UINT8-length, UINT8*-buffer> 
		and set the apropriate <length, buffer> pair in the MSG_DigestAuthentication struct
		according to the field that was received.
		Param:
			UINT8* buffer - the actual buffer
			UINT32& buffer_length - buffer length will be updated after each set
			MSG_DigestAuthentication& digestResponse - the struct that will be set
			DIGEST_FIELDS field - identify the field in the MSG_DigestAuthentication
			UINT32& OUT_indexBuffer - an offset to access the buffer
*/
AuthenticationResult	AuthenticatorDigest::parseBufferLength(UINT8* buffer, UINT32& buffer_length,
												   MSG_DigestAuthentication& digestResponse, 
												   DIGEST_FIELDS field, 
												   UINT32& OUT_indexBuffer)
{
	if(buffer_length < 1)
	{
		return AuthenticationResultFailure;
	}
	buffer_length--;
	UINT32 fieldLength = static_cast<UINT32>(buffer[OUT_indexBuffer]);
	OUT_indexBuffer++;

	if(buffer_length < fieldLength)
	{
		return AuthenticationResultFailure;
	}

	switch(field)
	{
		case DIGEST_FIELD_USERNAME:
			if(MAX_LENGTH_USER_NAME_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.UserNameLength = fieldLength;
			if (memcpy_s(digestResponse.UserName, ARRAYSIZE(digestResponse.UserName), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
			
		break;
		case DIGEST_FIELD_REALM:
			if(MAX_LENGTH_REALM_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.RealmLength = fieldLength;
			if (memcpy_s(digestResponse.Realm, ARRAYSIZE(digestResponse.Realm), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		case DIGEST_FIELD_NONCE:
			if(MAX_LENGTH_NONCE_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.NonceLength = fieldLength;
			if (memcpy_s(digestResponse.Nonce, ARRAYSIZE(digestResponse.Nonce), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		case DIGEST_FIELD_SERVICE:
			if(MAX_LENGTH_SERVICE_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.ServiceLength = fieldLength;
			if (memcpy_s(digestResponse.Service, ARRAYSIZE(digestResponse.Service), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		case DIGEST_FIELD_CNONCE:
			if(MAX_LENGTH_CNONCE_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.CNonceLength = fieldLength;
			if (memcpy_s(digestResponse.CNonce, ARRAYSIZE(digestResponse.CNonce), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		case DIGEST_FIELD_NC:
			if(MAX_LENGTH_NC_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.NCLength = fieldLength;
			if (memcpy_s(digestResponse.NC, ARRAYSIZE(digestResponse.NC), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		case DIGEST_FIELD_RESPONSE:
			if(MAX_LENGTH_RESPONSE_DIGEST < fieldLength)
			{
				return AuthenticationResultFailure;
			}
			digestResponse.ResponseLength = fieldLength;
			if (memcpy_s(digestResponse.Response, ARRAYSIZE(digestResponse.Response), buffer + OUT_indexBuffer, fieldLength))
			{
				return AuthenticationResultFailure;
			}
		break;
		default:
			return AuthenticationResultFailure;
	}
	buffer_length -= fieldLength;
	OUT_indexBuffer += fieldLength;
	return AuthenticationResultSuccess;
}

/**
		The function receive a buffer with format of <UINT8-length, UINT8*-buffer> 
		and set the apropriate <length, buffer> pair in the MSG_DigestAuthentication2 struct
		according to the field that was received.
		Param:
			UINT8* buffer - the actual buffer
			UINT32& buffer_length - buffer length will be updated after each set
			MSG_DigestAuthentication& digestResponse - the struct that will be set
			DIGEST_FIELDS field - identify the field in the MSG_DigestAuthentication
			UINT32& OUT_indexBuffer - an offset to access the buffer
*/
AuthenticationResult	AuthenticatorDigest::parseBufferLength(UINT8* buffer, UINT32& buffer_length,
												   MSG_DigestAuthentication2& digestResponse, 
												   DIGEST_FIELDS field, 
												   UINT32& OUT_indexBuffer)
{
	if (field != DIGEST_FIELD_QOP)
	{
		return parseBufferLength(buffer, buffer_length, digestResponse.DigestAuth, field, OUT_indexBuffer);
	}
	if(buffer_length < 1)
	{
		return AuthenticationResultFailure;
	}
	buffer_length--;
	UINT32 fieldLength = static_cast<UINT32>(buffer[OUT_indexBuffer]);
	OUT_indexBuffer++;
	// handle just qop field:

	if(buffer_length < fieldLength)
	{
		return AuthenticationResultFailure;
	}

	if(MAX_LENGTH_QOP_DIGEST < fieldLength)
	{
		return AuthenticationResultFailure;
	}
	digestResponse.QopLength = fieldLength;
	if (memcpy_s(digestResponse.Qop, ARRAYSIZE(digestResponse.Qop), buffer + OUT_indexBuffer, fieldLength))
	{
		return AuthenticationResultFailure;
	}

	buffer_length -= fieldLength;
	OUT_indexBuffer += fieldLength;
	return AuthenticationResultSuccess;
}


AuthenticationResult AuthenticatorDigest::parseDigestResponseRealmNonce(PacketBuffer& packet, 
													MSG_DigestAuthentication& digestResponse)
{
	MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)packet.data;
	// make sure the status in the header field is success and the response we
	// receive is digest
	if(AUTH_STATUS_FAILURE != reply->hdr.status || AUTH_TYPE_DIGEST != reply->authType)
	{
		return AuthenticationResultFailure;
	}
	
	UINT32 index = 0;
	UINT32 msgDigestLength = reply->dataLength;

	AuthenticationResult status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_REALM, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_NONCE, index)) != AuthenticationResultSuccess )
		return status;

	return (0 == msgDigestLength) ? AuthenticationResultSuccess : AuthenticationResultFailure;
}

AuthenticationResult AuthenticatorDigest::parseDigestResponseRealmNonce(PacketBuffer& packet, 
													MSG_DigestAuthentication2& digestResponse)
{
	MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)packet.data;
	// make sure the status in the header field is success and the response we
	// receive is digest
	if(AUTH_STATUS_FAILURE != reply->hdr.status || AUTH_TYPE_DIGEST2 != reply->authType)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorDigest2::parseDigestResponseRealmNonce> incorrect authentication type or packet status failure" << endl;
		return AuthenticationResultFailure;
	}
	
	UINT32 index = 0;
	UINT32 msgDigestLength = reply->dataLength;

	AuthenticationResult status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_REALM, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_NONCE, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_QOP, index)) != AuthenticationResultSuccess )
		return status;

	return (0 == msgDigestLength) ? AuthenticationResultSuccess : AuthenticationResultFailure;
}

/*
	The function will initialize the MSG_DigestAuthentication struct based on the
	Packetbuffer that was received.
*/
AuthenticationResult AuthenticatorDigest::parseDigestResponse(PacketBuffer& packet, 
													MSG_DigestAuthentication& digestResponse)
{
	MSG_AuthenticateSessionReply* reply = (MSG_AuthenticateSessionReply*)packet.data;
	// make sure the status in the header field is success and the response we
	// receive is digest
	if(AUTH_STATUS_FAILURE != reply->hdr.status || (AUTH_TYPE_DIGEST != reply->authType && AUTH_TYPE_DIGEST2 != reply->authType))
	{
		return AuthenticationResultFailure;
	}
	
	UINT32 index = 0;
	UINT32 msgDigestLength = reply->dataLength;

	AuthenticationResult status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_USERNAME, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_REALM, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_NONCE, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_SERVICE, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_CNONCE, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_NC, index)) != AuthenticationResultSuccess )
		return status;
	if ( (status = parseBufferLength(reply->data, msgDigestLength, 
									 digestResponse, 
									 DIGEST_FIELD_RESPONSE, index)) != AuthenticationResultSuccess )
		return status;

	return (0 == msgDigestLength) ? AuthenticationResultSuccess : AuthenticationResultFailure;
}

/**
	Fill the packet buffer based on the MSG_DigestAuthentication
*/
void AuthenticatorDigest::fillBuffer(PacketBuffer& packet, MSG_DigestAuthentication& request)
{
	UINT32 index = 0;
	index += sizeof(MSG_AuthenticateSession);

	// copy the user fields
	packet.data[index] = request.UserNameLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.UserName, request.UserNameLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.UserNameLength;
	// copy the realm fields
	packet.data[index] = request.RealmLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.Realm, request.RealmLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.RealmLength;
	// copy the nonce fields
	packet.data[index] = request.NonceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.Nonce, request.NonceLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.NonceLength;
	// copy the service fields
	packet.data[index] = request.ServiceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.Service, request.ServiceLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.ServiceLength;
	// copy the CNonce fields
	packet.data[index] = request.CNonceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.CNonce, request.CNonceLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.CNonceLength;
	// copy the NC fields
	packet.data[index] = request.NCLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.NC, request.NCLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.NCLength;
	// copy the Response fields
	packet.data[index] = request.ResponseLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.Response, request.ResponseLength))
	{
		printf("copy memory error");
		return;
	}
	index += request.ResponseLength;

	// update the length of the packet to be the previous length + the additional data
	UINT32 dataLength = (index - sizeof(MSG_AuthenticateSession));
	packet.length += dataLength;

	MSG_AuthenticateSession* authMsg = (MSG_AuthenticateSession*)packet.data;
	authMsg->dataLength = dataLength;
}

/**
	Fill the packet buffer based on the MSG_DigestAuthentication2
*/
void AuthenticatorDigest::fillBuffer(PacketBuffer& packet, MSG_DigestAuthentication2& request)
{
	UINT32 index = 0;
	index += sizeof(MSG_AuthenticateSession);

	// copy the user fields
	packet.data[index] = request.DigestAuth.UserNameLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.UserName, request.DigestAuth.UserNameLength))
	{
		return;
	}
	index += request.DigestAuth.UserNameLength;
	// copy the realm fields
	packet.data[index] = request.DigestAuth.RealmLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.Realm, request.DigestAuth.RealmLength))
	{
		return;
	}
	index += request.DigestAuth.RealmLength;
	// copy the nonce fields
	packet.data[index] = request.DigestAuth.NonceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.Nonce, request.DigestAuth.NonceLength))
	{
		return;
	}
	index += request.DigestAuth.NonceLength;
	// copy the service fields
	packet.data[index] = request.DigestAuth.ServiceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.Service, request.DigestAuth.ServiceLength))
	{
		return;
	}
	index += request.DigestAuth.ServiceLength;
	// copy the CNonce fields
	packet.data[index] = request.DigestAuth.CNonceLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.CNonce, request.DigestAuth.CNonceLength))
	{
		return;
	}
	index += request.DigestAuth.CNonceLength;
	// copy the NC fields
	packet.data[index] = request.DigestAuth.NCLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.NC, request.DigestAuth.NCLength))
	{
		return;
	}
	index += request.DigestAuth.NCLength;
	// copy the Response fields
	packet.data[index] = request.DigestAuth.ResponseLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.DigestAuth.Response, request.DigestAuth.ResponseLength))
	{
		return;
	}
	index += request.DigestAuth.ResponseLength;
	// copy the Qop fields
	packet.data[index] = request.QopLength;
	index++;
	if (memcpy_s(packet.data + index, DataSize - index, request.Qop, request.QopLength))
	{
		return;
	}
	index += request.QopLength;

	// update the length of the packet to be the previous length + the additional data
	UINT32 dataLength = (index - sizeof(MSG_AuthenticateSession));
	packet.length += dataLength;

	MSG_AuthenticateSession* authMsg = (MSG_AuthenticateSession*)packet.data;
	authMsg->dataLength = dataLength;
}

/**
	The function will try to authenticate without pwd
	and using the information that will be received (realm, nonce, cnonce ..)
	it will set the MSG_DigestAuthentication
	param:
		OUT MSG_DigestAuthentication& digestResponse - the structure that will be returned
*/
AuthenticationResult AuthenticatorDigest::authFirst(MSG_DigestAuthentication& digestResponse)
{
	AuthenticationResult status;
	PacketBuffer packet;
	packet.Memset();
	prepareAuthMsg(&packet, AUTH_TYPE_DIGEST);

	// fill in parameters for the first attempt: user, service.
	MSG_DigestAuthentication firstAttempt;
	memset(&firstAttempt, 0, sizeof(MSG_DigestAuthentication));
	firstAttempt.UserNameLength = static_cast<UINT8>(m_user.size());
	if (memcpy_s(firstAttempt.UserName, ARRAYSIZE(firstAttempt.UserName), m_user.c_str(), min((int)m_user.size(), MAX_LENGTH_USER_NAME_DIGEST)))
	{
		return AuthenticationResultFailure;
	}
	firstAttempt.ServiceLength = static_cast<UINT8>(RedirectionService.size());
	if (memcpy_s(firstAttempt.Service, ARRAYSIZE(firstAttempt.Service), RedirectionService.c_str(), RedirectionService.size()))
	{
		return AuthenticationResultFailure;
	}
	fillBuffer(packet, firstAttempt);

	// send the buffer
	if(m_socket->send(packet.data, packet.length) != packet.length)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::authFirst> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}

	// receive params from AMT 
	packet.Memset();
	
	// receive the base msg of MSG_AuthenticateSession
	packet.length = m_socket->receive(packet.data, sizeof(MSG_AuthenticateSession), RECEIVE_TIMEOUT);
	if(packet.length != sizeof(MSG_AuthenticateSession))
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::authFirst> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}

	// receive the data of the MSG_AuthenticateSession
	unsigned int dataLeft = ((MSG_AuthenticateSession*)packet.data)->dataLength;
	packet.length += m_socket->receive(packet.data + sizeof(MSG_AuthenticateSession), dataLeft, RECEIVE_TIMEOUT);
	// check data length
	if (packet.length > packet.size())
	{
		return AuthenticationResultFailure;
	}
	// make sure we receive all the data we expected
	if (packet.length != sizeof(MSG_AuthenticateSession)+dataLeft)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::authFirst> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		
	
	// the following function will parse the data in packet
	// and initialize the digestResponse structure
	status = parseDigestResponseRealmNonce(packet, digestResponse);
	if (AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest::authFirst> Failed to pase the data in packet and initialize the digestResponse structure" << endl;
		return status;
	}
		

	// set the username and service to the digestResponse
	digestResponse.UserNameLength = static_cast<UINT8>(m_user.size());
	if (memcpy_s(digestResponse.UserName, ARRAYSIZE(digestResponse.UserName), m_user.c_str(), min((int)m_user.size(), MAX_LENGTH_USER_NAME_DIGEST)))
	{
		return AuthenticationResultFailure;
	}
	digestResponse.ServiceLength = static_cast<UINT8>(RedirectionService.size());
	if (memcpy_s(digestResponse.Service, ARRAYSIZE(digestResponse.Service), RedirectionService.c_str(), RedirectionService.size()))
	{
		return AuthenticationResultFailure;
	}
	fillBuffer(packet, digestResponse);

	return AuthenticationResultSuccess;
}

/**
	The function will try to authenticate without pwd
	and using the information that will be received (realm, nonce, cnonce ..)
	it will set the MSG_DigestAuthentication
	param:
		OUT MSG_DigestAuthentication& digestResponse - the structure that will be returned
*/
AuthenticationResult AuthenticatorDigest::authFirst(MSG_DigestAuthentication2& digestResponse)
{
	AuthenticationResult status;
	PacketBuffer packet;
	packet.Memset();
	prepareAuthMsg(&packet, AUTH_TYPE_DIGEST2);

	// fill in parameters for the first attempt: user, service.
	MSG_DigestAuthentication2 firstAttempt;
	memset(&firstAttempt, 0, sizeof(MSG_DigestAuthentication2));
	firstAttempt.DigestAuth.UserNameLength = static_cast<UINT8>(m_user.size());
	if (memcpy_s(firstAttempt.DigestAuth.UserName, ARRAYSIZE(firstAttempt.DigestAuth.UserName, ), m_user.c_str(), m_user.size()))
	{
		return AuthenticationResultFailure;
	}
	firstAttempt.DigestAuth.ServiceLength = static_cast<UINT8>(RedirectionService.size());
	if (memcpy_s(firstAttempt.DigestAuth.Service, ARRAYSIZE(firstAttempt.DigestAuth.Service), RedirectionService.c_str(), RedirectionService.size()))
	{
		return AuthenticationResultFailure;
	}
	fillBuffer(packet, firstAttempt);

	// send the buffer
	if (m_socket->send(packet.data, packet.length) != packet.length)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::authFirst> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		

	// receive params from AMT 
	packet.Memset();
	
	// receive the base msg of MSG_AuthenticateSession
	packet.length = m_socket->receive(packet.data, sizeof(MSG_AuthenticateSession), RECEIVE_TIMEOUT);
	if (packet.length != sizeof(MSG_AuthenticateSession))
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::authFirst> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}
		

	// receive the data of the MSG_AuthenticateSession
	unsigned int dataLeft = ((MSG_AuthenticateSession*)packet.data)->dataLength;
	packet.length += m_socket->receive(packet.data + sizeof(MSG_AuthenticateSession), dataLeft, RECEIVE_TIMEOUT);
	// check data length
	if (packet.length > packet.size())
	{
		return AuthenticationResultFailure;
	}
	// make sure we receive all the data we expected
	if (packet.length != sizeof(MSG_AuthenticateSession)+dataLeft)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::authFirst> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}
		
	
	// the following function will parse the data in packet
	// and initialize the digestResponse structure
	status = parseDigestResponseRealmNonce(packet, digestResponse);
	if (AuthenticationResultSuccess != status)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<AuthenticatorDigest2::authFirst> Failed to pase the data in packet and initialize the digestResponse structure" << endl;
		return status;
	}
		

	// set the username and service to the digestResponse
	digestResponse.DigestAuth.UserNameLength = static_cast<UINT8>(m_user.size());
	if (memcpy_s(digestResponse.DigestAuth.UserName, ARRAYSIZE(digestResponse.DigestAuth.UserName), m_user.c_str(), m_user.size()))
	{
		return AuthenticationResultFailure;
	}
	digestResponse.DigestAuth.ServiceLength = static_cast<UINT8>(RedirectionService.size());
	if (memcpy_s(digestResponse.DigestAuth.Service, ARRAYSIZE(digestResponse.DigestAuth.Service), RedirectionService.c_str(), RedirectionService.size()))
	{
		return AuthenticationResultFailure;
	}
	fillBuffer(packet, digestResponse);

	return AuthenticationResultSuccess;
}

void AuthenticatorDigest::GenerateCNonce(char *outbuff) 
{
    srand((unsigned int)time(NULL));
    for(int i = 0; i < 32; i++) {
        int num = (int)(((double) rand()/ ((double)RAND_MAX+1)) * 16);
        switch(num) {
        case 0: case 1: case 2: case 3: case 4: case 5:
        case 6: case 7: case 8: case 9: 
            outbuff[i] = '0' + num;
            break;
        case 10: case 11: case 12: case 13: case 14: case 15:
            outbuff[i] = 'a' + (num-10);
            break;
        default:
            outbuff[i] = 'f';
        }
    }
    outbuff[32] = 0;
}