//  Copyright (C) 2007 Intel Corporation
#include "AuthenticatorKerberos.h"
#include "AuthenticationDefinitions.h"
#include <windows.h>
#include "Logger.h"
// disable the warning in the cencode (LIB64) related files
#pragma warning(disable:4244)

#define SEC_SUCCESS(Status) ((Status) >= 0)
#define MAX_SESSION_PACKET_LENGTH 12000

extern "C" {
	#include "cencode.h"
	#include "cdecode.h"
}


string AuthenticatorKerberos::error2String(SECURITY_STATUS status)
{
	switch (status)
	{
	case SEC_E_INSUFFICIENT_MEMORY:
		return "SEC_E_INSUFFICIENT_MEMORY - insufficient memory available";
	case SEC_E_INTERNAL_ERROR:
		return "SEC_E_INTERNAL_ERROR - internal SSPI error";
	case SEC_E_NO_CREDENTIALS:
		return "SEC_E_NO_CREDENTIALS - no credentials are available in the security package (Negotiate)";
	case SEC_E_NOT_OWNER:
		return "SEC_E_NOT_OWNER - caller doesn't have necessary credentials";
	case SEC_E_SECPKG_NOT_FOUND:
		return "SEC_E_SECPKG_NOT_FOUND - requested security package does not exist";
	case SEC_E_UNKNOWN_CREDENTIALS:
		return "SEC_E_UNKNOWN_CREDENTIALS - unknown credentials";
	case SEC_E_INVALID_HANDLE:
		return "SEC_E_INVALID_HANDLE - invalid handle passed";
	case SEC_E_INVALID_TOKEN:
		return "SEC_E_INVALID_TOKEN - invalid token passed";
	case SEC_E_LOGON_DENIED:
		return "SEC_E_LOGON_DENIED - the logon failed";
	case SEC_E_NO_AUTHENTICATING_AUTHORITY:
		return "SEC_E_NO_AUTHENTICATING_AUTHORITY - no authority could be contacted for authentication, domain name could be wrong";
	case SEC_E_TARGET_UNKNOWN:
		return "SEC_E_TARGET_UNKNOWN - the target was not recognized.";
	case SEC_E_UNSUPPORTED_FUNCTION:
		return "SEC_E_UNSUPPORTED_FUNCTION - specified flag is not valid";
	case SEC_E_WRONG_PRINCIPAL:
		return "SEC_E_WRONG_PRINCIPAL - different principal received";
	case SEC_E_OUT_OF_SEQUENCE:
		return "SEC_E_OUT_OF_SEQUENCE - message number is incorrect";
	case SEC_E_MESSAGE_ALTERED:
		return "SEC_E_MESSAGE_ALTERED - the message or signature supplied has been altered";
	default:
		return "unknown error";
	}
}

AuthenticatorKerberos::AuthenticatorKerberos(SPtr<Socket> socket, string ip, UINT16 port, 
											 string user, string pwd, string domain) : Authenticator(socket)
{
	m_ip = ip;
	m_port = port;
	m_user = user;
	m_pwd = pwd;
	m_domain = domain;
}

AuthenticatorKerberos::~AuthenticatorKerberos(void)
{
}

AuthenticationResult AuthenticatorKerberos::Authenticate()
{
	bool fatal;
	return _doAuthKerberos(&fatal);
}

int AuthenticatorKerberos::_base64Encode(unsigned char *out, const unsigned char *in, int inlen, int maxlen)
{    
	base64_encodestate state;
	unsigned char *out_orig = out;

	// check buffer size for Base64 conversion
	if (((inlen + 2) / 3 ) * 4 >= maxlen) {
		return -1;
	}

	base64_init_encodestate(&state);
	out += base64_encode_block((const char *)in, inlen, (char *)out, &state);
	out += base64_encode_blockend((char *)out, &state);

	return (int)(out - out_orig);
}

int AuthenticatorKerberos::_base64Decode(unsigned char *out, unsigned char *in, int inlen)
{
    int len = 0;
	base64_decodestate state;
	
	base64_init_decodestate(&state);
	len = base64_decode_block((char *)in, inlen, (char *)out, &state);

	return len;
}

bool AuthenticatorKerberos::_kerberosSupported()
{
	ULONG numPackages = 0;
	SecPkgInfo *secInfo = NULL;
	bool found = false;
	unsigned int i;

	if (EnumerateSecurityPackages(&numPackages, &secInfo) != SEC_E_OK) {
		return false;
	}

	for (i = 0; i < numPackages; i++) {
		if (strcmp(secInfo[i].Name, "Kerberos") == 0) {
			found = true;
			break;
		}
	}

	FreeContextBuffer(secInfo);

	return found;
}

bool AuthenticatorKerberos::_genClientContext(BYTE *pIn, DWORD cbIn, BYTE *pOut,
								DWORD *pcbOut, bool *done, char *spn,
								CredHandle *cred, CtxtHandle *context)
{
	SECURITY_STATUS status;
	SecBufferDesc   OutBuffDesc;
	SecBuffer       OutSecBuff;
	SecBufferDesc   InBuffDesc;
	SecBuffer       InSecBuff;
	ULONG           contextAttr;
	bool            createdContext = false;

	OutBuffDesc.ulVersion = SECBUFFER_VERSION;
	OutBuffDesc.cBuffers  = 1;
	OutBuffDesc.pBuffers  = &OutSecBuff;

	OutSecBuff.cbBuffer   = *pcbOut;
	OutSecBuff.BufferType = SECBUFFER_TOKEN;
	OutSecBuff.pvBuffer   = pOut;

	if (pIn) {
		InBuffDesc.ulVersion = SECBUFFER_VERSION;
		InBuffDesc.cBuffers  = 1;
		InBuffDesc.pBuffers  = &InSecBuff;

		InSecBuff.cbBuffer   = cbIn;
		InSecBuff.BufferType = SECBUFFER_TOKEN;
		InSecBuff.pvBuffer   = pIn;
	}

	status = InitializeSecurityContext(cred,
										(pIn) ? context : NULL,
										spn,
										ISC_REQ_MUTUAL_AUTH,
										0,
										SECURITY_NATIVE_DREP,
										(pIn) ? &InBuffDesc : NULL,
										0,
										context,
										&OutBuffDesc,
										&contextAttr,
										NULL);

	if (!SEC_SUCCESS(status)) {
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_genClientContext> Failed to initialize security context: " + error2String(status) << endl;
		goto out;
	}

	if (pIn == NULL) {
		createdContext = true;
	}

	if ((status == SEC_I_COMPLETE_NEEDED) ||
		(status == SEC_I_COMPLETE_AND_CONTINUE)) {
		// complete auth token
		status = CompleteAuthToken(context, &OutBuffDesc);
		if (!SEC_SUCCESS(status)) {
			Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_genClientContext> Failed to complete authentication token: " + error2String(status) << endl;
			goto out;
		}
	}

	*pcbOut = (DWORD)OutSecBuff.cbBuffer;

	*done = !((status == SEC_I_CONTINUE_NEEDED) ||
				(status == SEC_I_COMPLETE_AND_CONTINUE));

	return true;

out:
	if (createdContext) DeleteSecurityContext(context);

	return false;
}

AuthenticationResult AuthenticatorKerberos::_doAuthKerberos(bool *fatal)
{
	PacketBuffer packet(MAX_SESSION_PACKET_LENGTH);
	AuthenticationResult res = AuthenticationResultFailure;
	char spn[1024];
	DWORD spnLen = sizeof(spn);
	bool done = false;
	SECURITY_STATUS status;
	CtxtHandle context;
	CredHandle cred;
	DWORD cbIn;
	DWORD cbOut;
	bool hasCred = false;
	bool hasContext = false;
	MSG_AuthenticateSession *authMsg;
	MSG_AuthenticateSessionReply *repMsg;

	BYTE outBuffDec[MAX_SESSION_PACKET_LENGTH]; // non-base64
	//BYTE outBuff[MAX_SESSION_PACKET_LENGTH]; // base64

	authMsg = (MSG_AuthenticateSession *)packet.data;

	*fatal = false;

	if (!_kerberosSupported()) {
		Logger::GetInstance() << "Kerberos is not supported\n";
		goto out;
	}

	if (DsMakeSpn("http", m_ip.c_str(), NULL, m_port, NULL,
					&spnLen, spn) != ERROR_SUCCESS) {
		Logger::GetInstance() << "<AuthenticatorKerberos::_doAuthKerberos> Could not create service principal name (SPN) - invalid parameter or buffer size not enough" << endl;
		goto out;
	}

	SEC_WINNT_AUTH_IDENTITY_EX authIdentity;
	SecureZeroMemory(& authIdentity, sizeof authIdentity);

	if (! m_domain.empty() && ! m_user.empty()) {
		authIdentity.Version = SEC_WINNT_AUTH_IDENTITY_VERSION;
		authIdentity.Length = sizeof authIdentity;
		authIdentity.Domain = (unsigned char*)m_domain.c_str();
		authIdentity.DomainLength = (unsigned long)m_domain.size();
		authIdentity.User = (unsigned char*)m_user.c_str();
		authIdentity.UserLength = (unsigned long)m_user.size();
		authIdentity.Password = (unsigned char*)m_pwd.c_str();
		authIdentity.PasswordLength = (unsigned long)m_pwd.size();
		authIdentity.Flags = SEC_WINNT_AUTH_IDENTITY_ANSI;
	}

	status = AcquireCredentialsHandle(NULL, 
		"Negotiate", 
		SECPKG_CRED_OUTBOUND,
		NULL, 
		m_domain.empty() || m_user.empty() ? NULL : &authIdentity, 
		NULL, NULL, &cred, NULL);

	if (!(SEC_SUCCESS(status))) {
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Failed to acquire credential handle: " + error2String(status) << endl;
		goto out;
	}
	
	hasCred = true;

	cbOut = packet.size() - sizeof(MSG_AuthenticateSession);
	if (!_genClientContext(NULL, 0, outBuffDec, // authMsg->data,
							&cbOut,	&done, spn, &cred, &context)) {
		goto out;
	}
	hasContext = true;

	cbOut = _base64Encode(authMsg->data, outBuffDec, cbOut, packet.size() - sizeof(MSG_AuthenticateSession));
	if (cbOut == -1) {
		goto out;
	}

	// send output buffer to server
	//packet.data = outBuff;
	packet.length = 0;

	prepareAuthMsg(&packet, AUTH_TYPE_KERBEROS);

	authMsg->dataLength = cbOut;
	packet.length += authMsg->dataLength;

	if (m_socket->send(packet.data, packet.length) != packet.length)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Socket fail send" << endl;
		return AuthenticationResultFailure;
	}
		
	
	packet.Memset();

	packet.length = m_socket->receive(packet.data, sizeof(MSG_AuthenticateSessionReply), RECEIVE_TIMEOUT);
	repMsg = (MSG_AuthenticateSessionReply*)packet.data;
	unsigned int tmp = m_socket->receive(packet.data + packet.length, repMsg->dataLength, RECEIVE_TIMEOUT);
	if (packet.length + tmp > packet.size())
	{
		return AuthenticationResultFailure;
	}
	if (tmp != repMsg->dataLength)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Socket fail receive" << endl;
		return AuthenticationResultFailure;
	}

	packet.length += tmp;
	
	if(repMsg->hdr.status != AUTH_STATUS_SUCCESS)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Socket fail receive" << endl;
		res = AMTRes2AuthenticationResult(repMsg->hdr.status);
		goto out;
	}

	// We have been authenticated successfully. Now for Mutual Authentication.
	// From here failures are fatal - don't fall back to another auth algo.
	*fatal = true;

	if (repMsg->dataLength == 0) {
		// there is no new input data in the message to pass
		// to the SSPI Security library!
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Socket fail receive - no new input data in the message to pass" << endl;
		goto out;
	}

	repMsg->data[repMsg->dataLength] = '\0';

	cbIn = _base64Decode(outBuffDec, repMsg->data, repMsg->dataLength);
	if (cbIn == -1) {
		Logger::GetInstance(Logger::LOG_ERROR) << "<AuthenticatorKerberos::_doAuthKerberos> Failed to decode received message" << endl;
		goto out;
	}

	cbOut = packet.size();
	if (!_genClientContext(outBuffDec, cbIn, authMsg->data,
							&cbOut, &done, spn, &cred, &context)) {
		goto out;
	}

	if (!done) {
		// shouldn't happen. The sec lib is not done, but kerberos
		// authentication shouldn't take multiple tokens to complete.
		goto out;
	}

	res = AuthenticationResultSuccess;

out:
	if (hasContext) DeleteSecurityContext(&context);
	if (hasCred)    FreeCredentialsHandle(&cred);

	return res;
}
