//  Copyright (C) 2007 Intel Corporation
#ifndef _KVM_LIB_H_
#define _KVM_LIB_H_
#ifdef _WIN32
#ifndef _LIB
#define EXPORTED_FUNC __declspec(dllexport)
#define CALL_FUNC _stdcall
#endif
#else
#define EXPORTED_FUNC
#define CALL_FUNC
#endif
#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#define LENGTH_URL			256
#define LENGTH_USER_NAME	256
#define LENGTH_PWD			256
#define LENGTH_DOMAIN	    256

/*
	hold the connection information required to connect to
	Intel(r) AMT machine.
*/
#pragma pack(1)
typedef struct _ConnectionData
{
	char			host[LENGTH_URL];
	unsigned short	host_port;
	char			user[LENGTH_USER_NAME];
	char			pwd[LENGTH_PWD];
	char			proxy[LENGTH_URL];
	unsigned short	proxy_port;
	char			proxy_user[LENGTH_USER_NAME];
	char			proxy_pwd[LENGTH_PWD];
	unsigned int	clientID ; /*OUT parameter*/
	char            domain[LENGTH_DOMAIN];
}ConnectionData;
#pragma pack()

typedef enum _KVM_STATUS
{
	KVM_STATUS_OK = 0,
	KVM_STATUS_ERROR,
	KVM_STATUS_ERROR_INVALID_ARGUMENT,
	KVM_STATUS_ERROR_PROTOCOL_VIOLATION,
	KVM_STATUS_ERORR_LIBRARY_ALREADY_INITIALIZED,
	KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED,
	KVM_STATUS_ERORR_LIBRARY_NOT_STARTED,
	KVM_STATUS_ERORR_LIBRARY_STARTED,
	KVM_STATUS_ERORR_LISTENING_PORT_OCCUPIED,
	KVM_STATUS_ERORR_SIGNAL_SOCKET_FAILED_CONNECT,
}KVM_STATUS;

/*
	The library will call this function when RFB client 
	will try to connect to it. The library then will try to open
	a connection to an Intel(r) AMT machine and establish a KVM session.
	params:
	info - a struct that should be filled with the connection details
		   required to connect to an Intel(r) AMT machine.
*/
typedef void (CALL_FUNC *ConnectionInfoCallback)(ConnectionData* info);

/*
	Application should call on startup the Init function.
	params:
	root_certificate		- the appication should provide a root 
							  certificate in order to verify the 
							  Intel(r) AMT machine, the certificate will be
							  used only in TLS connection. Empty string should
							  be provided if no such certificate exists.
	personal_certificate	- the appication should provide a personal 
							  certificate in order to establisgh a TLS mutual 
							  connection with Intel(r) AMT machine. Empty string 
							  should be provided if no such certificate exists.
	certificate_pwd			- in case the personal certificate is 
							  password protected, this field should 
							  contain the password.
    getdata					- callback function that will be called
							  when an attempt to connect to the proxy was made.
							  The callback function should fill the connection
							  details required to connect to Intel(r) AMT machine.
	return:
			0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_Init(char* root_certificate, 
						   char* personal_certificate,
						   char* certificate_pwd,
						   ConnectionInfoCallback getdata);

/*
	Application should call on termination to this function.
	The function will free the resources.
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_Deinit();

/*
	When the proxy is not running it is possible to change the certificates. This function uses to set the library to work with OpenSSL in case of TLS connection. 
	params:
	root_certificate		- the appication should provide a root 
							  certificate in order to verify the 
							  Intel(r) AMT machine, the certificate will be
							  used only in TLS connection. Empty string should
							  be provided if no such certificate exists.
	personal_certificate	- the appication should provide a personal 
							  certificate in order to establisgh a TLS mutual 
							  connection with Intel(r) AMT machine. Empty string 
							  should be provided if no such certificate exists.
	certificate_pwd			- in case the personal certificate is 
							  password protected, this field should 
							  contain the password.
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_SetCertificates(char* root_certificate, 
									 char* personal_certificate,
									 char* certificate_pwd);

#ifdef _WIN32

/*
	When the proxy is not running it is possible to change the client certificate subject. This function uses to set the library to work with SSPI(Schannel) in case of TLS connection. 
	params:
	common_name				- the appication should provide a subject of the personal 
							  certificate in order to establisgh a TLS mutual 
							  connection with Intel(r) AMT machine. The first certificate with 
							  this subject string will be chosen from the store.
							  Empty string should be provided if the application wants the library 
							  to find itself any appropriate client certificate from the Windows Personal Store.
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_SetClientCertificate(char* common_name);

#endif


/*
	Calling this function will start the listening of the proxy on the specified 
	port. when calling this function with port > 0 the library will try to
	listen on the specified port. when calling this function with 0 the library
	will choose randomly a free port and listen on it the chosen port will be
	returned in the port parameter.
	params:
	IN/OUT port		- the listening port, in case 0 was provided the 
					  library will choose randomly a free port and this parameter
					  will hold the chosen port.
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_Start(unsigned short* port);

/*
	Calling this function will start the listening of the proxy on the specified
	port. when calling this function with port > 0 the library will try to
	listen on the specified port. when calling this function with 0 the library
	will choose randomly a free port and listen on it the chosen port will be
	returned in the port parameter.
	params:
	IN/OUT port		- the listening port, in case 0 was provided the
					  library will choose randomly a free port and this parameter
					  will hold the chosen port.
	amt_version		- the AMT version of the target host. Maximum length of 256 bytes (including NULL Terminated).
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_StartEx(unsigned short* port, const char* amt_version);

/*
	Stop the proxy (terminate the listening socket).
	return:
		0 <=> success
*/
EXPORTED_FUNC KVM_STATUS KVM_Stop();

/*
  Defines valid status values to be passed as parameters to 
  StatusCallback.
*/
typedef enum
{
    Disconnected = 1,
	DisconnectedSessionManagerFailed = 2,
	DisconnectedTLSFailed = 3,
	DisconnectedSocksError = 4,
	DisconnectedSocksAuthFailed = 5,
	DisconnectedAuditLogFullOrLocked = 6,
	DisconnectedInvalidParameter = 7,
	DisconnectedAuthFailed = 8,
	DisconnectedClientBusy = 9,
	DisconnectedUnsupported = 10,
} ConnectionStatus;


/*
	The calling application should implement the below prototype function
*/
typedef int (CALL_FUNC *StatusCallback)(ConnectionStatus status, unsigned int clientID);

/*
	Registering the status callback function, will enable the 
	calling application to receive notifications
	about status changes of a clientID.
*/
EXPORTED_FUNC void KVM_RegisterStatusCallback(StatusCallback statusCallback);
#ifdef __cplusplus
}
#endif // __cplusplus

#endif /*_KVM_H_*/
