//  Copyright (C) 2007 Intel Corporation
#ifndef _PROXY_MANAGER
#define _PROXY_MANAGER

#include <map>
#include <queue>
#include <set>
#include <string>

#ifndef _KVM_CONSOLE_DEBUG
#include "KVMLib.h"
#else
#include "KVMConsole.h"
#endif

#include "Event.h"
#include "Client.h"
#include "RWLock.h"
#include "Thread.h"
#include "PacketBuffer.h"
#include "SafeQueue.h"
#include <iostream>
#include "AuthenticationManager.h"
#include "SecString.h"


using namespace std;

#define DEFAULT_LISTEN_PORT	5900

#define LENGTH_URL			256
#define LENGTH_USER_NAME	256
#define LENGTH_PWD			256

static const UINT16 AMT_NON_SECURE_PORT = 16994;
static const UINT16 AMT_SECURE_PORT = 16995;

class ProxyManager
{
public:
	
	static ProxyManager& GetInstance();
	~ProxyManager(void);

	KVM_STATUS Init(ConnectionInfoCallback getdata);
	KVM_STATUS Deinit();
	
	KVM_STATUS SetCertificates(char* root_certificate, 
							  char* personal_certificate,
							  SecString<char> certificate_pwd);
	KVM_STATUS SetClientCertificate(const char* common_name);
	KVM_STATUS Start(UINT16* port, const char *amt_version);
	KVM_STATUS Stop();
	void RegisterStatusEvent(StatusCallback statusCallback);
	
private:
	ProxyManager();
	ProxyManager(ProxyManager& proxy);
	ProxyManager& operator=(const ProxyManager& proxy);
	KVM_STATUS SetAmtVersion(const char* amt_version);

	static KVM_STATUS AuthenticationResult2KVMStatus(AuthenticationResult res);

	Thread*				_OpenConnectionThread;
	Thread*				_RxTxThread;
	Thread*				_ListenerThread;
	static void			OpenConnectionThread(void* obj);
	static void			RxTxThread(void* obj);
	static void			ListenerThread(void* obj);
	
	// port will be used to listen for vnc clients
	unsigned short		_port;

	// Event that will be set only after the signal socket was connected
	Event _startEvent;

	// socket that will be used for signaling (used to wake the select method)
	SPtr<Socket>		_signalSocket;
	SPtr<Socket>		_listeningSocket;

	// hold all the connected clients
	map<ClientID, SPtr<Client> > _clients;
	RWLock						 _clientLock;
	
	// hold clients which is in the phase before authentication
	SafeQueue<SPtr<Client> > _OpeningConnectionQueue;

	// clients which are in the listening phase
	set<ClientID>			 _ListeningSet;
	RWLock				 	 _ListeningLock;

	// clients which have data to read
	SafeQueue<ClientID> _ReceiveQueue;

	// map between vnc client to amt client
	map<ClientID, ClientID> _ConnectionsMap;
	RWLock					_ConnectionsLock;

	// certificate information
	string _root_certificate;
	string _personal_certificate;
	SecString<char> _certificate_pwd;
	// SSPI client certificate infromation
	string _common_name;

	// AMT version of the target host
	string _amtVersion;
	int _majorVersion;

	// callback function to receive connection information
	ConnectionInfoCallback _getInfoCallback;
	// callback function to notify about client connection/disconnection
	StatusCallback	_statusCallback;

	// flag that will indicate on error during startup of the listener thread
	KVM_STATUS _startUpStatus;
	KVM_STATUS GetStartUpStatus();
	void SetStartUpStatus(KVM_STATUS status);
	RWLock			_startUpLock;

	// variables & function related to the state of the proxy
	bool _isRunning;
	RWLock _runningLock;
	bool IsRunning();
	void SetRunning(bool status);

	bool				_isInitialized;
	RWLock			_initializeLock;
	bool IsInitialized();
	void SetInitialization(bool status);

	// private helper functions
	int InitListeningThreadLoop(map<SOCKET, ClientID>& helperMap, fd_set& readFds, Socket* signalSocket);
	void MvListenClientToRxTx(ClientID id);
	KVM_STATUS _openConnection(ConnectionData* data, SPtr<Socket>& socket, SPtr<Client>& vnc_client);
	KVM_STATUS RemoveClient(ClientID id);
	void AddPair(SPtr<Client>& vnc_client, SPtr<Client>& amt_client);
	KVM_STATUS TryListen(UINT16* listening_port);
	void cleanDataStructures();
};

#endif /*_PROXY_MANAGER*/
