//  Copyright (C) 2007 Intel Corporation
#include "ProxyManager.h"
#include "Logger.h"
#include "AuthenticationManager.h"

#ifdef _WIN32
#define OFFSET_CN			3
#endif

#define AMT_VERSION_DIGITS				3
#define GCM_BLOCK_CIPHERS_AMT_VERSION	12		// GCM cipher suites with 256 block length are supported from AMT version 12
#define AMT_VERSION_MAX_LENGTH			256

ProxyManager& ProxyManager::GetInstance()
{
	static ProxyManager mgr;
	return mgr;
}

ProxyManager::ProxyManager() :
_root_certificate(""), _personal_certificate(""), _certificate_pwd(),_common_name(""),_statusCallback(NULL),
_OpenConnectionThread(NULL), _ListenerThread(NULL), _RxTxThread(NULL),
_OpeningConnectionQueue(), _ReceiveQueue(), _startEvent(false),
_ListeningSet(), _ListeningLock(), 
_ConnectionsMap(), _ConnectionsLock(),
_isRunning(false), _runningLock(),
_isInitialized(false), _initializeLock(), _startUpStatus(KVM_STATUS_OK), _startUpLock(), _amtVersion(""), _majorVersion(0), _port(0),
_getInfoCallback(NULL)
{
	#ifdef _WIN32
	WSAData wsaData;
	if (WSAStartup(MAKEWORD( 2 , 1), &wsaData) != 0) 
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::ProxyManager> can't initialize winsock library" << endl;
		return;
	}
	#endif
}

ProxyManager::~ProxyManager(void)
{
	if(IsRunning())
	{
		Stop();
	}
	_port = 0;
	_signalSocket = SPtr<Socket>(NULL);
	_listeningSocket = SPtr<Socket>(NULL);

	#ifdef _WIN32
	WSACleanup();
	#endif
}

KVM_STATUS ProxyManager::TryListen(UINT16* listening_port)
{
	_listeningSocket = SPtr<Socket>(new TCPServerSocket(*listening_port, "127.0.0.1"));
	if(0 == _listeningSocket->getPort())
	{
		return KVM_STATUS_ERORR_LISTENING_PORT_OCCUPIED;
	}
	_port = _listeningSocket->getPort();
	*listening_port = _port;
	Logger::GetInstance() << "create a listener on port: " << _port << endl;
	return KVM_STATUS_OK;
}

KVM_STATUS ProxyManager::Start(UINT16* listening_port, const char *amt_version)
{
	if(IsRunning())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Start> library is already started" << endl;
		return KVM_STATUS_ERORR_LIBRARY_STARTED;
	}

	if(!IsInitialized())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Start> library is not initialized" << endl;
		return KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED;
	}

	KVM_STATUS status;
	if((status = TryListen(listening_port)) != KVM_STATUS_OK)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::TryListen> can't listen on the specified port probably occupied" << endl;
		return status;
	}

	status = SetAmtVersion(amt_version);
	if (status != KVM_STATUS_OK)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetAmtVersion> Invalid AMT version" << endl;
		return status;
	}

	// create the running threads
	_OpenConnectionThread = new Thread(OpenConnectionThread, this);
	_RxTxThread = new Thread(RxTxThread, this);
	_ListenerThread = new Thread(ListenerThread, this);

	// change running flag to true
	SetRunning(true);

	// start the threads
	_OpenConnectionThread->start();
	_RxTxThread->start();
	_ListenerThread->start();

	// wait fot the signal socket to connect
	Logger::GetInstance() << "wait for signal socket to connect " <<  endl;
	_startEvent.wait();
	Logger::GetInstance() << "signal socket connected" <<  endl;

	// check if error occured during the start-up of the ListenerThread
	if((status = GetStartUpStatus()) != KVM_STATUS_OK)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::Start> error occured during the start-up of ListenerThread, try to stop all other tasks" <<  endl;
		Stop();
	}

	return status;
}

KVM_STATUS ProxyManager::Stop(void)
{
	if(!IsInitialized())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Stop> can't stop since library is not initialized" << endl;
		return KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED;
	}

	if(!IsRunning())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Stop> can't stop since library is not started" << endl;
		return KVM_STATUS_ERORR_LIBRARY_NOT_STARTED;
	}

	
	// change running flag to false
	SetRunning(false);

	// since pop may be blocked push dummy data to release
	_OpeningConnectionQueue.push(SPtr<Client>(NULL));
	ClientID id = INVALID_CLIENT_ID;
	_ReceiveQueue.push(id);

	Logger::GetInstance() << "singal threads to stop running" << endl;
	// signal the listener thread to finish wake the select
	int c = 0;
	_signalSocket->send((UINT8*)&c, 1);

	if(_OpenConnectionThread)
	{
		Logger::GetInstance() << "wait for OpenConnectionThread to stop running" << endl;
		_OpenConnectionThread->wait();
		Logger::GetInstance() << "OpenConnectionThread stopped running" << endl;
		delete _OpenConnectionThread;
		_OpenConnectionThread = NULL;
	}

	if(_RxTxThread)
	{
		Logger::GetInstance() << "wait for RxTxThread to stop running" << endl;
		_RxTxThread->wait();
		Logger::GetInstance() << "RxTxThread stopped running" << endl;
		delete _RxTxThread;
		_RxTxThread = NULL;
	}

	if(_ListenerThread)
	{
		Logger::GetInstance() << "wait for ListenerThread to stop running" << endl;
		_ListenerThread->wait();
		Logger::GetInstance() << "ListenerThread stopped running" << endl;
		delete _ListenerThread;
		_ListenerThread = NULL;
	}

	cleanDataStructures();
	
	_signalSocket = SPtr<Socket>(NULL);
	_listeningSocket = SPtr<Socket>(NULL);
	_port = 0;
	_startUpStatus = KVM_STATUS_OK;

	return KVM_STATUS_OK;
}

void ProxyManager::RegisterStatusEvent(StatusCallback statusCallback)
{
	_statusCallback = statusCallback;
}

// clean all the data structures
void ProxyManager::cleanDataStructures()
{
	// clean the clients set
	{
		Lock l(_clientLock, RWLock::READ_WRITE);
		_clients.clear();
	}
	// clean the opening connection queue
	_OpeningConnectionQueue.clear();
	
	// clean the listening set
	{
		Lock l(_ListeningLock, RWLock::READ_WRITE);
		_ListeningSet.clear();
	}
	// clean the receive queue
	_ReceiveQueue.clear();

	// clean the connection map
	{
		Lock l(_ConnectionsLock, RWLock::READ_WRITE);
		_ConnectionsMap.clear();
	}
}

KVM_STATUS ProxyManager::Init(ConnectionInfoCallback getdata)
{
	if(IsInitialized())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Init> ProxyManager already initialized" << endl;
		return KVM_STATUS_ERORR_LIBRARY_ALREADY_INITIALIZED;
	}

	_getInfoCallback = getdata;
	SetInitialization(true);
	return KVM_STATUS_OK;
}

KVM_STATUS ProxyManager::Deinit()
{
	KVM_STATUS status = KVM_STATUS_ERROR;
	
	if(IsInitialized())
	{
		if(IsRunning())
		{
			Logger::GetInstance() << "before deinit the library stop the running threads" << endl;
			this->Stop();
		}
		SetInitialization(false);
		status = KVM_STATUS_OK;
	}
	else
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::Deinit> can't deinit library because it was not initalized" << endl;
		status = KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED;
	}
	return status;
}

KVM_STATUS ProxyManager::SetCertificates(char* root_certificate, 
								 char* personal_certificate,
								 SecString<char> certificate_pwd)
{
	if (!IsInitialized())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetCertificates> can't set certificates since library is not initialized" << endl;
		return KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED; // must initialize library before setting certificate
	}
	if (IsRunning())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetCertificates> can't set certificates since library is already running" << endl;
		return KVM_STATUS_ERORR_LIBRARY_STARTED; // must stop library before setting certificates
	}

#ifdef _WIN32
	KVM_STATUS result = KVM_STATUS_OK;

	if (personal_certificate == NULL)
		result = SetClientCertificate("");
	else if (*personal_certificate == '\0')
		result = SetClientCertificate(personal_certificate);

	if(personal_certificate != NULL && *personal_certificate != '\0')
	{
		HANDLE fileHandle = NULL;							//Certificate file handle
		vector<BYTE> certBuffer;							//Pointer to the certificate's location in the memory
		DWORD numOfBytesRead = 0;							//Actual number of bytes read
		WIN32_FILE_ATTRIBUTE_DATA attributes;				//Certificate file attributes 
		PCCERT_CONTEXT certContext = NULL;					//Certificate context
		vector<CHAR> certSubjectBuffer;						//Buffer containing Subject field in the certificate
		DWORD bSize;										//Size of buffer containing Subject field in the certificate
		string certCommonName;

		HCERTSTORE hCertStore = NULL;			//virtual store handle
		vector<WCHAR> passUnicode;				//password in unicode format
		size_t passwordLength = 0;				//password's length in characters
		CRYPT_DATA_BLOB blob;

		//Open certificate file handle
		fileHandle = CreateFile(personal_certificate, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
		if (fileHandle == INVALID_HANDLE_VALUE)	{		//certificate file might not exist
			return KVM_STATUS_ERROR_INVALID_ARGUMENT;
		}

		//Get certificate file size
		if (GetFileAttributesEx(personal_certificate, GetFileExInfoStandard, &attributes) == FALSE){
			result = KVM_STATUS_ERROR;
			goto close;
		}


		DWORD fSize = attributes.nFileSizeLow;
		certBuffer.resize(fSize);

		//Load certificate to memory
		if (ReadFile(fileHandle, &certBuffer[0], fSize, &numOfBytesRead, NULL) == FALSE){
			result = KVM_STATUS_ERROR;
			goto close;
		}


		//set blob
		memset(&blob, 0, sizeof (blob));
		blob.pbData = &certBuffer[0];
		blob.cbData = fSize;

		//check if the certificate's file type is .pfx/.p12 extension 
		if (PFXIsPFXBlob(&blob) == TRUE)
		{
			string cert_pass(certificate_pwd.ToBase64String());

			//First, get password length
			passwordLength = mbstowcs(NULL, cert_pass.c_str(), 0);
			passUnicode.resize(passwordLength + 1);
			passwordLength = mbstowcs(&passUnicode[0], cert_pass.c_str(), passwordLength + 1);	//+1 to include null

			//create virtual store with the wanted certificate and get the certificate (will be in DER format)
			hCertStore = PFXImportCertStore(&blob, &passUnicode[0], CRYPT_EXPORTABLE);
			if (hCertStore == NULL){
				result = KVM_STATUS_ERROR_INVALID_ARGUMENT;			//might be incorrect/missing password
				goto close;
			}

			certContext = CertFindCertificateInStore(hCertStore, X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, 0, CERT_FIND_ANY, NULL, NULL);

		}
		else
		{
			BOOL convertRes;									//FALSE - DER encoded certificate,  TRUE - Base64 encoded certificate
			vector<BYTE> certBufferBase64;						//Pointer to the certificate's location in the memory - if the certificate is not der encoded (Base64 encoded)
			DWORD newBuffSize = fSize;

			//We don't know how the certificate is encoded so we try to change to DER encoded
			//If success - the certificate was Base64 encoded , failure - the certificate was already DER encoded
			convertRes = CryptStringToBinaryA((LPCSTR)&certBuffer[0], 0, CRYPT_STRING_BASE64HEADER, NULL, &newBuffSize, NULL, NULL);

			if (convertRes == TRUE){
				//Original Base64 format
				certBufferBase64.resize(fSize);
				convertRes = CryptStringToBinaryA((LPCSTR)&certBuffer[0], 0, CRYPT_STRING_BASE64HEADER, &certBufferBase64[0], &newBuffSize, NULL, NULL);
				if (convertRes == FALSE){
					result = KVM_STATUS_ERROR_INVALID_ARGUMENT;
					goto close;
				}

				certContext = CertCreateCertificateContext(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, &certBufferBase64[0], newBuffSize);
			}
			else
			{
				// already DER foramt
				certContext = CertCreateCertificateContext(X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, &certBuffer[0], fSize);
			}
		}

		if (certContext == NULL){
			result = KVM_STATUS_ERROR_INVALID_ARGUMENT;
			goto close;
		}

		//Find CN
		//Function returns how many bytes needed to read (including - NULL terminate)
		bSize = CertNameToStr(X509_ASN_ENCODING, &(certContext->pCertInfo->Subject), CERT_X500_NAME_STR, NULL, 0);
		if (bSize == 1){
			result = KVM_STATUS_ERROR;
			goto close;
		}

		//Allocate appropriate size to read and call function again
		certSubjectBuffer.resize(bSize);
		bSize = CertNameToStr(X509_ASN_ENCODING, &(certContext->pCertInfo->Subject), CERT_X500_NAME_STR, &certSubjectBuffer[0], bSize);
		if (bSize == 1){
			result = KVM_STATUS_ERROR;
			goto close;
		}


		//parsing buff to get the CN
		certCommonName = &certSubjectBuffer[0];
		size_t pos = certCommonName.find("CN");
		if (pos == string::npos){
			result = KVM_STATUS_ERROR_INVALID_ARGUMENT;
			goto close;
		}

		certCommonName = certCommonName.substr(pos + OFFSET_CN);
		pos = certCommonName.find(",");
		if (pos != string::npos)
			certCommonName = certCommonName.substr(0, pos);


		result = SetClientCertificate(certCommonName.c_str());

		if (result == KVM_STATUS_OK)
			Logger::GetInstance() << "<ProxyManager::SetCertificates> Successfully extracted personal certificate common name: " << certCommonName << endl;

	close:
		if (certContext != NULL)
			if (CertFreeCertificateContext(certContext) == FALSE)
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetCertificates> Failed closing certificate context: " << GetLastError() << endl;

		if (hCertStore != NULL)
			if (CertCloseStore(hCertStore, CERT_CLOSE_STORE_FORCE_FLAG) == FALSE)
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetCertificates> Failed closing certificates store: " << GetLastError() << endl;

		if (fileHandle != NULL)
			if (CloseHandle(fileHandle) == FALSE)
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetCertificates> Failed closing file handle: " << GetLastError() << endl;
	}

	return result;
#else
	_root_certificate = string(root_certificate);
	_personal_certificate = string(personal_certificate);
	_certificate_pwd = certificate_pwd;
	return KVM_STATUS_OK;
#endif

}

KVM_STATUS ProxyManager::SetClientCertificate(const char* common_name)
{
	#ifdef _WIN32
	if(!IsInitialized())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetClientCertificate> can't set client certificate since library is not initialized" << endl;
		return KVM_STATUS_ERORR_LIBRARY_NOT_INITIALIZED; // must initialize library before setting certificate
	}
	if(IsRunning())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::SetClientCertificate> can't set client certificate since library is already running" << endl;
		return KVM_STATUS_ERORR_LIBRARY_STARTED; // must stop library before setting certificates
	}

	_common_name = string(common_name);
	WinSSLSocket::setIsMutual(true);
	#endif
	return KVM_STATUS_OK;
}

KVM_STATUS ProxyManager::SetAmtVersion(const char* amt_version)
{
	if (amt_version == NULL || *amt_version == '\0')
	{
		_amtVersion = "";
		_majorVersion = 0;
		return KVM_STATUS_OK;
	}

	// Parse & validate version
	try {
		string valVersion(amt_version);
		
		if (AMT_VERSION_MAX_LENGTH < valVersion.length())
		{
			return KVM_STATUS_ERROR_INVALID_ARGUMENT;
		}

		// AMT version string is composed in the following format:
		// "XX.YY.ZZZZ"
		// XX - represent major version number
		// YY - represent minor version number
		// ZZ - represent revision version number
		// total 3 numbers.
		vector<int> versionNumbers;
		size_t dop = 0;	// Digit end pos
		for (int i = 0; i < AMT_VERSION_DIGITS - 1; ++i)
		{
			size_t pos = valVersion.find(".");
			if (pos == string::npos)
			{
				return KVM_STATUS_ERROR_INVALID_ARGUMENT;
			}
			versionNumbers.push_back(std::stoi(valVersion, &dop));
			if (pos != dop)
			{
				return KVM_STATUS_ERROR_INVALID_ARGUMENT;
			}
			valVersion = valVersion.substr(pos + 1);
			dop = 0;
		}
		versionNumbers.push_back(std::stoi(valVersion, &dop));	// Get revision version
		if (dop != valVersion.length())
		{
			return KVM_STATUS_ERROR_INVALID_ARGUMENT;
		}

		// If we arrived here then the version string is OK (we were able to parse it)
		// Save the version info
		_amtVersion = amt_version;
		_majorVersion = versionNumbers[0];
		Logger::GetInstance() << "AMT Version: " << _amtVersion << endl;
		Logger::GetInstance() << "AMT Major Version: " << _majorVersion << endl;
	}
	catch (...)
	{
		return KVM_STATUS_ERROR_INVALID_ARGUMENT;
	}
	
	return KVM_STATUS_OK;
}

void ProxyManager::RxTxThread(void* obj)
{
	Logger::GetInstance() << "RxTxThread started" << endl;
	ProxyManager* proxy = (ProxyManager*)obj;
	// allocate only 1 time the buffer and perform memset every iteration
	PacketBuffer packet;
	ClientID id;
	ClientID id_second;

	while(proxy->IsRunning())
	{
		packet.Memset();
		// pop the client from the queue (blocking call)
		id = proxy->_ReceiveQueue.pop();

		if(!proxy->IsRunning())
		{
			Logger::GetInstance() << "proxy stopped running going to break from loop RxTxThread " << endl;
			break;
		}

		// get the second client using the connection map
		{
			map<ClientID, ClientID>::iterator it;
			Lock l(proxy->_ConnectionsLock, RWLock::READ_ONLY);
			if((it = proxy->_ConnectionsMap.find(id)) == proxy->_ConnectionsMap.end())
			{
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't find pair to client: " <<  id << " in connection map ignore" << endl;
				continue ; // client not found
			}
			id_second = it->second;
		}

		// hold the socket for the first client to receive the data
		SPtr<Socket> id_sock = SPtr<Socket>(NULL);
		// hold the socket for the second client to send data to
		SPtr<Socket> id_second_sock = SPtr<Socket>(NULL);
		{
			map<ClientID, SPtr<Client> >::iterator it;
			Lock l(proxy->_clientLock, RWLock::READ_ONLY);
			// find the first client
			if((it = proxy->_clients.find(id)) == proxy->_clients.end())
			{
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't find client: " << id << " in clients map";
				continue ; // client not found
			}
			id_sock = it->second->GetSocket();
		
			// find the second client
			if((it = proxy->_clients.find(id_second)) == proxy->_clients.end())
			{
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't find second client: " << id_second << " in clients map" << endl;
				continue ; // client not found
			}
			id_second_sock = it->second->GetSocket();
		}
		
		// receive the data from the first client
		int res = id_sock->receive(packet.data, packet.size());
		if(res < 0)
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't receive data from client: " << id << " probably the connection was terminated" << endl;
			proxy->RemoveClient(id);
			continue;
		}
		packet.length = res;
		
		// send the data to the second client
		res = id_second_sock->send(packet.data, packet.length);
		if(res < 0)
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't send data to second client: " << id_second << " probably the connection was terminated" << endl;
			proxy->RemoveClient(id_second);
			continue;
		}
		
		while((res = id_sock->getInternalBUffer(packet.data, packet.size())) != 0)
		{
			packet.length = res;
			res = id_second_sock->send(packet.data, packet.length);
			if(res < 0)
			{
				Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::RxTxThread> can't send data to second client: " << id_second << " probably the connection was terminated" << endl;
				proxy->RemoveClient(id_second);
				break;
			}
		}

		// add the first client back to the listening set
		// the second client already exists in the listening set
		// need to signal the select from the listening set
		{
			Lock l(proxy->_ListeningLock, RWLock::READ_WRITE);
			proxy->_ListeningSet.insert(id);
			int c = 0;
			proxy->_signalSocket->send((UINT8*)&c, 1);
		}
		
	}
}

KVM_STATUS ProxyManager::RemoveClient(ClientID id)
{
	ClientID id_pair = Client::UNKNOWN_CLIENT;
	// try to remove the clients from the connection map
	{
		map<ClientID, ClientID>::iterator it;
		Lock l(_ConnectionsLock, RWLock::READ_WRITE);
		if((it = _ConnectionsMap.find(id)) != _ConnectionsMap.end())
		{
			id_pair = it->second;
			_ConnectionsMap.erase(it);
		}
		if((it = _ConnectionsMap.find(id_pair)) != _ConnectionsMap.end())
			_ConnectionsMap.erase(it);

	}
	// try to remove the clients from the listening set
	{
		set<ClientID>::iterator it;
		Lock l(_ListeningLock, RWLock::READ_WRITE);
		if((it = _ListeningSet.find(id)) != _ListeningSet.end())
			_ListeningSet.erase(it);
		if((it = _ListeningSet.find(id_pair)) != _ListeningSet.end())
			_ListeningSet.erase(it);
	}
	// try to remove the clients from the clients map
	ClientID vnc_client = INVALID_CLIENT_ID;
	{
		map<ClientID, SPtr<Client> >::iterator it, it_sec;
		Lock l(_clientLock, RWLock::READ_WRITE);
		if((it = _clients.find(id)) != _clients.end())
		{
			if(it->second->GetClientType() == Client::VNC_CLIENT)
			{
				vnc_client = it->first;
			}
			_clients.erase(it);
		}
		if((it_sec = _clients.find(id_pair)) != _clients.end())
		{
			if(it_sec->second->GetClientType() == Client::VNC_CLIENT)
			{
				vnc_client = it_sec->first;
			}
			_clients.erase(it_sec);
		}
	}
	// notify the calling application that vnc_client was removed
	if(_statusCallback && vnc_client != INVALID_CLIENT_ID)
	{
		Logger::GetInstance() << "Notify the calling application about client removal" << endl;
		//_statusCallback(Disconnected, vnc_client);
	}
	
	Logger::GetInstance() << "Remove client: " << id << endl;

	int c = 0;
	_signalSocket->send((UINT8*)&c,1);
	return KVM_STATUS_OK;
}


void ProxyManager::ListenerThread(void* obj)
{
	Logger::GetInstance() << "listenerThread started" << endl;
	ProxyManager* proxy = (ProxyManager*)obj;
	fd_set readFds;
	int fdCount = 0;	 
	map<SOCKET, ClientID> helperMap;
	PacketBuffer buffer;

	// get the listener socket
	SOCKET listenerFd = proxy->_listeningSocket->getSocket();

	// connect the signal socket to the listener
	proxy->_signalSocket = SPtr<Socket>(new TCPSocket(0, "", IPv4));
	SOCKET_LIB_RESULT res = proxy->_signalSocket->connect("localhost", proxy->_port);
	// if "hostname" is not found, try to connect by 127.0.0.1
	if(SOCKET_LIB_RES_SOCKET_ERROR == res)
		res = proxy->_signalSocket->connect("127.0.0.1", proxy->_port);
	if(SOCKET_LIB_RES_OK != res)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::ListenerThread> signal socket can't connect to listener socket" << endl;
		proxy->SetStartUpStatus(KVM_STATUS_ERORR_SIGNAL_SOCKET_FAILED_CONNECT);
		proxy->_startEvent.set();
		return;
	}
	Socket* signalSocket = proxy->_listeningSocket->accept();
	if(NULL == signalSocket)
	{
		Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::ListenerThread> listener socket can't accept signal socket" << endl;
		proxy->SetStartUpStatus(KVM_STATUS_ERORR_SIGNAL_SOCKET_FAILED_CONNECT);
		proxy->_startEvent.set();
		return;
	}

	signalSocket->setNonBlocking();

	SOCKET signalFd = signalSocket->getSocket();

	proxy->_startEvent.set();

	while(proxy->IsRunning())
	{
		// initialize the FD_SET using the clients map
		// initialize a map <socket,ClientID> for easy access
		{
			helperMap.clear();
			FD_ZERO(&readFds);
			fdCount = 0;	 
			set<ClientID>::iterator it;
			map<ClientID, SPtr<Client> >::iterator itClient;
			SOCKET sock = 0;
			Lock l(proxy->_ListeningLock, RWLock::READ_ONLY);
			for(it = proxy->_ListeningSet.begin(); it != proxy->_ListeningSet.end(); it++)
			{
				{
					Lock l1(proxy->_clientLock, RWLock::READ_ONLY);
					if((itClient = proxy->_clients.find(*it)) == proxy->_clients.end())
					{
						Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::ListenerThread> client id: " << *it << " found in the listening set but can't be find in the clients map" << endl;
						continue;
					}
					sock = itClient->second->GetSocket()->getSocket();
				}
				
				FD_SET(sock, &readFds);
 				if( static_cast<int>(sock) > fdCount )
				{
					fdCount = static_cast<int>(sock);
				}
				helperMap[sock] = *it;
			}
		}

		//add signal socket
		FD_SET(signalFd, &readFds);
		if( static_cast<int>(signalFd) > fdCount) {
			fdCount = static_cast<int>(signalFd);
		}
		
		// add listener socket
		FD_SET(listenerFd, &readFds);
		if( static_cast<int>(listenerFd) > fdCount) {
			fdCount = static_cast<int>(listenerFd);
		}

		//do select 
		fdCount++;
		int res = select(fdCount, &readFds, NULL, NULL, NULL );
		
		if (res <= 0) 
		{ 
			Logger::GetInstance(Logger::LOG_ERROR) << "<ProxyManager::ListenerThread> select return illegal value: " << res << endl;
			continue; 
		} 

		// if signal socket wake the select --> clear all the data
		if(FD_ISSET(signalFd, &readFds)) 
		{ 
		  if(!proxy->IsRunning())
			  break;
			 //clear all the data from signal socket
			do{
				buffer.Memset();
				buffer.length = signalSocket->receive(buffer.data, buffer.size());
			}while( buffer.length == buffer.size() );
			FD_CLR(signalFd, &readFds);
			res--;
		} 

		// if new connection add the client to the open_connections_queue
		if (FD_ISSET(listenerFd, &readFds)) 
		{ 
		  SPtr<Socket> newClient = SPtr<Socket>(proxy->_listeningSocket->accept()); 
		  SPtr<Client> client = SPtr<Client>(new Client(newClient, Client::VNC_CLIENT));
		  Logger::GetInstance() << "new client:" << client->GetClientID() << " connected" << endl;
		  proxy->_OpeningConnectionQueue.push(client);
			
		  FD_CLR(listenerFd, &readFds);
		  res--;
		} 

		// if there is data to receive push the client to the receive queue
		map<SOCKET, ClientID>::iterator itMap;
		for(itMap = helperMap.begin(); itMap != helperMap.end() && res != 0; itMap++)
		{
			if (FD_ISSET(itMap->first, &readFds)) 
			{
				proxy->MvListenClientToRxTx(itMap->second);
				FD_CLR(itMap->first, &readFds);
				res--;
			}
		}
	}
}

void ProxyManager::MvListenClientToRxTx(ClientID id)
{
	// remove the client handle from the listening set
	{
		Lock l(_ListeningLock, RWLock::READ_WRITE);
		_ListeningSet.erase(id);
	}
	_ReceiveQueue.push(id);
}

// open connection to the AMT machine using the data received
// the function will open the apropriate connection to AMT
// according to the ConnectionData struct
// return KVM_STATUS_SUCCESS if the connection was successfully opened
KVM_STATUS ProxyManager::_openConnection(ConnectionData* data, SPtr<Socket>& socket, SPtr<Client>& vnc_client)
{
	// check that the connectionData we have received are in the valid length
	if( (strlen(data->host) >= LENGTH_URL) || 
		(strlen(data->user) >= LENGTH_USER_NAME) ||
		(strlen(data->pwd) >= LENGTH_PWD) ||
		(strlen(data->proxy) >= LENGTH_URL) ||
		(strlen(data->proxy_user) >= LENGTH_USER_NAME) ||
		(strlen(data->proxy_pwd) >= LENGTH_PWD) ||
		(strlen(data->domain) >= LENGTH_DOMAIN))
	{
		if(_statusCallback)
			_statusCallback(DisconnectedInvalidParameter, vnc_client->GetClientID());
		return KVM_STATUS_ERROR_INVALID_ARGUMENT;
	}

	AuthenticationInfo info;
	info.Ip = data->host;
	info.UserName = data->user;
	info.Password = data->pwd;
	info.Port = data->host_port;
	
	// check if doamin is specified
	info.Domain =  (data->domain != NULL) ? data->domain: "";

	IP_VERSION ipVersionEndpoint = IPSocket::SupportedIPStack(info.Ip.c_str());
	IP_VERSION ipVersionSocksProxy = IPSocket::SupportedIPStack(data->proxy);
 
	// connection cancelled
	if(info.Ip.empty())
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "<ProxyManager::_openConnection> can't open connection since remote ip address is empty" << endl;
		return KVM_STATUS_ERROR;
	}

	// Check if target AMT can support GCM 256 block ciphers (suppoerted from AMT verion 12.0)
	bool enhancedTls = false;
	if (_majorVersion >= GCM_BLOCK_CIPHERS_AMT_VERSION)
	{
		enhancedTls = true;
	}

	if (strlen(data->proxy) > 0)
	{
		ProxySettings proxy_settings;
		memset(&proxy_settings, 0, sizeof(ProxySettings));
		if (strncpy_s(proxy_settings.server, ARRAYSIZE(proxy_settings.server), data->proxy, strlen(data->proxy)))
		{
			return KVM_STATUS_ERROR;
		}
		proxy_settings.port = data->proxy_port;
		proxy_settings.type = PROXY_SOCKS_V5;
		if (strncpy_s(proxy_settings.user, ARRAYSIZE(proxy_settings.user), data->proxy_user, strlen(data->proxy_user)))
		{
			return KVM_STATUS_ERROR;
		}
		if (strncpy_s(proxy_settings.password, ARRAYSIZE(proxy_settings.user), data->proxy_pwd, strlen(data->proxy_pwd)))
		{
			return KVM_STATUS_ERROR;
		}
		if(AMT_NON_SECURE_PORT == data->host_port)
		{
			Logger::GetInstance() << "created SOCKS socket to connect to Intel(r) AMT" << endl;
			socket = SPtr<Socket>(new SOCKSSocket(proxy_settings, ipVersionSocksProxy));
		}
		else
		{
			Logger::GetInstance() << "created SSL SOCKS socket to connect to Intel(r) AMT" << endl;
			SanitizedStr<char> cert_pwd;
			if (!_certificate_pwd.decrypt(cert_pwd))
			{
				Logger::GetInstance() << "Error: Descrypt password failed." << endl;
				return KVM_STATUS_ERROR;
			}
			socket = SPtr<Socket>(new SOCKSSocket(proxy_settings, _root_certificate, "", _personal_certificate, cert_pwd.str(), _common_name, ipVersionSocksProxy, enhancedTls));
		}
	}
	else
	{
		if(AMT_NON_SECURE_PORT == data->host_port)
		{
			Logger::GetInstance() << "created TCP socket to connect to Intel(r) AMT" << endl;
			socket = SPtr<Socket>(new TCPSocket(0, "", ipVersionEndpoint));
		}
		else
		{
			Logger::GetInstance() << "created SSL socket to connect to Intel(r) AMT" << endl;
			SanitizedStr<char> cert_pwd;
			if (!_certificate_pwd.decrypt(cert_pwd))
			{
				Logger::GetInstance() << "Error: Descrypt password failed." << endl;
				return KVM_STATUS_ERROR;
			}
			socket = SSLSocket::createSSLSocket(_root_certificate, "", _personal_certificate, cert_pwd.str(), _common_name, ipVersionEndpoint, "", enhancedTls);
			Logger::GetInstance() << "<ProxyManager::_openConnection> created WINSSL socket." << endl;
		}
	}
	
	SOCKET_LIB_RESULT res = socket->connect(info.Ip, info.Port);
	if(SOCKET_LIB_RES_SOCKS_AUTH_FAIL == res)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "can't open connection to Intel(r) AMT - reason SOCKS Auth fail" << endl;
		if(_statusCallback)
			_statusCallback(DisconnectedSocksAuthFailed, vnc_client->GetClientID());
		return KVM_STATUS_ERROR;
	}
	else if(SOCKET_LIB_RES_SOCKS_ERROR == res)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "can't open connection to Intel(r) AMT - reason SOCKS error" << endl;
		if(_statusCallback)
			_statusCallback(DisconnectedSocksError, vnc_client->GetClientID());
		return KVM_STATUS_ERROR;
	}
	else if(SOCKET_LIB_RES_TLS_CONNECTION_FAILED == res)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "can't open connection to Intel(r) AMT - reason TLS error" << endl;
		if(_statusCallback)
			_statusCallback(DisconnectedTLSFailed, vnc_client->GetClientID());
		return KVM_STATUS_ERROR;
	}
	else if(SOCKET_LIB_RES_OK != res)
	{
		Logger::GetInstance(Logger::LOG_WARNING) << "can't open connection to Intel(r) AMT - reason general error" << endl;
		if(_statusCallback)
			_statusCallback(Disconnected, vnc_client->GetClientID());
		return KVM_STATUS_ERROR;
	}
	
	Logger::GetInstance() << "opened connection to Intel(r) AMT machine" << endl;
	AuthenticationManager mgr(socket, info);
	AuthenticationResult kvmStatusRes = mgr.OpenSession();
	if(AuthenticationResultSuccess != kvmStatusRes)
	{
		ConnectionStatus status;
		switch (kvmStatusRes)
		{
		case AuthenticationResultFailure: status = DisconnectedAuthFailed;
			break;
		case AuthenticationResultAuditLogFailure: status = DisconnectedAuditLogFullOrLocked;
			break;
		case AuthenticationResultClientBusy: status = DisconnectedClientBusy;
			break;
		case AuthenticationResultUnsupported: status = DisconnectedUnsupported;
			break;
		default: status = DisconnectedSessionManagerFailed;
		}
		if(_statusCallback)		
			_statusCallback(status, vnc_client->GetClientID());
	}

	return AuthenticationResult2KVMStatus(kvmStatusRes);
}

void ProxyManager::OpenConnectionThread(void* obj)
{
	Logger::GetInstance() << "OpenConnectionThread started" << endl;
	ProxyManager* proxy = (ProxyManager*)obj;
	while(proxy->IsRunning())
	{
		SPtr<Client> vnc_client = SPtr<Client>(NULL);
		// pop the vnc client from the queue
		vnc_client = proxy->_OpeningConnectionQueue.pop();
		if(!proxy->IsRunning())
		{
			Logger::GetInstance() << "proxy stopped running going to break from loop OpenConnectionThread " << endl;
			break;
		}

		// invoke the callback function to get the connection details
		ConnectionData data;
		memset(&data, 0, sizeof(ConnectionData));
		data.clientID = vnc_client->GetClientID();
		Logger::GetInstance() << "invoke the callback function to receive connection details" << endl;
		proxy->_getInfoCallback(&data);

		// open connection to the AMT machine using the data received 
		SPtr<Socket> amt_connection = SPtr<Socket>(NULL);
			
		Logger::GetInstance() << "trying to open connection to Intel(r) AMT machine" << endl;
		// if the _openConnection failed --> continue listening
		if(KVM_STATUS_OK != proxy->_openConnection(&data, amt_connection, vnc_client))
		{
			Logger::GetInstance(Logger::LOG_WARNING) << "can't open connection to amt machine" << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "user: " << data.user << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "domain: " << data.domain << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "host: " << data.host << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "host_port: " << data.host_port << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "proxy: " << data.proxy << endl;
			Logger::GetInstance(Logger::LOG_WARNING) << "proxy_user: " << data.proxy_user << endl;
			continue;
		}

		SPtr<Client> amt_client = SPtr<Client>(new Client(amt_connection, Client::AMT_CLIENT));
		
		Logger::GetInstance() << "Add pair <amt_client:" << amt_client->GetClientID() << " ,vnc_client:" << vnc_client->GetClientID() << ">" << endl;
		
		amt_client->GetSocket()->setNonBlocking();
		vnc_client->GetSocket()->setNonBlocking();

		proxy->AddPair(vnc_client, amt_client);
		
		// send signal socket to wake the select
		int c = 0;
		proxy->_signalSocket->send((UINT8*)&c, 1);
	}
}

void ProxyManager::AddPair(SPtr<Client>& vnc_client, SPtr<Client>& amt_client)
{
	// add the connected clients to the ClientsSet
	{
		Lock l(_clientLock, RWLock::READ_WRITE);
		_clients[vnc_client->GetClientID()] = vnc_client;
		_clients[amt_client->GetClientID()] = amt_client;
	}
	
	// add the pair connection to the bidirectional map
	{
		Lock l(_ConnectionsLock, RWLock::READ_WRITE);
		_ConnectionsMap[vnc_client->GetClientID()] = amt_client->GetClientID();
		_ConnectionsMap[amt_client->GetClientID()] = vnc_client->GetClientID();
	}

	// add the 2 new clients to the listening set
	{
		Lock l(_ListeningLock, RWLock::READ_WRITE);
		_ListeningSet.insert(vnc_client->GetClientID());
		_ListeningSet.insert(amt_client->GetClientID());
	}
}

KVM_STATUS ProxyManager::AuthenticationResult2KVMStatus(AuthenticationResult res)
{
	KVM_STATUS kvm_res = KVM_STATUS_OK;
	switch(res)
	{
		case AuthenticationResultFailure:
			kvm_res = KVM_STATUS_ERROR;
		break;
		case AuthenticationResultSuccess:
			kvm_res = KVM_STATUS_OK;
		break;
	}
	return kvm_res;
}

bool ProxyManager::IsRunning()
{
	Lock l(_runningLock, RWLock::READ_ONLY);
	return _isRunning;
}
void ProxyManager::SetRunning(bool status)
{
	Lock l(_runningLock, RWLock::READ_WRITE);
	_isRunning = status;
}

bool ProxyManager::IsInitialized()
{
	Lock l(_initializeLock, RWLock::READ_ONLY);
	return _isInitialized;
}

void ProxyManager::SetInitialization(bool status)
{
	Lock l(_initializeLock, RWLock::READ_WRITE);
	_isInitialized = status;
}

KVM_STATUS ProxyManager::GetStartUpStatus()
{
	Lock l(_startUpLock, RWLock::READ_ONLY);
	return _startUpStatus;
}

void ProxyManager::SetStartUpStatus(KVM_STATUS status)
{
	Lock l(_startUpLock, RWLock::READ_WRITE);
	_startUpStatus = status;
}
