﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Net;
using System.Security;

namespace Intel.Management.Redirection
{
 
    /// <summary>
    /// KVM Redirection Proxy Library
    /// </summary>
    /// <remarks>The KVM Proxy Library serves as a proxy for viewer applications that connect to Intel AMT platforms using the Intel AMT redirection ports.</remarks>
    public class KVMRedirection :IDisposable
    {
        public const int MAX_PWD_LENGTH = 32;
        public const int MIN_PWD_LENGTH = 8;
        public const int MAX_USERNAME_LENGTH = 32;

        private ConnectionCallback _cb;
        ConnectionInfo _localSettings;

        /// <summary>
        /// Initializes a new instance of the KVMRedirection class
        /// </summary>
        public KVMRedirection()
        {
            _cb = new ConnectionCallback(DoConnectionCallback);
            _localSettings = new ConnectionInfo("localHost");
        }

        /// <summary>
        /// Gets or sets the URL of the target Intel AMT platform. 
        /// </summary>
        public string Host
        {
            get
            {
                return _localSettings.host;
            }
            set {
                //Valdiate Host
                if (Uri.CheckHostName(value) == UriHostNameType.Unknown)
                    throw new ArgumentException("Argument error - Unknown Host name");
                _localSettings.host = value;
            }
        }

        /// <summary>
        /// Gets or sets Port used to connect to the target platform 
        /// </summary>
        public int Port
        {
            get {
                return _localSettings.host_port;
            }
            set {
                //The largest possible source port number is 2^16 (=65535)
                if (value < 0 || value > Math.Pow(2.0, 16.0))
                {
                    throw new ArgumentException("Argument error - Invalid port number");
                }
                else
                    _localSettings.host_port = (ushort)value; 
            }
        }

        /// <summary>
        /// Username for a user with an Intel AMT  ACL entry that has sufficient privileges to initiate a KVM connection.
        /// </summary>
        public string Username
        {
            get {
                return _localSettings.user;
            }
            set {
                //Accepting null value for Kerberos connection
                if (String.IsNullOrEmpty(value))
                    _localSettings.user = "";
                else
                {
                    _localSettings.user = value;
                    if (_localSettings.user.Length > MAX_USERNAME_LENGTH)
                        throw new ArgumentException("Argument error - Username can contain up to 32 characters.");
                }
            }
        }
        /// <summary>
        /// Password for an AMT User ACL
        /// </summary>
        public SecureString Password
        {
            get {
                return _localSettings.pwd;
            }
            set
            {
                //Accepting null value for Kerberos connection
                if (value == null || value.Length == 0)
                {
                    _localSettings.pwd?.Dispose();
                    _localSettings.pwd = new SecureString();
                }
                else
                {
                    if (_localSettings.pwd.Length < MIN_PWD_LENGTH || _localSettings.pwd.Length > MAX_PWD_LENGTH)
                    {
                        _localSettings.pwd.Dispose();
                        throw new ArgumentException("Argument error - Password can contain between 8 to 32 characters.");
                    }    
                    _localSettings.pwd?.Dispose();
                    _localSettings.pwd = value;
                }
            }
        }
        /// <summary>
        /// Gets or sets proxy host FQDN or IP address,
        /// </summary>
        public string Proxy
        {
            get {
                return _localSettings.proxy;
            }
            set {
                //Valdiate Host
                if (Uri.CheckHostName(value) == UriHostNameType.Unknown)
                    throw new ArgumentException("Argument error - Unknown Host name");
                _localSettings.host = value;
            }
        }
        /// <summary>
        /// Gets or sets the proxy server port
        /// </summary>
        public int ProxyPort
        {
            get {
                return _localSettings.proxy_port;
            }
            set {
                //The largest possible source port number is 2^16 (=65535)
                if (value < 0 || value > Math.Pow(2.0, 16.0))
                {
                    throw new ArgumentException("Argument error - Invalid port number");
                }
                else
                    _localSettings.proxy_port = (ushort)value;
            }
        }
        /// <summary>
        /// Gets or sets the proxy server user 
        /// </summary>
        public string ProxyUser
        {
            get { 
                return _localSettings.proxy_user;
            }
            set {
                if (String.IsNullOrEmpty(value))
                    _localSettings.proxy_user = "";
                else
                {
                    _localSettings.proxy_user = value;
                    if (_localSettings.proxy_user.Length > MAX_USERNAME_LENGTH)
                        throw new ArgumentException("Argument error - Username can contain up to 32 characters.");
                }
            }
        }
        /// <summary>
        /// Gets or sets the proxy server password 
        /// </summary>
        public SecureString ProxyPassword
        {
            get {
                return _localSettings.proxy_pwd;
            }
            set {
                if (value == null)
                    _localSettings.proxy_pwd = new SecureString();
                else
                {
                    if (_localSettings.proxy_pwd.Length < MIN_PWD_LENGTH || _localSettings.proxy_pwd.Length > MAX_PWD_LENGTH)
                        throw new ArgumentException("Argument error - Password can contain between 8 to 32 characters.");
                    _localSettings.proxy_pwd?.Dispose();
                    _localSettings.proxy_pwd = value;
                }
            }
        }
        /// <summary>
        /// Gets or sets Windows Domain
        /// </summary>
        public string Domain
        {
            get {
                return _localSettings.domain;
            }
            set {
                _localSettings.domain = value;
            }
        }
        /// <summary>
        /// Gets the output parameter used to differentiate between clients.
        /// </summary>
        public uint ClientID
        {
            get { return _localSettings.clientID; }
        }

        /// <summary>
        /// This function initializes the library. 
        /// </summary>
        /// <returns>zero on success</returns>
        public int Init()
        {
            return Init(string.Empty, string.Empty, null);
        }
        /// <summary>
        /// This function initializes the library. 
        /// </summary>
        /// <param name="rootCert">A path to a root certificate </param>
        /// <param name="personalCert">A path to a client certificate </param>
        /// <param name="personalPassword">Password for the client certificate</param>
        /// <returns>zero on success</returns>
        public int Init(string rootCert, string personalCert, SecureString personalPassword)
        {
            int result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.KVM_Init(rootCert, personalCert, personalPassword, _cb);
            }
            else
            {
                result = Sdk32Bit.KVM_Init(rootCert, personalCert, personalPassword, _cb);
            }
            return result;
        }
        /// <summary>
        /// This function releases the resources acquired by Init. It is called when an application terminates a connection via the proxy.
        /// </summary>
        /// <returns>zero on success</returns>
        public int Deinit()
        {
            int result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.KVM_Deinit();
            }
            else
            {
                result = Sdk32Bit.KVM_Deinit();
            }
            return result;
        }

        /// <summary>
        /// This function updates the certificates used in connection with an already defined connection
        /// </summary>
        /// <param name="rootCert">A path to a root certificate </param>
        /// <param name="personalcert">A path to a client certificate </param>
        /// <param name="personalPassword">Password for the client certificate</param>
        /// <returns>zero on success</returns>
        public int SetCertificates(string rootCert, string personalcert, SecureString personalPassword)
        {
            int result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.KVM_SetCertificates(rootCert, personalcert, personalPassword);
            }
            else
            {
                result = Sdk32Bit.KVM_SetCertificates(rootCert, personalcert, personalPassword);
            }
            return result;
        }
        /// <summary>
        /// This function starts the listening of the proxy to a specified port. 
        /// </summary>
        /// <param name="portNumer">The selected port is returned in the portNumber</param>
        /// <returns>zero on success</returns>
        public int Start(out int portNumer)
        {
            int result;

            //_info = _localSettings;
            ushort sdkPort = 0;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.KVM_Start(ref sdkPort);
            }
            else
            {
                result = Sdk32Bit.KVM_Start(ref sdkPort);
            }

            portNumer = sdkPort;

            return result;
        }

        /// <summary>
        /// Invoking this function stops listening on the selected port.
        /// </summary>
        /// <returns>zero on success</returns>
        public int Stop()
        {
            int result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.KVM_Stop();
            }
            else
            {
                result = Sdk32Bit.KVM_Stop();
            }
            return result;
        }

        /// <summary>
        /// Declaration of a callback function; this function will be passed to
        /// the KVM_init function
        /// </summary>
        /// <param name="s">the structure filled by the callback function</param>
        internal delegate void ConnectionCallback(ref ConnectionInfo s);

        /// <summary>
        /// This function is called when a new VNC connection is
        /// initiated
        /// </summary>
        /// <param name="info">A structure containing the latest configuration settings</param>
        internal void DoConnectionCallback(ref ConnectionInfo info)
        {
            info.host = _localSettings.host;
            info.host_port = _localSettings.host_port;
            info.user = _localSettings.user;
            info.pwd = _localSettings.pwd;

            info.proxy = _localSettings.proxy;
            info.proxy_port = _localSettings.proxy_port;
            info.proxy_user = _localSettings.proxy_user;
            info.proxy_pwd = _localSettings.proxy_pwd;
            info.domain = _localSettings.domain;

            _localSettings.clientID = info.clientID;

        }

        #region IDisposableMembers

        private bool _disposed = false;

        protected virtual void Dispose(bool disposing)
        {
            if (_disposed)
                return;
            if (disposing)
            {
                _localSettings?.Dispose();
            }
            _disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        ~KVMRedirection()
        {
            Dispose(false);
        }
        #endregion
        internal class Sdk64Bit
        {
            [DllImport("KVMLib.dll")]
            public static extern int KVM_Init(string root_cert, string perosnal_cert, SecureString personal_pwd, ConnectionCallback p);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Deinit();

            [DllImport("KVMLib.dll")]
            public static extern int KVM_SetCertificates(string root_cert, string perosnal_cert, SecureString personal_pwd);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Start(ref ushort port);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Stop();
        }

        internal class Sdk32Bit
        {
            [DllImport("KVMLib.dll")]
            public static extern int KVM_Init(string root_cert, string perosnal_cert, SecureString personal_pwd, ConnectionCallback p);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Deinit();

            [DllImport("KVMLib.dll")]
            public static extern int KVM_SetCertificates(string root_cert, string perosnal_cert, SecureString personal_pwd);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Start(ref ushort port);

            [DllImport("KVMLib.dll")]
            public static extern int KVM_Stop();
        }

    }


    /// <summary>
    /// This struct should be filled with platform connection information
    /// when a new viewer connects to the kvmlib
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public class ConnectionInfo:IDisposable
    {
        private const int MAX_STRING_FIELD_SIZE = 256;

        /// <summary>
        /// Intel(r) AMT Hostname / IP Address
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
        public string host;
        /// <summary>
        /// 16994 for non-TLS connections, 16995 for TLS connections
        /// </summary>
        public ushort host_port;
        /// <summary>
        /// Username of Intel(r) AMT user
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
        public string user;
        /// <summary>
        /// Password of Intel(r) AMT user
        /// </summary>
        public SecureString pwd
        {
            get { return pwd; }
            set {
                pwd?.Dispose();
                pwd = value ?? new SecureString();
            }
        }
        /// <summary>
        /// SOCKS proxy Hostname
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
        public string proxy;
        /// <summary>
        ///  SOCKS proxy Port
        /// </summary>
        public ushort proxy_port;
        /// <summary>
        /// Username of the SOCKS proxy user
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
        public string proxy_user;
        /// <summary>
        /// Password of the SOCKS proxy user
        /// </summary>
        public SecureString proxy_pwd
        {
            get { return proxy_pwd; }
            set
            {
                proxy_pwd?.Dispose();
                proxy_pwd = value ?? new SecureString();
            }
        }

        public uint clientID;

        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
        public string domain;

        public ConnectionInfo(string host)
        {
            this.host = host;
            host_port = 5900;
            user = string.Empty;
            proxy = string.Empty;
            proxy_user = string.Empty;
            proxy_port = 8080;
            domain = string.Empty;
            clientID = 0;
            pwd = null;
            proxy_pwd = null;
        }

        #region IDisposable Implementation

        private bool _disposed = false;

        /// <summary>
        /// Implement IDisposable method 
        /// </summary>
        /// <param name="disposing"></param>
        protected virtual void Dispose(bool disposing)
        {
            if (_disposed)
                return;
            if (disposing)
            {
                pwd?.Dispose();
                proxy_pwd?.Dispose();
            }

            _disposed = true;
        }
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        ~ConnectionInfo()
        {
            Dispose(false);
        }
        #endregion


    }

}
