﻿using System;
using System.Collections.Generic;
using System.Text;

using System.Runtime.InteropServices;
using System.Net;
using System.IO;

namespace Intel.Management.Redirection
{

    /// <summary>
    /// Defines error types for the redirection library
    /// </summary>
    public enum IMRResult : int
    {
        /// <summary>
        ///  The requested operation was successfully executed.
        /// </summary>
        Ok = 0,
        /// <summary>
        /// Some unspecified error has occurred while executing the operation.
        /// </summary>
        Error = 1,
        /// <summary>
        ///  An invalid parameter was specified to the function. 
        /// </summary>
        InvalidParameter = 2,
        /// <summary>
        /// The Redirection Library was not yet initialized.
        /// </summary>
        NotInitialized = 3,
        /// <summary>
        /// The Redirection Library has already been initialized.
        /// </summary>
        AlreadyInitialized = 4,
        /// <summary>
        /// Memory allocation has failed; not enough memory.
        /// </summary>
        MemAllocFailed = 5,
        /// <summary>
        ///  The requested operation is unsupported by the library.
        /// </summary>
        Unsupported = 6,
        /// <summary>
        ///  The specified client ID does not exist in the library’s list.
        /// </summary>
        ClientNotFound = 7,
        /// <summary>
        ///  A client with the specified properties (e.g., IP address) already exists in the library’s list.
        /// </summary>
        DuplicateClient = 8,
        /// <summary>
        /// The specified client has not been initialized by the library, although it exists in its list. This indicates an internal error in the library.
        /// </summary>
        ClientNotActive = 9,
        /// <summary>
        ///  The specified client has already been initialized by the library. This indicates an internal error in the library.
        /// </summary>
        ClientActive = 10,
        /// <summary>
        ///  A session of the requested type (either SOL or IDE-R) is already opened with the specified client.
        /// </summary>
        SessionAlreadyOpen = 11,
        /// <summary>
        ///  A session of the requested type (either SOL or IDE-R) is already opened with the specified client.
        /// </summary>
        SessionClosed = 12,
        /// <summary>
        ///  A socket error has occurred.
        /// </summary>
        SocketError = 13,
         /// <summary>
        ///   An internal error has occurred in the protocol handlers in the library.
        /// </summary>
        UnknownProtocol = 14,
        /// <summary>
        ///   An internal error has occurred in the protocol handlers in the library.
        /// </summary>
        ProtocolAlreadyRegistered = 15,
        /// <summary>
        ///   An internal error has occurred while trying to send a packet.
        /// </summary>
        Pending = 16,
        /// <summary>
        ///   An unexpected protocol packet has been received.
        /// </summary>
        UnexpectedPacket = 17,
        /// <summary>
        ///  timeout has occurred while trying to execute the operation.
        /// </summary>
        Timeout = 18,
        /// <summary>
        ///   A corrupted or malformed protocol packet has been received.
        /// </summary>
        CorruptPacket = 19,
        /// <summary>
        ///  An operating system error has occurred.
        /// </summary>
        OperatingSystemError = 20,
        /// <summary>
        ///  The client uses an IDE-R protocol of a version incompatible with the library’s version.
        /// </summary>
        VersionNotSupported = 21,
        /// <summary>
        ///   An IDE-R command is in progress.
        /// </summary>
        IderCommandRunning = 22,
        /// <summary>
        /// An error occurred while trying to store client information.
        /// </summary>
        StorageFailure = 23,
        /// <summary>
        ///  An unknown library result code was specified.
        /// </summary>
        Unknown = 24,
        /// <summary>
        ///  Authentication with the client has failed.
        /// </summary>
        AuthFailed = 25,
        /// <summary>
        /// IDER session cannot be established because of unknown reason (Intel AMT sent an “unknown” status)
        /// </summary>
        ClientTypeUnknown = 26,
        /// <summary>
        /// The client already has an open session of the relevant function (either SOL or IDE-R) with some other application. 
        /// </summary>
        ClientBusy = 27,
        /// <summary>
        ///  The library attempted to connect to a host capability that does not support IDE-R.
        /// </summary>
        ClientUnsupported = 28,
        /// <summary>
        /// General error returned when trying to establish an IDE-R session.
        /// </summary>
        ClientError = 29,
        /// <summary>
        ///  The function was provided with a buffer with insufficient space to successfully complete the operation.
        /// </summary>
        NotEnoughSpace = 30,
        /// <summary>
        ///   The requested operation cannot be executed because the SOL session is in loopback mode.
        /// </summary>
        SessionLoopBack = 31,
        /// <summary>
        ///  The library has failed to establish a TLS connection with the client. There might be a problem verifying the client’s certificate.
        /// </summary>
        TlsConnectionFailed = 32,
        /// <summary>
        ///  SOCKSv5 proxy returned an error.
        /// </summary>
        SocksError = 33,
        /// <summary>
        ///  Library failed to connect due to authentication error.
        /// </summary>
        SocksAuthFail = 34,
        /// <summary>
        ///  Local enable required before function will succeed.
        /// </summary>
        OOBSolDisabled = 35,
        /// <summary>
        ///   Audit log is either full or locked.
        /// </summary>
        AuditLogFull = 36,
        /// <summary>
        /// Invalid parameter in function call
        /// </summary>
        ClientInvalidOption=37,
        /// <summary>
        ///   Command cannot complete until user consent sequence completes.
        /// </summary>
        UserConsentRequired = 38,
        /// <summary>
        /// Must be last entry 
        /// </summary>
        Count = 39,  
    };


    /// <summary>
    /// The Redirection client types
    /// </summary>
    public enum ClientType : int
    {
        /// <summary>
        /// An Intel AMT (non-secure) client
        /// </summary>
        Tcp = 1,
        /// <summary>
        ///An Intel AMT secure client
        /// </summary>
        Tls = 2,
        /// <summary>
        /// Unknown Client
        /// </summary>
        Uknown = 3
    };

    /// <summary>
    /// The Redirection proxy types
    /// </summary>
    public enum ProxyType : int
    {
        /// <summary>
        /// The ProxySettings structure should be ignored.
        /// </summary>
        NoProxy = 0,
        /// <summary>
        /// The ProxySettings structure describes a SOCKSv5 proxy.
        /// </summary>
        SocksV5 = 1,
    };


    /// <summary>
    /// Defines possible IDE-R device states
    /// </summary>
    public enum DeviceState : int
    {
       /// <summary>
       ///   The device is disabled.
       /// </summary>
       Disabled = 0,
       /// <summary>
       ///   The device is enabled
       /// </summary>
        Enabled = 1
    };
        

    /// <summary>
    /// Defines possible timing options for the IMR_IDERSetDeviceState command
    /// </summary>
    public enum SetOption : int
    {
        /// <summary>
        ///  Change of device at PCI reset.
        /// </summary>
        OnReset = 0,
        /// <summary>
        ///  Change the specified device settings in a graceful manner:
        /// </summary>
        Gracefully = 1,
        /// <summary>
        ///  Change the specified cable register settings immediately, whether there is a pending IDE command from its host or not. 
        /// </summary>
        Immediately = 2,
    };

    /// <summary>
    ///  Defines possible commands sent by the console to the client to change the state of the IDE-R devices
    /// </summary>
    public enum SetOperation : int
    {
        /// <summary>
        ///  Request from the client to enable IDE-R devices.
        /// </summary>
        Enable = 0,
        /// <summary>
        ///  Request from the client to disable IDE-R devices.
        /// </summary>
        Disable = 1,
        /// <summary>
        ///  Currently not supported.  If specified, the library will return an error.
        /// </summary>
        Nop = 2
    };

    

    /// <summary>
    ///  Defines possible SetDevice result values returned by the client for each IDER device.
    /// </summary>
    public enum SetResult : int
    {
        /// <summary>
        ///  This result informs the console that the IDE interface is in active use.
        /// </summary>
        Rejected = 0,
        /// <summary>
        ///  Disable/Enable successfully done.
        /// </summary>
        Done = 1
    };

    /// <summary>
    ///  This structure is used to return the library API version.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IMRVersion
    {
        /// <summary>
        ///  The major version number. 
        /// </summary>
        public ushort major;
        /// <summary>
        ///  The minor version number. 
        /// </summary>
        public ushort minor;
    }


    /// <summary>
    ///  This structure is used to pass time-out parameters when establishing an IDE-R session.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IDERTout
    {
        /// <summary>
        ///  The client receive timeout, in milliseconds.
        /// </summary>
        public ushort rx_timeout;
        /// <summary>
        ///  The client’s command transmit timeout. 
        /// </summary>
        public ushort tx_timeout;
        /// <summary>
        ///  The client heartbeat interval.
        /// </summary>
        public ushort hb_timeout;
    }

    /// <summary>
    ///  This structure is used for providing information for a specific client.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1, CharSet = CharSet.Ansi)]
    public struct ClientInfo
    {
        /// <summary>
        ///  The client’s type (TCP or TLS)
        /// </summary>
        public ClientType type;
        /// <summary>
        ///  Client IP address  or the  valid hostname of the client.
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        public string ip;
        /// <summary>
        ///  The Globally Unique ID of the client
        /// </summary>
        public Guid guid;
    }

    /// <summary>
    ///  This structure is used as an output parameter when querying client IDE devices state
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IDERDeviceState
    {
        /// <summary>
        ///  Default primary devices state at the Client FW.
        /// </summary>
        public DeviceState pri_default;
        /// <summary>
        ///  Current primary devices state at the Client FW.
        /// </summary>
        public DeviceState pri_current;
        /// <summary>
        ///  Default secondary devices state at the Client FW.
        /// </summary>
        public DeviceState sec_default;
        /// <summary>
        ///  Current secondary devices state at the Client FW.
        /// </summary>
        public DeviceState sec_current;
    }

    /// <summary>
    ///  This structure is used as an input parameter when sending a “set IDE devices state” command to the client.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IDERDeviceCmd
    {
        /// <summary>
        ///  Primary device operation (enable or disable).
        /// </summary>
        public SetOperation pri_op;
        /// <summary>
        /// Primary device operation timing.
        /// </summary>
        public SetOption pri_timing;
        /// <summary>
        ///  Not supported by the IMR library. The SDK does not support enabling/disabling secondary IDE devices
        /// </summary>
        public SetOperation sec_op;
        /// <summary>
        ///  Not supported by the IMR library
        /// </summary>
        public SetOption sec_timing;
    }
    /// <summary>
    /// This structure is used as an output parameter when setting client IDE devices state. 
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IDERDeviceResult
    {
        /// <summary>
        ///  Set command result for primary devices.
        /// </summary>
        public SetResult pri_res;
        /// <summary>
        /// Set command result for secondary devices. 
        /// </summary>
        public SetResult sec_res;
    }


    /// <summary>
    ///  This structure is used as an output parameter when getting the IDE-R session statistics.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct IDERStatistics
    {
        /// <summary>
        ///  TRUE if session is in ERROR state.
        /// </summary>
        [MarshalAs(UnmanagedType.Bool)]
        public bool error_state;
        /// <summary>
        ///   TRUE if there is a read/write command in progress.
        /// </summary>
        [MarshalAs(UnmanagedType.Bool)]
        /// <summary>
        ///  Number of session re-opens due to error recovery.
        /// </summary>
        public bool data_transfer;
        /// <summary>
        ///   Number of session re-opens due to error recovery.
        /// </summary>
        public ushort num_reopen;
        /// <summary>
        ///  Number of ErrorOccured messages received.
        /// </summary>
        public uint num_error;
        /// <summary>
        ///   Number of ResetOccured messages received.
        /// </summary>
        public uint num_reset;
        /// <summary>
        ///  Last data transfer (read/write) length in bytes.
        /// </summary>
        public uint last_cmd_length;
        /// <summary>
        ///   Bytes of data sent to the client.
        /// </summary>
        public uint data_sent;
        /// <summary>
        ///  Bytes of data received from the client.
        /// </summary>
        public uint data_received;
        /// <summary>
        ///   Messages sent during the session.
        /// </summary>
        public uint packets_sent;
        /// <summary>
        ///   Messages received during the session.
        /// </summary>
        public uint packets_received;
    }

    /// <summary>
    ///  This is the response structure when querying the supported IDE-R features of the client with an IMR_IDERClientFeatureSupported command.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct FeaturesSupported
    {
        /// <summary>
        ///  TRUE if client FW supports enable/disable of primary IDE devices.
        /// </summary>
        [MarshalAs(UnmanagedType.Bool)]
        public bool ider_dev_pri;
        /// <summary>
        ///  TRUE if client FW supports enable/disable of secondary IDE devices.
        /// </summary>
        [MarshalAs(UnmanagedType.Bool)]
        public bool ider_dev_sec;
        /// <summary>
        ///  This field is reserved for future use.
        /// </summary>
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 30)]
        public byte[] reserved;
    }

    /// <summary>
    ///  This structure is used to pass the parameters required to establish an SOL or IDE-R session.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct TCPSessionParams
    {
        public const int MAX_PWD_LENGTH = 32;
        public const int MIN_PWD_LENGTH = 8;
        public const int MAX_USERNAME_LENGTH = 32;

        /// <summary>
        ///  The user’s name
        /// </summary>
        [MarshalAs(UnmanagedType.LPStr, SizeConst = 128)]
        private string userName;
        public string user_name
        {
            get { return userName; }
            set
            {
                //Accepting null value for Kerberos connection
                if (String.IsNullOrEmpty(value))
                    userName = "";
                else
                {
                    userName = value;
                    if (userName.Length > MAX_USERNAME_LENGTH)
                        throw new ArgumentException("Argument error - Username can contain up to 32 characters.");
                }
            }
        }
        /// <summary>
        ///  The user’s password
        /// </summary>
        [MarshalAs(UnmanagedType.LPStr, SizeConst = 128)]
        private string userPswd;
        public string user_pswd
        {
            get { return userPswd; }
            set
            {
                //Accepting null value for Kerberos connection
                if (String.IsNullOrEmpty(value))
                    userPswd = "";
                else
                {
                    userPswd = value;
                    if (userPswd.Length < MIN_PWD_LENGTH || userPswd.Length > MAX_PWD_LENGTH)
                        throw new ArgumentException("Argument error - password must contain between 8 to 32 characters");
                }   
            }
        }
    }

    /// <summary>
    ///  This structure is used to pass the parameters required to establish an SOL or IDE-R session with Proxy Settings
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct TCPSessionParamsEx
    {
        #region consts
        public const int MAX_PWD_LENGTH = 32;
        public const int MIN_PWD_LENGTH = 8;
        public const int MAX_USERNAME_LENGTH = 32;
        #endregion
        /// <summary>
        ///  Fixed value of 2
        /// </summary>
        public int version { get; set; } // must be 2
        /// <summary>
        ///  The user’s name
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        private string userName;
        public string user_name
        {
            get { return userName; }
            set
            {
                //Accepting null value for Kerberos connection
                if (String.IsNullOrEmpty(value))
                    userName = "";
                else
                {
                    userName = value;
                    if (userName.Length > MAX_USERNAME_LENGTH)
                        throw new ArgumentException("Invalid Argument - Username can contain up to 32 characters.");
                }
            }
        }

        /// <summary>
        ///  The user’s password
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        private string userPswd;
        public string user_pswd
        {
            get { return userPswd; }
            set
            {
                //Accepting null value for Kerberos connection
                if (String.IsNullOrEmpty(value))
                    userPswd = "";
                else
                {
                    userPswd = value;
                    if (userPswd.Length < MIN_PWD_LENGTH || userPswd.Length > MAX_PWD_LENGTH)
                        throw new ArgumentException("Invalid Argument - password must contain between 8 to 32 characters");
                }   
            }
        }

        /// <summary>
        ///  Determines ProxyType (None or Socksv5)
        /// </summary>
        public ProxyType proxy_type { get; set; }
        /// <summary>
        ///  MPS Hostname or IPv4 IP address in a.b.c.d notation
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        private string proxyServer;
        public string proxy_server
        {
            get { return proxyServer; }
            set
            {
                if (Uri.CheckHostName(value) == UriHostNameType.Unknown)
                    throw new ArgumentException("Argument error - Unknown Hostname.");
                proxyServer = value;
            }
        }
        /// <summary>
        ///  Port used by the MPS to accept SOCKSv5 connections
        /// </summary>
        private int proxyPort;
        public int proxy_port
        {
            get { return proxyPort; }
            set
            {
                //The largest possible source port number is 2^16 (=65535)
                if (value < 0 || value > Math.Pow(2.0, 16.0))
                    throw new ArgumentException("Argument error - Invalid port number");
                proxyPort = value;
            }
        }
        /// <summary>
        /// SOCKSv5 proxy username. 
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        private string proxyUser;
        public string proxy_user
        {
            get { return proxyUser; }
            set
            {
                if (String.IsNullOrEmpty(value))
                    proxyUser = "";
                else
                {
                    proxyUser = value;
                    if (proxyUser.Length > MAX_USERNAME_LENGTH)
                        throw new ArgumentException("Argument error - Username can contain up to 32 characters.");
                }
            }
        }
        /// <summary>
        ///  SOCKSv5 proxy password. 
        /// </summary>
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 128)]
        private string proxyPswd;
        public string proxy_pswd
        {
            get { return proxyPswd; }
            set
            {
                if (String.IsNullOrEmpty(value))
                    proxyPswd = "";
                else
                {
                    proxyPswd = value;
                    if (proxyPswd.Length < MIN_PWD_LENGTH || proxyPswd.Length > MAX_PWD_LENGTH)
                        throw new ArgumentException("Invalid Argument - password must contain between 8 to 32 characters");
                }
            }
        }
    }

    /// <summary>
    ///  These values are defined to select a loopback mode for SOL sessions.  
    /// </summary>
    public enum SOLLoopbackMode : int
    {
        /// <summary>
        ///  No loopback is used in the SOL session.
        /// </summary>
        None = 0,
        /// <summary>
        ///  The session uses loopback as defined by the RS232C specification in Microsoft Windows Hardware Compatibility Test (HCT) Kit, in the Serial Port Stress Test section.
        /// </summary>
        RS232C = 1
    }

    /// <summary>
    ///  This structure is used to pass time-out parameters when establishing an SOL session.
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1, CharSet = CharSet.Ansi)]
    public struct SOLTout
    {
        /// <summary>
        ///  The client transmit overflow timeout. 
        /// </summary>
        public ushort	tx_over_timeout;
        /// <summary>
        ///  The client transmit buffering timeout. 
        /// </summary>
        public ushort	tx_buf_timeout;
        /// <summary>
        ///  The client heartbeat interval.
        /// </summary>
        public ushort	hb_interval;
        /// <summary>
        ///  The client’s FIFO receive flush timeout. 
        /// </summary>
        public ushort	fifo_rx_flush_timeout;
        /// <summary>
        ///  The client’s receive timeout.
        /// </summary>
        public ushort   rx_timeout;
    }

   

    /// <summary>
    ///  Valid values for the optname parameter when using the IMR_SetOpt and IMR_GetOpt functions
    /// </summary>
    public enum SetClientOption : int
    {
        /// <summary>
        ///  Legacy ME Link Control
        /// </summary>
        ME = 0,
        /// <summary>
        ///  Console Link Preference 
        /// </summary>
        Host = 1,
    }

    /// <summary>
    ///  Class representing the Loopback Parameter 
    /// </summary>
    public class SOLParams
    {
        SOLLoopbackMode _mode;

        /// <summary>
        /// Initializes a new instance of the SOLParams class 
        /// </summary>
        public SOLParams()
        {
            _mode = SOLLoopbackMode.None;
        }

        /// <summary>
        /// Gets or sets the SOL loopback mode
        /// </summary>
        public SOLLoopbackMode Mode
        {
            get
            {
                return _mode;
            }
            set
            {
                _mode = value;
            }
        }
    }

    /// <summary>
    ///  Class representing a SOL receive buffer
    /// </summary>
    public class ReceiveBuffer
    {
        int _size;
        byte[] _data;

   

        /// <summary>
        /// Initializes a new instance of the ReceiveBuffer of a given size
        /// </summary>
        /// <param name="size">The size of the buffer</param>
        public ReceiveBuffer(int size)
        {
            _size = size;
        }

        /// <summary>
        /// Initializes a new instance of the ReceiveBuffer with and existing byte array
        /// </summary>
        /// <param name="size">exiting byte array</param>
        public ReceiveBuffer(byte[] data)
        {
            _size = data.Length;
            _data = data;
        }

        /// <summary>
        ///  Gets or sets the size of the data associated with the buffer
        /// </summary>
        public int Size
        {
            get
            {
                return _size;
            }
            set
            {
                _size = value;
            }
        }

        /// <summary>
        ///  Gets or sets the data associated with the buffer
        /// </summary>
        public byte[] Data
        {
            get
            {
                if (_data == null)
                    _data = new byte[_size];
                return _data;
            }
            set
            {
                _data = value;
                _size = Data.Length;
            }
        }
    }

    /// <summary>
    ///  Media Redirection Library Class
    /// </summary>
    public class MediaRedirection
    {

        /// <summary>
        /// Initializes a new instance of the MediaRedirection with and existing byte array
        /// </summary>
        public MediaRedirection() { }

  
        /// <summary>
        /// This function initializes the library.
        /// </summary>
        /// <param name="version">Library version structure</param>
        /// <returns>An IMRResult class containing the result code</returns>
        /// <remarks>Must be called before calling any other API function of this library.</remarks>
        public IMRResult Init(out IMRVersion version)
        {
            return Init(out version, null);
        }

        /// <summary>
        /// This function initializes the library.
        /// </summary>
        /// <param name="version">Library version structure</param>
        /// <param name="iniFile">Path to the Library INI file</param>
        /// <returns>An IMRResult class containing the result code</returns>
        /// <remarks>Must be called before calling any other API function of this library.</remarks>
        public IMRResult Init(out IMRVersion version, string iniFile)
        {
            version = new IMRVersion();
            IMRResult result;
            if (!System.IO.File.Exists(iniFile))
                throw new ArgumentException("File not found.");
            FileInfo fileInfo = new FileInfo(iniFile);
            if (fileInfo.Length == 0)
                throw new ArgumentException("The selected file is empty.");

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_Init(ref version, iniFile);
            }
            else
            {
                result = Sdk32Bit.IMR_Init(ref version, iniFile);
            }
            return result;
        }

        /// <summary>
        /// This function shuts down the library, releasing all system resources.
        /// </summary>
        /// <returns>An IMRResult class containing the result code</returns>
        /// <remarks>After calling this function, no other API function of this library may be called, except for IMR_Init().</remarks>
        public IMRResult Close()
        {
            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_Close();
            }
            else
            {
                result = Sdk32Bit.IMR_Close();
            }
            return result;
        }

        /// <summary>
        /// This function adds a client to the library internal client list.
        /// </summary>
        /// <param name="clientInfo">ClientInfo properties</param>
        /// <param name="clientId">Resulting client ID</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult AddClient(ClientInfo clientInfo, out uint clientId)
        {
            return AddClient(clientInfo.type,clientInfo.ip,clientInfo.guid,out clientId);
        }


        /// <summary>
        /// This function adds a client to the library internal client list.
        /// </summary>
        /// <param name="clientType">The type of client to be added.</param>
        /// <param name="ip">Client’s IP address or a valid hostname</param>
        /// <param name="clientId">Resulting client ID</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult AddClient(ClientType clientType, string ip, out uint clientId)
        {
            Guid uuid = Guid.Empty;
            return AddClient(clientType,ip,uuid, out clientId);
        }


        /// <summary>
        /// This function adds a client to the library internal client list.
        /// </summary>
        /// <param name="clientType">The type of client to be added.</param>
        /// <param name="ip">Client’s IP address or a valid hostname</param>
        /// <param name="guid">The client’s Global Unique ID.</param>
        /// <param name="clientId">Resulting client ID</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult AddClient(ClientType clientType, string ip, Guid guid, out uint clientId)
        {
            if (String.IsNullOrEmpty(ip))
                throw new ArgumentException("Argument error - Host name is null or an empty string.");
            if (Uri.CheckHostName(ip) == UriHostNameType.Unknown)
                throw new ArgumentException("Argument error - Unknown Host name.");

            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_AddClient(clientType,ip,ref guid,out clientId);
            }
            else
            {
                result = Sdk32Bit.IMR_AddClient(clientType, ip, ref guid, out clientId);
            }
            return result;
        }

        /// <summary>
        /// This function removes a client from the library’s internal client list.
        /// </summary>
        /// <param name="clientId">The Client ID</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult RemoveClient(uint clientId)
        {
            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_RemoveClient(clientId);
            }
            else
            {
                result = Sdk32Bit.IMR_RemoveClient(clientId);
            }
            return result;
        }
        
        /// <summary>
        /// This function removes all clients from the library’s internal client list.
        /// </summary>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult RemoveAllClients()
        {
            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_RemoveAllClients();
            }
            else
            {
                result = Sdk32Bit.IMR_RemoveAllClients();
            }
            return result;
        }

        /// <summary>
        /// This function returns the properties of the specified client.
        /// </summary>
        /// <param name="hostUri">The uri to the amt host</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public ClientInfo GetClientInfo(Uri hostUri)
        {
            ClientInfo result = new ClientInfo();

            if (hostUri == null)
                throw new ArgumentException("Invalid Argument - Invalid Uri");

            result.guid = Guid.Empty;
            
            result.ip = hostUri.Host;
            if (string.Compare(hostUri.Scheme,"https",true)==0)
            {
                result.type = ClientType.Tls;
            }
            else if (string.Compare(hostUri.Scheme,"http",true)==0)
            {
                result.type = ClientType.Tcp;
            }
            else
            {
                result.type = ClientType.Uknown;
            }

            return result;
        }

        /// <summary>
        /// This function returns the properties of the specified client.
        /// </summary>
        /// <param name="clientId">A client ID</param>
        /// <param name="info">The client properties</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult GetClientInfo(uint clientId, out ClientInfo info)
        {
            info = new ClientInfo();
            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_GetClientInfo(clientId, ref info);
            }
            else
            {
                result = Sdk32Bit.IMR_GetClientInfo(clientId, ref info);
            }
            return result;
        }


        /// <summary>
        /// Creates library session parameters
        /// </summary>
        /// <returns>A TCPSessionParamsEx structure</returns>
        public TCPSessionParamsEx CreateSessionParams()
        {
            return CreateSessionParams(string.Empty, string.Empty);
        }

        /// <summary>
        /// Creates library session parameters
        /// </summary>
        /// <param name="user">AMT user name</param>
        /// <param name="password">AMT password</param>
        /// <returns>A TCPSessionParamsEx structure with user credentials</returns>
        public TCPSessionParamsEx CreateSessionParams(string user, string password)
        {
            TCPSessionParamsEx result = new TCPSessionParamsEx();
            result.version = 2;

            result.user_name = user;
            result.user_pswd = password;

            result.proxy_type = ProxyType.NoProxy;

         



            return result;

        }

        /// <summary>
        /// Creates a structure with default IDER timemouts
        /// </summary>
        /// <returns>A IDERTout structure</returns>
        public IDERTout GetDefaultIDERTout()
        {
            IDERTout result = new IDERTout();
            result.rx_timeout = 10000;
            result.tx_timeout = 0;
            result.hb_timeout = 5000;

            return result;
        }

      
        /// <summary>
        /// This function opens an IDE-R session with the specified client over a new TCP connection.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="tcpParams">A TCPSessionParamsEx structure</param>
        /// <param name="touts">A IDERTout with timeout values</param>
        /// <param name="drive0">name of the floppy disk drive</param>
        /// <param name="drive1">name of the CD or DVD drive</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult IDEROpenTCPSession(uint clientId, TCPSessionParamsEx tcpParams,
                                    IDERTout touts,
                                    string drive0, string drive1)
        {
            //If this parameter is NULL, the function GetDriveType uses the root of the current directory.
            if (GetDriveType(drive0 + "\\") == DriveType.Unknown || GetDriveType(drive0 + "\\") == DriveType.NoRootDirectory)
                throw new ArgumentException(drive0 + " - Is an invalid Drive.");
            if (GetDriveType(drive1 + "\\") == DriveType.Unknown || GetDriveType(drive1 + "\\") == DriveType.NoRootDirectory)
                throw new ArgumentException(drive1 + " - Is an invalid Drive.");

            IMRResult result;
            if (tcpParams.version == 0)
                tcpParams.version = 2;

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDEROpenTCPSessionEx(clientId, ref tcpParams,
                                    ref touts,
                                    drive0, drive1);
            }
            else
            {
                result = Sdk32Bit.IMR_IDEROpenTCPSessionEx(clientId, ref tcpParams,
                                    ref touts,
                                    drive0, drive1);
            }


            return result;
        }

        /// <summary>
        /// This function sets information that is necessary for certificate verification when establishing a TLS session.
        /// </summary>
        /// <param name="rootCert">The path to a file containing Certificate Authority certificates</param>
        /// <param name="privateCert">The path to a file containing a private certificate </param>
        /// <param name="certPass">String specifying the password protecting the privateCert file</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SetCertificateInfo(string rootCert, string privateCert, string certPass)
        {
            IMRResult result;
            FileInfo fileInfo;
            if (!System.IO.File.Exists(rootCert))
                throw new ArgumentException("File not found.");
            fileInfo = new FileInfo(rootCert);
            if (fileInfo.Length == 0)
                throw new ArgumentException("The selected file is empty.");

            //Using private certificate
            if (!String.IsNullOrEmpty(privateCert))
            {
                if (!System.IO.File.Exists(privateCert))
                    throw new ArgumentException("File not found.");
                fileInfo = new FileInfo(privateCert);
                if (fileInfo.Length == 0)
                    throw new ArgumentException("The selected file is empty.");
            }

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SetCertificateInfo(rootCert,privateCert,certPass);
            }
            else
            {
                result = Sdk32Bit.IMR_SetCertificateInfo(rootCert, privateCert, certPass);
            }
            return result;
        }

        /// <summary>
        /// This function sets information that is necessary for client certificate verification when establishing a TLS session with mutual authentication.
        /// </summary>
        /// <param name="commonName">The client certificate common name</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SetClientCertificate(string commonName)
        {
            IMRResult result;

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SetClientCertificate(commonName);
            }
            else
            {
                result = Sdk32Bit.IMR_SetClientCertificate(commonName);
            }
            return result;
        }
        /// <summary>
        /// This function sets the option of accepting self-signed certificate in TLS connection.
        /// </summary>
        /// <param name="acceptSelfSignedCert"></param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SetAcceptSelfSignedCertificate(bool acceptSelfSignedCert)
        {
            IMRResult result;

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SetAcceptSelfSignedCertificate(acceptSelfSignedCert);
            }
            else
            {
                result = Sdk32Bit.IMR_SetAcceptSelfSignedCertificate(acceptSelfSignedCert);
            }
            return result;
        }

        /// <summary>
        /// This function queries the state of client IDE devices.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="state">A IDERDeviceState structure containing the state</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult IDERGetDeviceState(uint clientId, out IDERDeviceState state)
        {
            IMRResult result;
                  
            state = new IDERDeviceState();
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDERGetDeviceState(clientId,ref state);
            }
            else
            {
                result = Sdk32Bit.IMR_IDERGetDeviceState(clientId,ref state);
            }
            return result;
        }

        /// <summary>
        /// This function controls the client IDE device(s) state. Devices can be disabled and enabled through this function.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="cmd">A IDERDeviceCmd containing the command to send</param>
        /// <param name="deviceResult">A IDERDeviceResult containing the device result</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult IDERSetDeviceState(uint clientId, IDERDeviceCmd cmd, out IDERDeviceResult deviceResult)
        {
            IMRResult result;
            deviceResult = new IDERDeviceResult();

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDERSetDeviceState(clientId, ref cmd,ref deviceResult);
            }
            else
            {
                result = Sdk32Bit.IMR_IDERSetDeviceState(clientId,ref cmd, ref deviceResult);
            }
            return result;
        }

        /// <summary>
        /// This function polls the active IDE-R session.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="stats">A IDERStatistics class containing the session statistics</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult IDERGetSessionStatistics(uint clientId, out IDERStatistics stats)
        {
            IMRResult result;
            stats = new IDERStatistics();

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDERGetSessionStatistics(clientId, ref  stats);
            }
            else
            {
                result = Sdk32Bit.IMR_IDERGetSessionStatistics(clientId, ref stats);
            }
            return result;
        }

        /// <summary>
        /// This function queries the client about the special features that it supports.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="supported">A FeaturesSupported structure</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult IDERClientFeatureSupported(uint clientId, out FeaturesSupported supported)
        {
            IMRResult result;
            supported = new FeaturesSupported();

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDERClientFeatureSupported(clientId, ref supported);
            }
            else
            {
                result = Sdk32Bit.IMR_IDERClientFeatureSupported(clientId, ref supported);
            }
            return result;
        }

        /// <summary>
        /// This function closes an open IDE-R session with the specified client. 
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <returns></returns>
        /// <remarks>After the function returns no other operations can be performed. Closing a session during a write data operation can cause data corruption.</remarks>
        public IMRResult IDERCloseSession(uint clientId)
        {
            IMRResult result;
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_IDERCloseSession(clientId);
            }
            else
            {
                result = Sdk32Bit.IMR_IDERCloseSession(clientId);
            }
            return result;
        }

        /// <summary>
        /// Gets default SOL Timeout values
        /// </summary>
        /// <returns>A SOLTout structure</returns>
        public SOLTout GetDefaultSOLTout()
        {
            SOLTout result = new SOLTout();
            result.rx_timeout = 10000;
            result.fifo_rx_flush_timeout = 100;
            result.hb_interval = 5000;
            result.tx_buf_timeout = 100;
            result.tx_over_timeout = 0;

            return result;
        }

        /// <summary>
        /// Get default SOL params
        /// </summary>
        /// <returns>A SOLParams class</returns>
        public SOLParams GetDefaultSOLParams()
        {
            return new SOLParams();
        }

        /// <summary>
        /// This function opens an SOL session with the specified client over a new TCP connection.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="tcpParams">A TCPSessionParamsEx structure with sessions properties</param>
        /// <param name="touts">A SOLTout structure with timeout values</param>
        /// <param name="loopback">A SOLParams with loopback preference</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SOLOpenTCPSession(uint clientId, TCPSessionParamsEx tcpParams,
                       SOLTout touts, SOLParams loopback)
        {

            IMRResult result;
            SOLLoopbackMode copyMode = loopback.Mode;
       
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SOLOpenTCPSessionEx(clientId, tcpParams, touts, ref copyMode);
            }
            else
            {
                TCPSessionParamsEx copyParams = tcpParams;
                SOLTout copyTout = touts;
                result = Sdk32Bit.IMR_SOLOpenTCPSessionEx(clientId, ref copyParams, ref copyTout, ref copyMode);
            }
            loopback.Mode = copyMode;
            return result;
        }

        

        /// <summary>
        /// This function closes an open SOL session with the specified client. 
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <returns>An IMRResult class containing the result code</returns>
        /// <remarks>After the function returns no other operations can be performed until a new session is initialized.</remarks>
        public IMRResult SOLCloseSession(uint clientId)
        {
            IMRResult result;

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SOLCloseSession(clientId);
            }
            else
            {
                result = Sdk32Bit.IMR_SOLCloseSession(clientId);
            }
            return result;
        }

        /// <summary>
        /// This function retrieves SOL data that has been stored.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="buffer">The Buffer into which received data will be copied.</param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SOLRecieveText(uint clientId, ReceiveBuffer buffer)
        {
            IMRResult result;
           
            int copySize = buffer.Size;
            if (IntPtr.Size == 8)
            {

                result = Sdk64Bit.IMR_SOLReceiveText(clientId, buffer.Data, ref copySize);
       
            }
            else
            {
                result = Sdk32Bit.IMR_SOLReceiveText(clientId, buffer.Data, ref copySize);
            }
            buffer.Size = copySize;
            return result;
        }

        /// <summary>
        /// This function sends text (keyboard input) to the client, where it will be received as incoming data from the serial controller.
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="data">The buffer of data to be sent. </param>
        /// <param name="len">The length of the data to be sent.  </param>
        /// <returns>An IMRResult class containing the result code</returns>
        public IMRResult SOLSendText(uint clientId, byte[] data, int len)
        {
            IMRResult result;

            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_SOLSendText(clientId, data, len);
            }
            else
            {
                result = Sdk32Bit.IMR_SOLSendText(clientId, data, len);
            }
            return result;
        }

        /// <summary>
        /// This function returns the error string associated with the specified error code.
        /// </summary>
        /// <param name="result">One of the library error codes.</param>
        /// <returns>A string matching the specified error code </returns>
        public string GetErrorString(IMRResult result)
        {
            StringBuilder builder = new StringBuilder(1024);
            if (IntPtr.Size == 8)
            {
                result = Sdk64Bit.IMR_GetErrorString(result, builder);
            }
            else
            {
                result = Sdk32Bit.IMR_GetErrorString(result, builder);
            }

            return builder.ToString();
            
        }

        /// <summary>
        /// Create a TerminalLink that a Terminal Window can connect too
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <returns>A TerminalLink for a Terminal Window</returns>
        public TerminalLink CreateTerminalLink(uint clientId)
        {
            return new TerminalLink(clientId,this);
        }

        internal class Sdk32Bit
        {
            [DllImport("imrsdk.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_Init(ref IMRVersion version, string ini_file);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_Close();

            [DllImport("imrsdk.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_AddClient(ClientType new_client_type, string client_ip, ref Guid guid, out uint new_client_id);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_RemoveClient(uint client_id);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_RemoveAllClients();

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_GetClientInfo(uint client_id, ref ClientInfo info);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDERCloseSession(uint client_id);

            [DllImport("imrsdk.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_IDEROpenTCPSession(uint client_id, ref TCPSessionParams tcpParams,
                                    ref IDERTout touts,
                                    string drive0, string drive1);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDEROpenTCPSessionEx(uint client_id, ref TCPSessionParamsEx tcpParams,
                                    ref IDERTout touts,
                                    string drive0, string drive1);


            [DllImport("imrsdk.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_SetCertificateInfo(string root_cert,
                string private_cert, string cert_pass);

            [DllImport("imrsdk.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_SetClientCertificate(string common_name);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SetAcceptSelfSignedCertificate(bool acceptSelfSignedCert);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDERGetDeviceState(uint client_id, ref IDERDeviceState state);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDERSetDeviceState(uint client_id, ref IDERDeviceCmd cmd, ref IDERDeviceResult result);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDERGetSessionStatistics(uint client_id, ref IDERStatistics stat);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_IDERClientFeatureSupported(uint client_id, ref FeaturesSupported supported);

          
            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SOLOpenTCPSessionEx(uint client_id, ref TCPSessionParamsEx tcpParams,
                            ref SOLTout touts, ref SOLLoopbackMode loopback);


            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SOLOpenTCPSessionOOB(uint client_id, TCPSessionParamsEx tcpParams,
                            SOLTout touts, ref SOLLoopbackMode loopback);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SOLCloseSession(uint client_id);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SOLSendText(uint client_id, byte[] data, int len);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_SOLReceiveText(uint client_id,byte[] data,ref int len);

            [DllImport("imrsdk.dll")]
            public static extern IMRResult IMR_GetErrorString(IMRResult r, [MarshalAs(UnmanagedType.LPStr)] StringBuilder str);

        }      

        internal class Sdk64Bit
        {
            [DllImport("imrsdk_x64.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_Init(ref IMRVersion version, string ini_file);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_Close();

            [DllImport("imrsdk_x64.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_AddClient(ClientType new_client_type, string client_ip, ref Guid guid, out uint new_client_id);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_RemoveClient(uint client_id);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_RemoveAllClients();

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_GetClientInfo(uint client_id, ref ClientInfo info);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDERCloseSession(uint client_id);

            [DllImport("imrsdk_x64.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_IDEROpenTCPSession(uint client_id, ref TCPSessionParams tcpParams,
                                    ref IDERTout touts,
                                    string drive0, string drive1);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDEROpenTCPSessionEx(uint client_id, ref TCPSessionParamsEx tcpParams,
                                    ref IDERTout touts,
                                    string drive0, string drive1);


            [DllImport("imrsdk_x64.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_SetCertificateInfo(string root_cert,
                string private_cert, string cert_pass);

            [DllImport("imrsdk_x64.dll", CharSet = CharSet.Ansi)]
            public static extern IMRResult IMR_SetClientCertificate(string common_name);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SetAcceptSelfSignedCertificate(bool acceptSelfSignedCert);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDERGetDeviceState(uint client_id, ref IDERDeviceState state);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDERSetDeviceState(uint client_id, ref IDERDeviceCmd cmd, ref IDERDeviceResult result);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDERGetSessionStatistics(uint client_id, ref IDERStatistics stat);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_IDERClientFeatureSupported(uint client_id, ref FeaturesSupported supported);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SOLOpenTCPSessionEx(uint client_id, TCPSessionParamsEx tcpParams,
                            SOLTout touts,  ref SOLLoopbackMode loopback);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SOLOpenTCPSessionOOB(uint client_id, TCPSessionParamsEx tcpParams,
                            SOLTout touts, ref SOLLoopbackMode loopback);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SOLCloseSession(uint client_id);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SOLSendText(uint client_id,byte[] data, int len);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SOLReceiveText(uint client_id,byte[] data,ref int len);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_GetErrorString(IMRResult r, [MarshalAs(UnmanagedType.LPTStr)] StringBuilder str);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_SetOpt(uint client_id, int optname,byte[] opvalue,int optlen);

            [DllImport("imrsdk_x64.dll")]
            public static extern IMRResult IMR_GetOpt(uint client_id, int optname,byte[] opvalue,ref int optlen);

        }
        [DllImport("kernel32.dll")]
        public static extern DriveType GetDriveType([MarshalAs(UnmanagedType.LPStr)] string lpRootPathName);
    }
}
