﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Threading;
using System.Net.Sockets;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Intel.Management.Redirection
{

    /// <summary>
    /// Defines a terminal link for a terminal console 
    /// </summary>
    public class TerminalLink: IDisposable
    {
    
      
        MediaRedirection _imrClass;
        int _port;
        uint _clientId;
        bool _telnetEscaping;
        
        Thread _writeThread;
        Thread _readThread;

        private ManualResetEvent _readyEvent = new ManualResetEvent(false);
        private ManualResetEvent _connectedEvent = new ManualResetEvent(false);
        private ManualResetEvent _readEvent = new ManualResetEvent(false);
        private ManualResetEvent _writeEvent = new ManualResetEvent(false);
        private ManualResetEvent _shutdownEvent = new ManualResetEvent(true);

        static int _messageSize = 64*1024;


        byte[] _data;
        byte[] _input;
        Socket _socket;
        Socket _rSocket;
        Exception _readExp;
        Exception _writeExp;
        IMRResult _lastResult;

        internal class WriteState
        {
            internal Socket rSocket;
            internal MemoryStream stream;


            internal WriteState( Socket r, MemoryStream m)
            {
                rSocket = r;
                stream = m;
            }
        }

  
        /// <summary>
        /// Initializes a new instance of the KVMRedirection class
        /// </summary>
        /// <param name="clientId">The ID of the relevant client.</param>
        /// <param name="imrClass">A MediaRedirection class</param>
        public TerminalLink(uint clientId,MediaRedirection imrClass)
        {
            _imrClass = imrClass;

            _clientId = clientId;
            _port = 2000;
            _telnetEscaping = true;


            _data=null;
            _input=null;
            _socket=null;
            _rSocket=null;
            _readExp=null;
            _writeExp=null;
            _lastResult= IMRResult.SessionClosed;

            ThreadStart threadDelegate = new ThreadStart(ProcessRead);
            _readThread = new Thread(threadDelegate);
         

            threadDelegate = new ThreadStart(ProcessWrite);
            _writeThread = new Thread(threadDelegate);
        }


      
        /// <summary>
        /// Opens a terminal 
        /// </summary>
        /// <returns>True if the link is opened</returns>
        public bool Open()
        {

            if (!_shutdownEvent.WaitOne(0))
                throw new InvalidOperationException();

            //set all the events to non-signaled 
            _readEvent.Reset();
            _writeEvent.Reset();
            _readyEvent.Reset();
            _connectedEvent.Reset();

            _shutdownEvent.Reset(); //set to non-signaled (false)

            _readThread.Start();
            _writeThread.Start();
            
            //wait for connect to begin
            _readyEvent.WaitOne();
         

            //don't wait should return non-singnaled (false) 
            return !_shutdownEvent.WaitOne(0);
        }

        /// <summary>
        /// Closes a terminal link
        /// </summary>
        public void Close()
        {
            try
            {
                _shutdownEvent.Set();
                _connectedEvent.Set();
                
                _imrClass.SOLCloseSession(_clientId);
       
                //wait for the write thread to shutdown
                _writeThread.Join();

                _data = null;
                _input = null;
				if(_socket != null && _socket.Connected) _socket.Close();
                if(_rSocket != null && _rSocket.Connected) _rSocket.Close();
                _readExp = null;
                _writeExp = null;
                _lastResult = IMRResult.SessionClosed;
            }
            catch (Exception e)
            {
                throw;
            }
        }

        private void ProcessRead()
        {
            try
            {
                // create a socket for the terminal
                _socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                
                //bind the socket to a local port
                IPEndPoint ipLocal = new IPEndPoint(IPAddress.Loopback, _port);
                _socket.Bind(ipLocal);
                _socket.Listen(15);
                
             
                IAsyncResult asyncResult = _socket.BeginAccept(new AsyncCallback(AcceptCallback),null);
                _readyEvent.Set();

                //wait for the connect to be done
                _connectedEvent.WaitOne();

                
                while (!_shutdownEvent.WaitOne(0))
                {
                    if (_data == null)
                        _data = new byte[_messageSize];

                    EndPoint endpoint = _rSocket.RemoteEndPoint;

                    _readEvent.Reset();
                    _rSocket.BeginReceiveFrom(_data, 0, _data.Length, SocketFlags.None,
                        ref endpoint, new AsyncCallback(ReceiveCallback), null);

                    _readEvent.WaitOne();
                }
            }
            catch (Exception e)
            {
                _readExp = e;
            }
            finally
            {
                _readyEvent.Set();
                _shutdownEvent.Set();

                //wait for reading thread to shutdown 
                _writeThread.Join();
                if (_rSocket != null && _rSocket.Connected)
                {
                    _rSocket.Shutdown(SocketShutdown.Both);
                    _rSocket.Close();
                }
                if (_socket != null && _socket.Connected)
                {
                    _socket.Close();
                }
            }

        }

        private void ProcessWrite()
        {
            IMRResult result = IMRResult.Ok;
            try
            {
                _connectedEvent.WaitOne();


                int waitFor = 5;
                while (!_shutdownEvent.WaitOne(waitFor))
                {
                    int len = 0;
                    int sendLen = 0;

                    if (_input == null)
                        _input = new byte[_messageSize];

                    ReceiveBuffer solBuffer = new ReceiveBuffer(_input);

                    result = _imrClass.SOLRecieveText(_clientId, solBuffer);
                    if (result == IMRResult.Ok && solBuffer.Size >0)
                    {
                        using(MemoryStream inStream = new MemoryStream())
                        {
                            sendLen = 0;
                            len = solBuffer.Size;

                            // escape IAC codes  xOff
                            for (int i = 0; i < len; i++)
                            {
                                byte b = solBuffer.Data[i];
                                if (b == byte.MaxValue && _telnetEscaping)
                                {
                                    inStream.WriteByte(b);
                                    sendLen++;
                                }
                                inStream.WriteByte(b);
                                sendLen++;
                            }

                            //send data to Link

                            _writeEvent.Reset();
                            _rSocket.BeginSend(inStream.GetBuffer(), 0, sendLen, SocketFlags.None,
                                        new AsyncCallback(SendCallback), inStream);
                            _writeEvent.WaitOne();
                        }
                        waitFor = 0;
                       
                       
                    }
                    else if (result==IMRResult.Ok)
                    {
                        waitFor = 5;
                    }
                    else
                    {
                        _shutdownEvent.Set();
                        _lastResult = result;
                    }
                }
            }
            catch (Exception e)
            {
                _writeExp = e;   
            }

            //alows set the IMResult to somthing to end read thread
            _shutdownEvent.Set();
          
        }

    
        
        private void AcceptCallback(IAsyncResult result)
        {
     
            _rSocket = _socket.EndAccept(result);
            _rSocket.Blocking = false;
            _connectedEvent.Set();
             
        }

        private void ReceiveCallback(IAsyncResult result)
        {
            try
            {
                int len = _rSocket.EndReceive(result);
                int sendLen = 0;

                using (MemoryStream outStream = new MemoryStream())
                {
                    for (int i = 0; i < len; i++)
                    {
                        //skip IAC code (0xFF 0xFF)
                        if (_data[i] == byte.MaxValue && i > 0 && _data[i - 1] == byte.MaxValue)
                            continue; // skip the second xFF
                        outStream.WriteByte(_data[i]);
                        sendLen++;
                    }

                    IMRResult r = IMRResult.Ok;

                    if (sendLen > 0)
                        r = _imrClass.SOLSendText(_clientId, outStream.GetBuffer(), sendLen);

                    if (r != IMRResult.Ok)
                    {
                        _lastResult = r;
                        _shutdownEvent.Set();
                    }
                }

            }
            catch (Exception e)
            {
                _readExp = e;
                _shutdownEvent.Set();
            }
            finally
            {
                _readEvent.Set();
            }
               
        }


        private void SendCallback(IAsyncResult result)
        {
            try
            {
                MemoryStream stream = (MemoryStream)result.AsyncState;


                _rSocket.EndSend(result);
                stream.Close();

            }
            finally
            {
                _writeEvent.Set();
            }
        }

        /// <summary>
        /// The port the terminal link will listen on
        /// </summary>
        public int Port
        {
            get
            {
                return _port;
            }
            set
            {
                _port = value;
            }
        }


        /// <summary>
        /// The message size the terminal link will expect
        /// </summary>
        public static int MessageSize
        {
            get
            {
                return _messageSize;
            }
            set
            {
                _messageSize = value;
            }
        }

        #region IDisposable Members

        private bool _disposed = false;

        protected virtual void Dispose(bool disposing)
        {
            if (_disposed)
                return;
            if (disposing)
            {
                if (_readyEvent != null)
                    _readyEvent.Close();
                if (_connectedEvent != null)
                    _connectedEvent.Close();
                if (_readEvent != null)
                    _readEvent.Close();
                if (_writeEvent != null)
                    _writeEvent.Close();
                if (_shutdownEvent != null)
                    _shutdownEvent.Close();
                if(_socket != null)
                    _socket.Close();
                if (_rSocket != null)
                    _rSocket.Close();
            }
            _disposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
